#!/usr/bin/env lua
--[[
	说话服务
--]]
local coaf=require "coaf.core"
local _dbg = require "coaf.debug"
local thread = require "coaf.thread"

coaf.ChDir(coaf.GetAppDir(arg))

local socket = require "cosocket.core"
socket.default_close_on_exec = true

local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"
local event = require "coaf.event"

-- 加载 talk 模块
local mo_TalkBack = require "mod_TalkBack"

---- 日志模块
local withLog = false
for _x=1, #arg do
	if arg[_x] == "--log" then
		withLog = true
		break
	end
end

if withLog then
	-- ** initialize the log file **
	local roll = require "coaf.log.rollfile"
	os.execute( "/bin/mkdir -p /var/log/" )
	local logF = roll.Create("/var/log/talkservice.log")
	if not logF or not logF:init() then
		_dbg.Error("TALK_SERVICE", "Fail to open log file: /var/log/talkservice.log, so the log messages should be output to console.")
		withLog = false
	else
		_dbg.AddLogger("logfile", logF, nil, nil)
	end
	-- ** log **
end

coaf.Init( "talkService" )

if withLog then
	local stdio = require "coaf.log.stdio"
	stdio.SetDefaultFilterName( "TALKSERVER" )
	stdio.Start()
end

service.InitSocket()
client.InitSocket( socket, adapter )

_G.g_event = event.createNew( "talk" )

local cosvc_Talk
local function Main(args)
	cosvc_Talk = service.New( "talkService", "unix:/var/run/talk.sock" )
	if cosvc_Talk then
		_dbg.Info( "TALK", "CoService of 'talk' is created." )
		cosvc_Talk:CreateCoServerProxy()
	else
		_dbg.Error( "TALK", "Fail created CoService!" )
	end
	-- 添加对讲路由
	mo_TalkBack.Init()
	cosvc_Talk:ExposeObject("/talkback", mo_TalkBack )

end

coaf.Start(Main, arg)

event.free( "talk" )
_dbg.Print( "TALK", "Service terminated." )