#!/usr/bin/env lua

local coaf=require "coaf.core"
local _dbg = require "coaf.debug"
local thread = require "coaf.thread"

coaf.ChDir(coaf.GetAppDir(arg))

local socket = require "cosocket.core"
socket.default_close_on_exec = true

local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"

-- 加载 talk 模块
local mod_client = require "client"

---- 日志模块
local withLog = false
for _x=1, #arg do
	if arg[_x] == "--log" then
		withLog = true
		break
	end
end

if withLog then
	-- ** initialize the log file **
	local roll = require "coaf.log.rollfile"
	os.execute( "/bin/mkdir -p /var/log/" )
	local logF = roll.Create("/var/log/kilolinkClient.log")
	if not logF or not logF:init() then
		_dbg.Error("KILOLINK_CLIENT", "Fail to open log file: /var/log/kilolinkClient.log, so the log messages should be output to console.")
		withLog = false
	else
		_dbg.AddLogger("kilolinkClient", logF, nil, nil)
	end
	-- ** log **
end

coaf.Init( "kilolinkClientServices" )

if withLog then
	local stdio = require "coaf.log.stdio"
	stdio.SetDefaultFilterName( "kilolinkClient" )
	stdio.Start()
end

service.InitSocket()
client.InitSocket( socket, adapter )

local function init_ifname()
	os.execute("/bin/echo 1 > /proc/sys/net/ipv4/ip_forward")
	os.execute("/bin/echo 0 > /proc/sys/net/ipv4/conf/all/rp_filter")
	-- os.execute("/bin/echo 0 > /proc/sys/net/ipv4/conf/wwan0/rp_filter")
	  -- os.execute("/bin/echo 0 > /proc/sys/net/ipv4/conf/wwan1/rp_filter")
end

local cosvc_Talk
local function Main(args)
	cosvc_Talk = service.New( "kilolinkClient", "unix:/var/run/klnkClient.sock" )
	if cosvc_Talk then
		_dbg.Info( "KClient", "CoService of 'kilolinkClient' is created." )
		cosvc_Talk:CreateCoServerProxy()
	else
		_dbg.Error( "KClient", "Fail created CoService!" )
	end

	init_ifname()
	mod_client.ReInit()
	cosvc_Talk:ExposeObject("/client", mod_client )

	coaf.RemoveDeadSignals("*")
end

coaf.Start(Main, arg)

_dbg.Print( "TALK", "Service terminated." )