local CONFIG_FILE = "/data/configs/block_loop_out.conf"

require "configuredb"
require "block_loop_out_config"
require "features"

local getset = require "kiloview.getSetHelper"
local _dbg = require "coaf.debug"
local coaf = require "coaf.core"
local thread = require "coaf.thread"
local client = require "coaf.client"

local http=require("socket.http")
local ltn12 = require("ltn12")
local json = require("cjson")

local md5 = require("md5")

local frameTaskTimer = nil
local piclist = {}
local latesttime = 0

http.TIMEOUT = 1	-- 设置HTTP超时时间 防止因http造成的阻塞

---------------------------------------------------
_G.blo_config = CONFIGS.InstanceOf( BLO_Template, "blo_config" )
---------------------------------------------------

local _M = {
}

local BLO_SVC = {
	Identifier = "BLO",
	FriendlyName = "Encoder BLO service",
	Version = "1.0",
	ConfigPrefix = "blo",
}

---------------------------------------------------

local function getHTTPResponse(url,body)
	local pl = require "pl.pretty"
    local ret = {}
	--print(url)
	local res, code, headers = http.request{
		url = url,
		method = "POST",
		headers =
			{
				["Content-Type"] = "application/json",
				["Content-length"] = string.len(body)
			},
		source = ltn12.source.string(body),
		sink = ltn12.sink.table(ret)
	}

	if #ret > 0 then
		return json.decode(ret[1])
	end
	return nil
end

local function GetCoAFProxy(objPath, service, loc, timeout)
	local proxy
	local cli
	cli = client.New( service, loc or "local", nil, timeout or 3 ) 
	if not cli then
		_dbg.Error( "CODEC", "Fail to create connection to service: " .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true)
	if not proxy then
		_dbg.Error( "CODEC", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end

local function CheckHeart()
	BLO_SVC.ScheduleDelayedHeartTask()
end

local function CheckFrame()
	BLO_SVC.ScheduleDelayedFrameTask()
end

local function initHDMIHPD()
	local proxy = GetCoAFProxy("/AVIO/MODULE", "codec")
	if proxy then 
		proxy.UpdateInterruptStatus(2)
		proxy:Destroy()
	end
end

local function resetFrameTimer()
	BLO_SVC.snapPicFrameTask()
end

---------------------------------------------------

function BLO_SVC:Activate()
	--Nothing
end

function BLO_SVC:Start()
	--Nothing
end

function BLO_SVC:Stop()
	--Nothing
end

function BLO_SVC:Reset()
--TODO: Nothing
end

function BLO_SVC:Terminate()
--TODO: Nothing
end

function BLO_SVC:Control(action,params)
	if action == "activate" then
		self:Activate()
		return 0, {}
	elseif action == "start" then
		self:Start()
		return 0, {}
	elseif action == "stop" then
		self:Stop()
		return 0, {}
	elseif action == "Reset" then
		self:Reset()
		return 0, {}
	elseif action == "terminate" then
		self:Terminate()
		return 0, {}
	elseif action == "update" then
		return 0, {}
	else
		return -1, {}
	end
end

function BLO_SVC:Set(params)
	local cnt = getset.SetConfigureValues( self.ConfigPrefix, params )
	return cnt
end

function BLO_SVC:SetEnv(params)
	local cnt = getset.SetValues( "_ENV", params )
	return cnt
end

function BLO_SVC:Get(inParams)
	local cnt = getset.GetValues( self.ConfigPrefix, inParams )
	return cnt,inParams
end

function BLO_SVC:GetEnv(inParams)
	local cnt = getset.GetValues( "_ENV", inParams )
	return cnt,inParams
end

function BLO_SVC:RestoreDefault()
	--TODO:
	return 0
end

function BLO_SVC:LoadConfigureFile(filename)
	CONFIGS.BeginUpdate()
	local r = pcall( dofile, filename )
	CONFIGS.Submit()
	if r then
		return 1
	else
		return 0
	end
end

function BLO_SVC.getBindInfoSendInfo()
	return {
		["deviceId"] = blo_config["deviceId"],
		["deviceModel"] = blo_config["deviceModel"],
		["versionName"] = blo_config["versionName"],
		["location"] = blo_config["location"],
		["address"] = blo_config["address"],
		["contacter"] = blo_config["contacter"],
		["phone"] = blo_config["phone"],
		["pushUrl"] = "",--blo_config["pushUrl"],
		["decodeUrl"] = "",--blo_config["decodeUrl"],
	}
end

function BLO_SVC.getConfig()
	local res = BLO_SVC.getBindInfoSendInfo()
	res["ip"] = blo_config["ip"]
	res["port"] = blo_config["port"]
	res["deviceType"] = blo_config["deviceType"]
	res["picFrame"] = blo_config["picFrame"]
	return res
end

function BLO_SVC.setConfig(data)
	local i,v 
	if not data then
		return false
	end
	for i,v in pairs(data) do 
		if blo_config[i] then
			blo_config[i] = v
		end
	end
	blo_config:SaveFile()
	blo_config:ResetChanges()
	return true
end

function BLO_SVC.bindMonitoring()
	local url = "http://" .. blo_config["ip"] .. ":" .. blo_config["port"] .. "/hawkeye/advertisement/monitor/device/init"
	local data = json.encode(BLO_SVC.getBindInfoSendInfo())
	local ret = getHTTPResponse(url,data)
	if ret and ret["code"] and tonumber(ret["code"]) == 0 then
		BLO_SVC.resetSnapPicFrameTimer()
		return true
	else
		return false
	end
end

function BLO_SVC.ScheduleDelayedHeartTask()
	if not blo_config["status"] or not tonumber(blo_config["status"]) or not blo_config["deviceId"] or blo_config["deviceId"] == "" then
		return
	end
	local url = "http://" .. blo_config["ip"] .. ":" .. blo_config["port"] .. "/hawkeye/advertisement/monitor/device/heartbeat"
	local tmp = {
		deviceId = blo_config["deviceId"],
		status = blo_config["status"]
	}
	local data = json.encode(tmp)
	local ret = getHTTPResponse(url,data)
	if ret and ret["code"] and tonumber(ret["code"]) == 0 and ret["command_code"] and tonumber(ret["command_code"]) > 1 then
		local proxy = GetCoAFProxy("/AVIO/MODULE", "codec")
		if tonumber(ret["command_code"]) == 2 then
			--阻断
			if proxy then
				proxy.UpdateInterruptStatus(0)
				blo_config["status"] = 1
			end	
		elseif tonumber(ret["command_code"]) == 3 then
			--恢复
			if proxy then
				proxy.UpdateInterruptStatus(1)
				blo_config["status"] = 0
			end	
		end
		if proxy then
			proxy:Destroy()
		end
		blo_config:SaveFile()
		blo_config:ResetChanges()
	end
end

function BLO_SVC.ScheduleDelayedFrameTask()
	if not blo_config["deviceType"] or not tonumber(blo_config["deviceType"]) or not blo_config["deviceId"] or blo_config["deviceId"] == "" then
		return
	end

	if piclist and #piclist > 50 then
		piclist = nil
	elseif piclist and #piclist > 1 then
		table.sort(piclist,function(a,b)
			return a["fileName"] < b["fileName"]
		end)
		local i,v 
		for i,v in pairs(piclist) do
			if tonumber(v["fileName"]) > latesttime then
				latesttime = tonumber(v["fileName"])
			else
				v = nil
			end
		end
	elseif piclist and #piclist > 0 then
		if tonumber(piclist[1]["fileName"]) > latesttime then
			latesttime = tonumber(piclist[1]["fileName"]) 
		else
			piclist = nil
		end
	else
		piclist = nil
	end

	local url = "http://" .. blo_config["ip"] .. ":" .. blo_config["port"] .. "/hawkeye/advertisement/monitor/device/sendFile"

	local tmp = {
		["deviceId"] = blo_config["deviceId"],
		["deviceType"] = blo_config["deviceType"],
		["fileList"] = piclist,
	}

	local data = json.encode(tmp)
	local ret = getHTTPResponse(url,data)
	if ret and ret["code"] and tonumber(ret["code"]) == 0 then
		piclist = {}
	end
end

function BLO_SVC.resetSnapPicFrameTimer()
	if not blo_config["picFrame"] or tonumber(blo_config["picFrame"]) <= 0 then
		return
	end
	if frameTaskTimer then
		--TODO:将定时器清空
		frameTaskTimer:destory()
		frameTaskTimer = nil
	end
	local delay = 3
	if blo_config["picFrame"] >= 1 then
		delay = blo_config["picFrame"]
	elseif blo_config["picFrame"] > 0 then
		delay = 1 / tonumber(blo_config["picFrame"])
	end
	frameTaskTimer = thread.NewTimer( resetFrameTimer, 3, delay)
	if frameTaskTimer then
		frameTaskTimer:start()
	end
end

function BLO_SVC.snapPicFrameTask()
	local moduleProxy = GetCoAFProxy( "/DefaultApp/02", "streamer")
	if not moduleProxy then
		print( "*** Could not get streaming service module proxy of stream  02" )
		return
	end

	moduleProxy:Control( "snap", {
		["path"] = "/tmp",
		["file"] = "FRAME",
		["suffix"] = ".jpg",
		["count"] = 1
	} )
	moduleProxy:Destroy()

	local fh, err = io.open ("/tmp/FRAME.jpg", "rb")
	local contents,fileinfo
	if fh then
		contents = fh:read("*a")
		fh:close()
	end
	local f = io.popen("/bin/stat -c '%Y-%s' /tmp/FRAME.jpg")
	if f then
		fileinfo = f:read()
		f:close()
	end
	if fileinfo and contents then
		local _s,_e,time,size = string.find(fileinfo,"(%d+)-(%d+)")
		local base64code = ""
		-- local base64code = base64.base64encode(contents)
		f = io.popen("/bin/cat /tmp/FRAME.jpg | /bin/base64", "r")
		if f then
			local ln = f:read("*l")
			while ln do
				base64code = base64code .. ln
				ln = f:read("*l")
			end
			f:close()
		end
		local md5code = md5.sumhexa(contents)
		if time and size then
			piclist = piclist or {}
			local data = {
				["fileName"] = time,
				["fileSize"] = size,
				["fileMd5"] = md5code,
				["fileBase64"] = base64code
			}
			table.insert(piclist,data)
		end
	end
end
---------------------------------------------------

function _M.Init(  cosvc )
	if cosvc then
		cosvc:ExposeObject("/BLO", BLO_SVC)
	end

	blo_config:LoadFile( CONFIG_FILE )
	blo_config:ResetChanges()

	print( "BLO Service online." )

	if _G.FRATURE_BLOCK_LOOP_OUT then
		thread.NewTimer( initHDMIHPD, 1):start()
		thread.NewTimer( CheckHeart, 0.01, 5 ):start()
		thread.NewTimer( CheckFrame, 0.01, 5 ):start()
		BLO_SVC.resetSnapPicFrameTimer()
	end
end

return _M
