--The Encoder service
local fmtUtil = require "formatUtil"
local osd = require "osd"
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local client = require "coaf.client"

require "features"	--To detect product type.

local done,hwChk = pcall( require, "kiloview." .. _G.DEVICE_TYPE .. ".hardwareCheck" )

if done and type(hwChk) == "table" then
	hwChk.Check()
else
	hwChk = nil
end

local EXT_TOOLS = "/usr/local/bin/test_encode"

local function DeviceWithOnchipOutput()
	if _G.DEVICE_TYPE == "G2" and hwChk and hwChk.hardwareVersionCode < 0x17 then
		return true
	else
		return false
	end
end

local function RESTRICT_WIDTH( v, lessThan )
	v = (math.floor(v+15) & 0xfffffff0)
	if lessThan then
		lessThan = math.floor(lessThan) & 0xfffffff0
		if v > lessThan then
			return lessThan
		end
	end
	return v
end

local function RESTRICT_HEIGHT( v, lessThan )
	v = (math.floor(v+7) & 0xfffffff8)
	if lessThan then
		lessThan = math.floor(lessThan) & 0xfffffff8
		if v > lessThan then
			return lessThan
		end
	end
	return v
end

---------------------------------------------------
local UPDATE_STREAM_ENCMODE_PERIOD = 3

--Main buffer
local MAIN_ENC_BUFFER = "X"
local MAIN_ENC_BUFFER_NUM = 0
local MAIN_BUFFER_MAX_WIDTH = 1920
local MAIN_BUFFER_MAX_HEIGHT = 1080
--[[
-- Sub stream buffer
local SUB_ENC_BUFFER = "Y"
local SUB_ENC_BUFFER_NUM = 1
local SUB_BUFFER_MAX_WIDTH = 720
local SUB_BUFFER_MAX_HEIGHT = 576
]]

--[[
--Snapshot stream buffer
local SNAP_ENC_BUFFER = "K"
local SNAP_ENC_BUFFER_NUM = 3
local SNAP_BUFFER_MAX_WIDTH = 1280
local SNAP_BUFFER_MAX_HEIGHT = 720
]]

local SUB_ENC_BUFFER = "K"
local SUB_ENC_BUFFER_NUM = 3
local SUB_BUFFER_MAX_WIDTH = 1280
local SUB_BUFFER_MAX_HEIGHT = 720

local SNAP_ENC_BUFFER = "Y"
local SNAP_ENC_BUFFER_NUM = 1
local SNAP_BUFFER_MAX_WIDTH = 720
local SNAP_BUFFER_MAX_HEIGHT = 576

--Preview/loopback buffer
local PREV_ENC_BUFFER = "J"
local PREV_ENC_BUFFER_NUM = 2
local PREV_BUFFER_MAX_WIDTH = 1920
local PREV_BUFFER_MAX_HEIGHT = 1080

-------------------------------

local DEFAULT_SUB_WIDTH = 720
local DEFAULT_SUB_HEIGHT = 480

-- local DEFAULT_SNAP_WIDTH = 720
-- local DEFAULT_SNAP_HEIGHT = 576

local DEFAULT_SNAP_WIDTH = 640  --NOTE:为了适应缩放最低分辨率，不然编码init阶段会报错。
local DEFAULT_SNAP_HEIGHT = 480
local DEFAULT_PREVIEW_SNAP_RATE = 5	--5 fps for preview.
local DEFAULT_PREVIEW_SNAP_BITRATE = 4000000
local DEFAULT_PREVIEW_SNAP_QUALITY = 80

local STREAMER_BUS_NAME = "com.kiloview.SUNSHINE.StreamServer"
local STREAMER_OBJ_PATH = "/com/kiloview/SUNSHINE/StreamServer"
local STREAMSERVER_INTF = "com.kiloview.SUNSHINE.StreamServer.iServer"

local STREAM_DEFAPP_OBJ_PATH = "/com/kiloview/SUNSHINE/StreamServer/DefaultApp"
local STREAMAPP_INTF = "com.kiloview.SUNSHINE.StreamServer.iApplication"

local iStreamDefaultAppCfgProxy

local function GetStreamerProxy()
	if iStreamDefaultAppCfgProxy then
		return iStreamDefaultAppCfgProxy
	end

	local proxy
	local cli
	cli = client.New( "streamer", "local", nil, 2 ) --2 seconds timeout.
	if not cli then
		_dbg.Error( "CODEC", "Fail to create connection to service 'local/streamer'" )
		return nil
	end
	proxy = cli:CreateProxy("/DefaultApp", true) --Exclusive
	if not proxy then
		_dbg.Error( "CODEC", "Fail to create object proxy for 'local/streamer:/DefaultApp'" )
		return nil
	end
	iStreamDefaultAppCfgProxy = proxy
	return proxy
end

----------------------------------------------------
local dynamic_bitrate_set_cache = {}

local function UpdateStreamerDynamicBitrateSet( stream_id, bitrate, dyn_br )
	--NOTE: Must delay do this, since the caller may be a C routine (configdb). This is an old
	--      version problem.
	dynamic_bitrate_set_cache[ stream_id ] = { bitrate = bitrate, dyn_br = dyn_br }
end

----------------------------------------------------

local _M = {
	encoder_objs = {},
	firstRun = true,

	current_enc_mode = "",
	current_snap_type = "",
	
	current_video_input = {
		width = 0,
		height = 0,
		signal_locked = 0,
		freerun = 0,
		interlaced = 0,
		frame_rate = 0,
		format_name = "",
		hdcp = 0,
		crop = false,
		crop_width = 0,
		crop_height = 0,
		crop_x = 0,
		crop_y = 0,
		rmf = "0",
	},

	current_audio_input = {
		sampling = 0,
		channels = 0,
		sample_size = 0,
		sample_width = 0,
	},

	bitrate_rec = {
		["A"] = {},
		["B"] = {},
		["C"] = {},
		["D"] = {}
	}
}

---------------------------------------------------
local MAX_BR = 52420000

local function AdjustBitrate( stream, setBr )
	if setBr >= MAX_BR then
		setBr = MAX_BR
	end

	local kkk, vvv
	local totalBr = setBr
	for kkk, vvv in pairs( _M.bitrate_rec ) do
		if kkk ~= stream and type(vvv) == "table" and vvv.started then
			totalBr = totalBr + (vvv.bitrate or 0)
		end
	end

	if totalBr >= MAX_BR then
		for kkk, vvv in pairs( _M.bitrate_rec ) do
			if kkk ~= stream and type(vvv) == "table" and vvv.started then
				local bitrate = math.floor( MAX_BR * (vvv.bitrate/totalBr) )
				print( "**Adjust bitrate:", kkk, bitrate )
				os.execute( EXT_TOOLS .. " -" .. kkk .. " --bitrate " .. bitrate )
			end
		end
		setBr = math.floor( MAX_BR * (setBr/totalBr) )
	else
		for kkk, vvv in pairs( _M.bitrate_rec ) do
			if kkk ~= stream and type(vvv) == "table" and vvv.started then
				local bitrate = vvv.bitrate
				print( "**Adjust bitrate:", kkk, bitrate )
				os.execute( EXT_TOOLS .. " -" .. kkk .. " --bitrate " .. bitrate )
			end
		end
	end
	return setBr
end

local function ConstructEncoder( idr )

	local ENC_SVC = {

		Identifier = idr,
		FriendlyName = "Encoder-" .. idr .. " service",
		idr = idr,
		vCodec = "",
		aCodec = "",
		status = "idle",

		holdConfigs = {
			enabled = 0,
			video = {
				scale = { fly = false, value = nil },
				scale_width = { fly = false, value = nil },
				scale_height = { fly = false, value = nil },
				codec = { fly = false, value = nil },
				br_ctrl = { fly = true, value = nil },	--??
				bitrate = { fly = true, value = nil },
				dynamic_bitrate = { fly = true, value = nil },
				reduce_framerate = { fly = true, value = nil },
				profile = { fly = false, value = nil },
				avclevel = { fly = true, value = nil }, --Since this is no used.
				gop_size = { fly = true, value = nil },
				gop = { fly = false, value = nil },
				framerate_mode = { fly = true, value = nil }, --??
				framerate = { fly = true, value = nil },
				quality = { fly = true, value = nil },
				ref_frames = { fly = false, value = nil },
				qp_max = { fly = true, value = nil},
				qp_min = { fly = true, value = nil},
				chrome = { fly = true, value = nil }
			},

			audio = {
				codec = { fly = false, value = nil },
				br_ctrl = { fly = true, value = nil }, --??
				bitrate = { fly = true, value = nil }
			}
		},

		informations = {
			enabled = false,
			video = {
				width = 0,
				height = 0,
				codec = "H264",
				br_ctrl = "cbr",
				bitrate = 0,
				suggestBitrate = nil,
				profile = "high",
				avclevel = 40,
				framerate = 0,
				quality = 0,
				ref_frames = 0,
				chrome = false,
				gop = "IP",
				gop_size = 0,
			},

			audio = {
				codec = "NONE",
				br_ctrl = "cbr",
				bitrate = 0
			}
		}
	}

	if idr == "00" then	--Main stream
		ENC_SVC.cfgId = 0
		ENC_SVC.encode_Buffer = MAIN_ENC_BUFFER
		ENC_SVC.encode_BufferNum = MAIN_ENC_BUFFER_NUM
		ENC_SVC.encode_Stream = "A"
		ENC_SVC.max_width = MAIN_BUFFER_MAX_WIDTH
		ENC_SVC.max_height = MAIN_BUFFER_MAX_HEIGHT
	elseif idr == "01" then --Secondary stream
		ENC_SVC.cfgId = 1
		ENC_SVC.encode_Buffer = SUB_ENC_BUFFER
		ENC_SVC.encode_BufferNum = SUB_ENC_BUFFER_NUM
		ENC_SVC.encode_Stream = "B"
		ENC_SVC.max_width = SUB_BUFFER_MAX_WIDTH
		ENC_SVC.max_height = SUB_BUFFER_MAX_HEIGHT

	elseif idr == "02" then --Snap stream
		ENC_SVC.cfgId = 2
		ENC_SVC.encode_Buffer = SNAP_ENC_BUFFER
		ENC_SVC.encode_BufferNum = SNAP_ENC_BUFFER_NUM
		ENC_SVC.encode_Stream = "C"
		ENC_SVC.max_width = SNAP_BUFFER_MAX_WIDTH
		ENC_SVC.max_height = SNAP_BUFFER_MAX_HEIGHT
	else
		return nil
	end

	function ENC_SVC:Activate()
	end

	function ENC_SVC:Start()
		if self.status == "stopped" or self.status == "idle" or self.status == "error" or self.status == "disabled" then
			self.status = "starting"	--Mark starting.
		end
	end

	function ENC_SVC:_UpdateConfig(avFlag, onlyUpdate)
		local cfgs = _G.encoder.stream[self.cfgId]
		if not cfgs then
			print( "ERROR: Invalid configure for stream '" .. self.idr .. "', so could not update it!" )
			self.status = "error"
			return
		end

		local require_reset = false
		local require_reset_input = false
		local require_video_fly = false
		local require_audio_fly = false

		if cfgs.enabled ~= self.holdConfigs.enabled then
			self.holdConfigs.enabled = cfgs.enabled
			require_reset = true
		end

		local k,v
		if not avFlag or avFlag == "all" or avFlag == "*" or avFlag == "video" then
			for k,v in pairs(self.holdConfigs.video) do
				if cfgs.video[k] ~= v.value then
					v.value = cfgs.video[k]
					if not v.fly then
						require_reset = true
						if k == "scale" or k == "scale_width" or k == "scale_height" then
							require_reset_input = true
						end
					else
						require_video_fly = true
					end
				end
			end
		end

		if not avFlag or avFlag == "all" or avFlag == "*" or avFlag == "audio" then
			for k,v in pairs(self.holdConfigs.audio) do
				if cfgs.audio[k] ~= v.value then
					v.value = cfgs.audio[k]
					if not v.fly then
						require_reset = true
					else
						require_audio_fly = true
					end
				end
			end
		end
		
		UpdateStreamerDynamicBitrateSet( self.cfgId, cfgs.video.bitrate, cfgs.video.dynamic_bitrate )

		if onlyUpdate then return end

		if require_reset then
			print( "No-flying changable parameters changed, so reset the encoder of channel '" .. self.idr .. "'." )
			self:Stop()
			self:Start()	--Because of start, all informations are updated.

			if require_reset_input then	--Reset the input (so change the buffer size for encoding)
				_M.firstRun = true
			end

			return
		end

		--Else fly parameters.
		if require_video_fly then
			print( "Change encoder flying parameters of channel '" .. self.idr .. "'." )

			local cfg = cfgs.video	--Video configure.

			local isMjpeg = false
			local cmd = " -" .. self.encode_Stream
			if self.idr == "02" or cfg.codec == "MJPEG" or cfg.codec == "mjpeg"  then
				isMjpeg = true
			end
			self.informations.video.codec = cfg.codec

			--Level ignore.
			if not isMjpeg then

				if cfg.gop_size <= 0 or cfg.gop_size >4095 then
					if _M.current_video_input.frame_rate > 0 then
						cmd = cmd .. " -N " .. math.floor(_M.current_video_input.frame_rate)
						self.informations.video.gop_size = math.floor(_M.current_video_input.frame_rate)
					else
						cmd = cmd .. " -N 60"
						self.informations.video.gop_size = 60
					end
				else
					cmd = cmd .. " -N " .. cfg.gop_size
					self.informations.video.gop_size = cfg.gop_size
				end

				cmd = cmd .. " --idr 1" --TODO
			end

			if cfg.chrome > 0 then
				cmd = cmd .. " --chrome 1"
				self.informations.video.chrome = false
			else
				cmd = cmd .. " --chrome 0"
				self.informations.video.chrome = false
			end

			local bitrateScale = 1
			if isMjpeg and _G.encoder.snap_type == "preview" then	--For preview snap case, reduce the frame-rate
				local frate = (_M.current_video_input.frame_rate > 0) and _M.current_video_input.frame_rate or cfg.framerate
				local actual_frate = DEFAULT_PREVIEW_SNAP_RATE
				if actual_frate > frate then
					actual_frate = frate
				end
				cmd = cmd .. " --frame-factor " .. math.floor(actual_frate) .. "/" .. math.floor(frate)
				self.informations.video.framerate = actual_frate
				bitrateScale = actual_frate / frate

				self.informations.video.br_ctrl = "cbr"
				self.informations.video.bitrate = DEFAULT_PREVIEW_SNAP_BITRATE
				cmd = cmd .. " --bc cbr --bitrate " .. math.floor( DEFAULT_PREVIEW_SNAP_BITRATE/bitrateScale )
			
				self.informations.video.quality = DEFAULT_PREVIEW_SNAP_QUALITY
				cmd = cmd .. " -q " .. cfg.quality
			else			
				if cfg.framerate_mode == "custom" then
					local frate = (_M.current_video_input.frame_rate > 0) and _M.current_video_input.frame_rate or cfg.framerate
					--@@BUG FIX: If user specified frame-rate large than actual input, it should be error.
					local actual_frate = cfg.framerate
					if actual_frate > frate then
						actual_frate = frate
					end
				
					cmd = cmd .. " --frame-factor " .. math.floor(actual_frate) .. "/" .. math.floor(frate)
					self.informations.video.framerate = actual_frate
					bitrateScale = actual_frate / frate
				elseif cfg.framerate_mode == "half" then
					local frate = (_M.current_video_input.frame_rate > 0) and _M.current_video_input.frame_rate or 60
					if frate >= 50 then
						cmd = cmd .. " --frame-factor 1/2" --NOTE: Must consider the case of framerate be 59.94
						self.informations.video.framerate = frate/2
						bitrateScale = 0.5
					else
						cmd = cmd .. " --frame-factor 1/1"
						self.informations.video.framerate = frate
						bitrateScale = 1
					end
				else
					local frate = (_M.current_video_input.frame_rate > 0) and _M.current_video_input.frame_rate or 60
					--if self.idr == "00" or frate < 50 then --main stream or framerate < 50
						cmd = cmd .. " --frame-factor 1/1"
						self.informations.video.framerate = frate
						bitrateScale = 1
					--else --sub stream/snap stream, only half if >= 50
					--	cmd = cmd .. " --frame-factor " .. math.floor(frate/2) .. "/" .. math.floor(frate)
					--	self.informations.video.framerate = math.floor(frate/2)
					--	bitrateScale = 0.5
					--end
				end
		
				self.informations.video.bitrateScale = bitrateScale

				local setBr = math.floor(cfg.bitrate/bitrateScale)
				_M.bitrate_rec[ self.encode_Stream ].bitrate = setBr
				setBr = AdjustBitrate( self.encode_Stream, setBr )

				_M.bitrate_rec[ self.encode_Stream ].started = true

				if cfg.br_ctrl == "cbr" then
					cmd = cmd .. " --bc cbr --bitrate " .. math.floor(setBr) .. " --qp-limit-i " .. cfg.qp_min .. "~" .. cfg.qp_max .. " --qp-limit-p " .. cfg.qp_min .. "~" .. cfg.qp_max .. " --qp-limit-b " .. cfg.qp_min .. "~" .. cfg.qp_max
					self.informations.video.br_ctrl = "cbr"
					self.informations.video.bitrate = cfg.bitrate
				else
					cmd = cmd .. " --bc vbr --bitrate " .. math.floor(setBr)  .. 
						  " --vbr-bitrate " .. math.min(math.floor((setBr) * 0.2),8000000) .. "~" .. math.min(math.floor(setBr*1.0),45000000) .. 
						  " --qp-limit-i 6~51 --qp-limit-p 6~51 --qp-limit-b 6~51"

					self.informations.video.br_ctrl = "vbr"
					self.informations.video.bitrate = cfg.bitrate
				end

				if cfg.reduce_framerate and cfg.reduce_framerate ~= 0 then
					cmd = cmd .. " --skip-frame-mode 2"
				else
					cmd = cmd .. " --skip-frame-mode 0"
				end

				if isMjpeg then
					cmd = cmd .. " -q " .. cfg.quality
				end
				self.informations.video.quality = cfg.quality
			end

			print( "**Update:", cmd )
			os.execute( EXT_TOOLS .. cmd )

			UpdateStreamerDynamicBitrateSet( self.cfgId, cfg.bitrate, cfg.dynamic_bitrate )
			self:UpdateDynamicBitrate()
		end
	end

	--Actual start.
	function ENC_SVC:_ActualStart()
		local cfgs = _G.encoder.stream[self.cfgId]
		if not cfgs then
			print( "ERROR: Invalid configure for stream '" .. self.idr .. "', so could not start it!" )
			self.status = "error"
			return
		end

		self.informations.enabled = (cfgs.enabled ~= 0)

		if cfgs.enabled == 0 then
			self:Stop()
			self.status = "disabled"
			return
		end

		local cfg = cfgs.video	--Video configure.

		local isMjpeg = false
		local cmd = " -" .. self.encode_Stream .. " -b " .. self.encode_BufferNum

		self.informations.video.codec = cfg.codec

		if self.idr == "02" or cfg.codec == "MJPEG" or cfg.codec == "mjpeg" then
			cmd = cmd .. " -m"
			isMjpeg = true
		else
			cmd = cmd .. " -h"
		end
		cmd = cmd .. " " .. self.informations.video.width .. "x" .. self.informations.video.height
		
		local advanceEnc = false

		if not isMjpeg then
			if cfg.profile == "base" or cfg.profile == "baseline" then
				cmd = cmd .. " --profile 0"
				self.informations.video.profile = "baseline"
			elseif cfg.profile == "main" then
				cmd = cmd .. " --profile 1"
				advanceEnc = true
				self.informations.video.profile = "main"
			else
				cmd = cmd .. " --profile 2"
				advanceEnc = true
				self.informations.video.profile = "high"
			end
		end

		local maxGopM = 0

		--Level ignore.
		if not isMjpeg then
			if not advanceEnc or cfg.gop == "IP" or cfg.gop == "ip" then
				cmd = cmd .. " -M 1"
				self.informations.video.profile = "IP"
			elseif cfg.gop == "IBP" or cfg.gop == "IPB" or cfg.gop == "ibp" or cfg.gop == "ipb" then
				cmd = cmd .. " -M 2"
				maxGopM = 1
				self.informations.video.profile = "IBP"
			elseif cfg.gop == "IBBP" or cfg.gop == "ibbp" then
				cmd = cmd .. " -M 3"
				maxGopM = 2
				self.informations.video.profile = "IBBP"
			else
				cmd = cmd .. " -M 1"
				self.informations.video.profile = "IP"
			end

			if cfg.gop_size <= 0 or cfg.gop_size >4095 then
				if _M.current_video_input.frame_rate > 0 then
					cmd = cmd .. " -N " .. math.floor(_M.current_video_input.frame_rate)
					self.informations.video.gop_size = math.floor(_M.current_video_input.frame_rate)
				else
					cmd = cmd .. " -N 60"
					self.informations.video.gop_size = 60
				end
			else
				cmd = cmd .. " -N " .. cfg.gop_size
				self.informations.video.gop_size = cfg.gop_size
			end

			cmd = cmd .. " --idr 1" --TODO
		end
		
		if cfg.ref_frames > 1 and _M.current_video_input.rmf ~= "90" and _M.current_video_input.rmf ~= "270" and maxGopM == 0 and advanceEnc then
			cmd = cmd .. " --multi-ref-p 1 --long-ref-enable 1 --gop 8 --max-gop-M 2"
		elseif maxGopM > 0 then
			--cmd = cmd .. " --multi-ref-p 0 --long-ref-enable 0 --gop 8 --max-gop-M " .. maxGopM
			cmd = cmd .. " --gop 8 --max-gop-M " .. maxGopM
		elseif advanceEnc then
			cmd = cmd .. " --gop 0"
		else
			cmd = cmd .. " --gop 0"
		end

		if cfg.chrome > 0 then
			cmd = cmd .. " --chrome 1"
			self.informations.video.chrome = true
		else
			cmd = cmd .. " --chrome 0"
			self.informations.video.chrome = false
		end

		local bitrateScale = 1

		if isMjpeg and _G.encoder.snap_type == "preview" then	--For preview snap case, reduce the frame-rate
			local frate = (_M.current_video_input.frame_rate > 0) and _M.current_video_input.frame_rate or cfg.framerate
			local actual_frate = DEFAULT_PREVIEW_SNAP_RATE
			if actual_frate > frate then
				actual_frate = frate
			end
			cmd = cmd .. " --frame-factor " .. math.floor(actual_frate) .. "/" .. math.floor(frate)
			self.informations.video.framerate = actual_frate
			bitrateScale = actual_frate / frate
			
			self.informations.video.br_ctrl = "cbr"
			self.informations.video.bitrate = DEFAULT_PREVIEW_SNAP_BITRATE
			cmd = cmd .. " --bc cbr --bitrate " .. math.floor( DEFAULT_PREVIEW_SNAP_BITRATE/bitrateScale )
			
			self.informations.video.quality = DEFAULT_PREVIEW_SNAP_QUALITY
			cmd = cmd .. " -q " .. cfg.quality
		else
			if cfg.framerate_mode == "custom" then
				local frate = (_M.current_video_input.frame_rate > 0) and _M.current_video_input.frame_rate or cfg.framerate
				--@@BUG FIX: If user specified frame-rate large than actual input, it should be error.
				local actual_frate = cfg.framerate
				if actual_frate > frate then
					actual_frate = frate
				end

				cmd = cmd .. " --frame-factor " .. math.floor(actual_frate) .. "/" .. math.floor(frate)
				self.informations.video.framerate = actual_frate
				bitrateScale = actual_frate / frate
			elseif cfg.framerate_mode == "half" then
				local frate = (_M.current_video_input.frame_rate > 0) and _M.current_video_input.frame_rate or 60
				if frate >= 50 then
					--cmd = cmd .. " --frame-factor " .. math.floor(frate/2) .. "/" .. math.floor(frate)
					cmd = cmd .. " --frame-factor 1/2" --NOTE: Must consider the case of framerate be 59.94
					self.informations.video.framerate = frate/2
					bitrateScale = 0.5
				else
					cmd = cmd .. " --frame-factor 1/1"
					self.informations.video.framerate = frate
					bitrateScale = 1
				end
			else
				local frate = (_M.current_video_input.frame_rate > 0) and _M.current_video_input.frame_rate or 60
				--if self.idr == "00" or frate < 50 then --main stream or framerate < 50
					cmd = cmd .. " --frame-factor 1/1"
					self.informations.video.framerate = frate
					bitrateScale = 1
				--else --sub stream/snap stream, only half if >= 50
				--	cmd = cmd .. " --frame-factor " .. math.floor(frate/2) .. "/" .. math.floor(frate)
				--	self.informations.video.framerate = math.floor(frate/2)
				--	bitrateScale = 0.5
				--end
			end
		
			self.informations.video.bitrateScale = bitrateScale

			local setBr = math.floor(cfg.bitrate/bitrateScale)
			_M.bitrate_rec[ self.encode_Stream ].bitrate = setBr
			setBr = AdjustBitrate( self.encode_Stream, setBr )

			if cfg.br_ctrl == "cbr" then
				self.informations.video.br_ctrl = "cbr"
				self.informations.video.bitrate = cfg.bitrate
				cmd = cmd .. " --bc cbr --bitrate " .. math.floor(setBr)
			else
				self.informations.video.br_ctrl = "vbr"
				self.informations.video.bitrate = cfg.bitrate
				cmd = cmd .. " --bc vbr --bitrate " .. math.floor(setBr)  .. " --vbr-bitrate " .. math.min(math.floor((setBr) * 0.2),8000000) .. "~" .. math.min(math.floor((setBr)*1.0),45000000)
			end

			if cfg.reduce_framerate and cfg.reduce_framerate ~= 0 then
				cmd = cmd .. " --skip-frame-mode 2"
			else
				cmd = cmd .. " --skip-frame-mode 0"
			end

			if isMjpeg then
				cmd = cmd .. " -q " .. cfg.quality
			end
			self.informations.video.quality = cfg.quality
		end
		
		local rmf = string.lower( _M.current_video_input.rmf or "0" )
		
		if rmf == "90" then
			cmd = cmd .. " --hflip 0 --vflip 0 --rotate 1"
		elseif rmf == "180" then
			cmd = cmd .. " --hflip 1 --vflip 1 --rotate 0"
		elseif rmf == "270" then
			cmd = cmd .. " --hflip 1 --vflip 1 --rotate 1"
		elseif rmf == "mirror" then
			cmd = cmd .. " --hflip 1 --vflip 0 --rotate 0"
		elseif rmf == "flip" then
			cmd = cmd .. " --hflip 0 --vflip 1 --rotate 0"
		else
			cmd = cmd .. " --hflip 0 --vflip 0 --rotate 0"
		end
		
		if not isMjpeg then
			if cfg.br_ctrl == "cbr" then
				cmd = cmd .. " --i-qp-reduce 2 --qp-limit-i " .. cfg.qp_min .. "~" .. cfg.qp_max .. " --qp-limit-p " .. cfg.qp_min .. "~" .. cfg.qp_max .. " --qp-limit-b " .. cfg.qp_min .. "~" .. cfg.qp_max
			else
				cmd = cmd .. " --i-qp-reduce 2 --qp-limit-i 6~51 --qp-limit-p 6~51 --qp-limit-b 6~51"
			end
		end

		print( "**Start:", cmd .. " -e" )
		local r,tag,code = os.execute( EXT_TOOLS .. cmd .. " -e" )	--Start encoding.
		if r and tag == "exit" then
			if code == 0 then
				self.status = "started"
				if not isMjpeg then
					_M.bitrate_rec[ self.encode_Stream ].started = true
				end

				UpdateStreamerDynamicBitrateSet( self.cfgId, cfg.bitrate, cfg.dynamic_bitrate )
				self:UpdateDynamicBitrate()
				osd.UpdateOSD()
			else
				if self.status == "retry-1" then
					self.status = "retry-2"
				elseif self.status == "retry-2" then
					self.status = "retry-3"
				elseif self.status == "retry-3" then
					self.status = "stall"
				else
					self.status = "retry-1"
				end
				--self.status = "error"
			end
		else
			if self.status == "retry-1" then
				self.status = "retry-2"
			elseif self.status == "retry-2" then
				self.status = "retry-3"
			elseif self.status == "retry-3" then
				self.status = "stall"
			else
				self.status = "retry-1"
			end
			--self.status = "error"
		end
	end

	function ENC_SVC:Stop()
		if self.status == "started" or self.status == "idle" or self.status == "stall" then
			if self.status ~= "idle" then	--Keep the idle status because init.
				self.status = "stopping"
			end
			local cmd = " -" .. self.encode_Stream .. " -s"
			print( "** Stop: ", cmd )
			os.execute( EXT_TOOLS .. cmd )
			self.informations.video.suggestBitrate = nil
			_M.bitrate_rec[ self.encode_Stream ].started = false
			--NOTE: Changed to GOP setting back
			--[[
			cmd = " -" .. self.encode_Stream .. " --multi-ref-p 0 --long-ref-enable 0 --gop 0 --max-gop-M 1"
			print( "** Reset: ", cmd )
			os.execute( EXT_TOOLS .. cmd )
			]]
			if self.status ~= "idle" then
				self.status = "stopped"
			end
		end
	end

	function ENC_SVC:Reset()
	--TODO:
		self:Stop()
		self:Start()
	end

	function ENC_SVC:Terminate()
		self:Stop()
	end

	function ENC_SVC:UpdateConfigure(target)
		self:_UpdateConfig(target)
	end

	function ENC_SVC:UpdateDynamicBitrate(targetBr)
		local cfg = _G.encoder.stream[self.cfgId].video

		targetBr = tonumber(targetBr)
		if targetBr then
			targetBr = math.ceil(targetBr/4000)*4000
			if targetBr == self.informations.video.suggestBitrate then
				return
			end
			self.informations.video.suggestBitrate = targetBr
		end

		if self.status == "started" or self.status == "stall" then --??? Stall.
			if cfg.dynamic_bitrate and cfg.dynamic_bitrate ~= 0 then
				local br = self.informations.video.suggestBitrate
				if not br then
					br = self.informations.video.bitrate
				elseif br <= 0 then
					br = 64000
				end

				local setBr = math.floor(br/(self.informations.video.bitrateScale or 1))
				_M.bitrate_rec[ self.encode_Stream ].bitrate = setBr
				setBr = AdjustBitrate( self.encode_Stream, setBr )

				--Test only:
				--print( "*** Dynamic adjust bitrate to:", setBr )
				os.execute( EXT_TOOLS .. " -" .. self.encode_Stream .. " --bitrate " .. setBr )
			end
		end
	end

	function ENC_SVC:Enable(target,enable)
		--TODO:
		return true
	end

	function ENC_SVC:UpdateEncoding(target)
		if self.status == "started" or self.status == "stall" then --??? Stall.
			print( "***Updated encoding of '" .. self.encode_Stream .. "'" )
			os.execute( EXT_TOOLS .. " -" .. self.encode_Stream .. " --force-idr" )
		end

		return true
	end

	function ENC_SVC:GetH26xSPS()
		--TODO:
		return 0, {}
	end

	function ENC_SVC:GetH26xPPS()
		--TODO:
		return 0, {}
	end

	function ENC_SVC:GetVideoProperties()
		local result = {}
		result["video.codec"] = self.informations.video.codec
		result["video.framerate"] = self.informations.video.framerate
		result["video.width"] = self.informations.video.width
		result["video.height"] = self.informations.video.height
		result["video.progressive"] = true --TODO: always be progressive
		result["video.bitrate"] = self.informations.video.bitrate
		return 6, result
	end

	function ENC_SVC:GetVideoEncStatus()
		local result = {}
		result["video.encoding.status"] = (self.status == "started" and "running" or self.status)
		result["video.encoding.frames"] = 0 --TODO:
		result["video.encoding.bytes"] = 0 --TODO:
		result["video.encoding.framerate"] = self.informations.video.framerate
		result["video.encoding.bitrate"] = self.informations.video.bitrate
		result["video.encoding.errors"] = 0 --TODO:
		result["video.capture.errors"] = 0 --TODO:
		result["video.capture.framerate"] = _M.current_video_input.frame_rate

		return 7, results
	end
	
	function ENC_SVC:GetAudioProperties()
		local result = {}
		--TODO:
		result["audio.codec"] = "NONE"
		result["audio.channels"] = 2
		result["audio.sample"] = 48000
		result["audio.sampleSize"] = 1024
		result["audio.sampleWidth"] = 16
		result["audio.bitrate"] = 64000

		return 6, result
	end

	function ENC_SVC:GetAudioConfigs()	--TODO:
		return 2, {0x11, 0x90}
	end

	function ENC_SVC:GetAudioEncStatus()
		local result = {}
		--TODO:
		result["audio.encoding.status"] = "idle"
		result["audio.encoding.frames"] = 0
		result["audio.encoding.bytes"] = 0
		result["audio.encoding.framerate"] = 0
		result["audio.encoding.bitrate"] = 0
		result["audio.encoding.errors"] = 0
		result["audio.capture.framerate"] = 0
		result["audio.capture.errors"] = 0
		result["audio.capture.overflows"] = 0
		result["audio.capture.underflows"] = 0
		return 10, result
	end

	function ENC_SVC:Control(action,params)
		if action == "activate" then
			self:Activate()
			return 0, {}
		elseif action == "start" then
			self:Start()
			return 0, {}
		elseif action == "stop" then
			self:Stop()
			return 0, {}
		elseif action == "Reset" then
			self:Reset()
			return 0, {}
		elseif action == "terminate" then
			self:Terminate()
			return 0, {}
		elseif action == "update-dynamic-bitrate" then
			self:UpdateDynamicBitrate(params and params["bitrate"])
		elseif action == "update-config" then
			if self:UpdateConfigure( params and params["target"] or "all" ) then
				return 0, {}
			else
				return -1, {}
			end
		elseif action == "enable" then
			if self:Enable( params and params["target"] or "all", params and params["enable"] or true ) then
				return 0, {}
			else
				return -1, {}
			end
		elseif action == "update-encoding" then
			if self:UpdateEncoding( params and params["target"] or "all" ) then
				return 0, {}
			else
				return -1, {}
			end
		elseif action == "get-video-props" then
			return self:GetVideoProperties()
		elseif action == "get-video-status" then
			return self:GetVideoEncStatus()
		elseif action == "get-audio-props" then
			return self:GetAudioProperties()
		elseif action == "get-audio-status" then
			return self:GetAudioEncStatus()
		elseif action == "get-audio-configs" then
			return self:GetAudioConfigs()
		else
			return -1, {}
		end
	end

	return ENC_SVC
end

----------------------------------------------------

function _M.Init( cosvc )
	local encoderSvcs = {
		[1] = { idr = "00" },
		[2] = { idr = "01" },
		[3] = { idr = "02" },
	}

	if not _G.encoder then
		print( "WARNING: Invalid 'encoder' configure when constructing encoder objects." )
	end

	local k,v 
	for k,v in pairs(encoderSvcs) do
		
		local svc_obj = ConstructEncoder( v.idr )
		if not svc_obj then
			print( "WARNING: Could not create encoder service instance of '" .. v.idr .. "', skip it and so it could not be online." )
		else
			_M.encoder_objs[v.idr] = svc_obj

			if cosvc then
				cosvc:ExposeObject("/Codec/Encoders/" .. v.idr, svc_obj)
			end
		end
	end

	print( "Each encoder service online." )
end

---- Each second checking

local encModeUpdateCounter = 0

----------------------------------
local function CalculateImageSize(stream_name, cfg, max_width, max_height, default_width, default_height)
	local act_w, act_h, aspect_ratio
	if not _M.current_video_input.crop then
		act_w = _M.current_video_input.width
		act_h = _M.current_video_input.height
	else
		act_w = _M.current_video_input.crop_width
		act_h = _M.current_video_input.crop_height
	end
	if act_w < 16 then act_w = 16 end
	if act_h < 16 then act_h = 16 end
	aspect_ratio = act_w/act_h

	local w, h
	if cfg.scale == "default" then
		if default_width and default_width <= act_w then
			w = default_width
			h = math.floor(w/aspect_ratio)
		else
			w = act_w
			h = act_h
		end

		if h < 240 then --The minimal height must >= 240
			h = 240
			w = math.floor(h*aspect_ratio)
			if w > max_width then
				w = max_width
			end
		end
	else
		w = cfg.scale_width
		h = cfg.scale_height
		if w > act_w then
			print( "** WARNING: The " .. stream_name .. " stream scaling width " .. w .. " is big than actual input " .. act_w .. ", so using actual input size." )
			w = act_w
		end

		if h > act_h then
			print( "** WARNING: The " .. stream_name .. " stream scaling height " .. h .. " is big than actual input " .. act_h .. ", so using actual input size." )
			h = act_h
		end

		if w > max_width then
			print( "***WARNING: The " .. stream_name .. " stream width " .. w .. " is big than limit " .. max_width .. ", so force it to be limited size." )
			w = max_width
		end
		if h > max_height then
			print( "***WARNING: The " .. stream_name .. " stream height " .. h .. " is big than limit " .. max_height .. ", so force it to be limited size." )
			h = max_height
		end
	end
	return w, h
end

----------------------------------

local VideoFormatArgsMap = {
	{ mine = "width", argName = "cap_cap_w", formatChanged = true },
	{ mine = "height", argName = "cap_cap_h", formatChanged = true },
	{ mine = "signal_locked", argName = "signal_locked", signalChanged = true, mustCheck = true },
	{ mine = "freerun", argName = "free_run", signalChanged = true, mustCheck = true },
	{ mine = "interlaced", argName = "interlaced", formatChanged = true, mustCheck = true },
	{ mine = "frame_rate", argName = "frame_rate_value", formatChanged = true },
	{ mine = "hdcp", argName = "hdcp_content", mustCheck = true },
	{ mine = "crop", argName = "out_crop", formatChanged = true, mustCheck = true },
	{ mine = "crop_width", argName = "out_crop_width", formatChanged = true },
	{ mine = "crop_height", argName = "out_crop_height", formatChanged = true },
	{ mine = "crop_x", argName = "out_crop_x", formatChanged = true },
	{ mine = "crop_y", argName = "out_crop_y", formatChanged = true },
	{ mine = "rmf", argName = "rmf", formatChanged = true },
}
----------------------------------

function _M.Check()
	local mo_avio = require "avio"
	if not mo_avio.avioModule or not mo_avio.avioModule.lowerDriverReady then	--Lower driver not configured, so ignore
		print( "Lower driver not ready! Wait." )
		return
	end
	
	if GetStreamerProxy() then
		local stream_id, opt
		for stream_id, opt in pairs(dynamic_bitrate_set_cache) do
			--NOTE: Use _NOTIFY to avoid wait ACK
			iStreamDefaultAppCfgProxy._NOTIFY:SetEnv( { 
				["stream[" .. stream_id .. "].bitrateLimit"] = opt.bitrate,
				["stream[" .. stream_id .. "].enable_dynamic_bitrate"] = opt.dyn_br
			} )
		end
		dynamic_bitrate_set_cache = {} --Clear.
	end

	local signalChanged = false
	local formatChanged = false

	if _M.firstRun or mo_avio.avioModule.video.formatChanged then
		mo_avio.avioModule.video.formatChanged = false
		local vFormat
		if mo_avio.avioModule.converter then
			vFormat = mo_avio.avioModule.video.convFormat
		else
			vFormat = mo_avio.avioModule.video.format
		end

		if not vFormat then
			return
		end
		
		local k, v
		for k = 1,#VideoFormatArgsMap do
			if _M.current_video_input[ VideoFormatArgsMap[k].mine ] ~= vFormat[ VideoFormatArgsMap[k].argName ] 
				and (VideoFormatArgsMap[k].mustCheck or vFormat[ VideoFormatArgsMap[k].argName ])  --If the value is valid, otherwise means no signal locked (without converter front-end)
			then
				_M.current_video_input[ VideoFormatArgsMap[k].mine ] = vFormat[ VideoFormatArgsMap[k].argName ]
				if VideoFormatArgsMap[k].formatChanged then
					formatChanged = true
				end
				if VideoFormatArgsMap[k].signalChanged then
					signalChanged = true
				end
			end
		end
	end

	if not _M.current_video_input.width or
		_M.current_video_input.width <= 240 or
		not _M.current_video_input.height or
		_M.current_video_input.height <= 240 then
		return --Invalid format.
	end

	if _M.firstRun or mo_avio.avioModule.video.sourceChanged then
		mo_avio.avioModule.video.sourceChanged = false
		--TODO: Do some actions when source changed?
	end

	if _M.firstRun or mo_avio.avioModule.audio.formatChanged then
		mo_avio.avioModule.audio.formatChanged = false
	end

	if _M.firstRun or mo_avio.avioModule.audio.sourceChanged then
		mo_avio.avioModule.audio.sourceChanged = false
	end

	--Combine encode_mode@snap_type
	local encMode = _G.encoder.encode_mode
	
	if _M.current_enc_mode ~= encMode then
		_M.current_enc_mode = encMode
		formatChanged = true	--Force this.
	end
	
	if _M.current_snap_type ~= _G.encoder.snap_type then
		_M.current_snap_type = _G.encoder.snap_type
		formatChanged = true	--Force this.
	end

	-- To notify streamer, to disable output for OEM version.
	if signalChanged then
		if GetStreamerProxy() then
			--Use _NOTIFY to avoid wait Ack
			iStreamDefaultAppCfgProxy._NOTIFY:Set( { ["streamer.enable_streaming"] = (_M.current_video_input.signal_locked and not _M.current_video_input.free_run) and 1 or 0 } )
		end
	end

	--if _M.firstRun or formatChanged then	--Format changed, so require stop all encoding, re-init buffers, then notify streamer to re-open.
	if _M.firstRun or formatChanged or ( not mo_avio.avioModule.converter and signalChanged ) then
		for k,v in pairs(_M.encoder_objs) do
			if _M.firstRun then
				v:_UpdateConfig( "all", true ) --hold configs only.
			end
			v:Stop()
		end

		_M.firstRun = false

		local bufferUses = {
			["X"] = "enc", --Should always be used.
			["Y"] = "off",
			["J"] = "off",
			["K"] = "off"
		}
	
		local iCmd = " -i 0"

		if DeviceWithOnchipOutput() and _G.avio and _G.avio.preview.enable ~= 0 then
			bufferUses["J"] = "prev"
			
			if _G.avio.preview.format == "auto" then
				local fmt = fmtUtil.GetMatchedFormat( _M.current_video_input.width, _M.current_video_input.height, _M.current_video_input.interlaced )
				iCmd = iCmd .. " -V " .. fmt 
			else		
				iCmd = iCmd .. " -V " .. _G.avio.preview.format
			end

			iCmd = iCmd .. " --hdmi --enc-mode 0 -" .. PREV_ENC_BUFFER .. " --btype=prev --bmaxsize=".. PREV_BUFFER_MAX_WIDTH .. "x" .. PREV_BUFFER_MAX_HEIGHT .. " --bsize="
			local w, h = fmtUtil.GetSizeFromFormat( _G.avio.preview.format )
			if not w or w <=0 or not h or h <= 0 then	--???? TODO:
				w = _M.current_video_input.width & 0xfffffff0
				h = _M.current_video_input.height & 0xfffffff8
			end

			iCmd = iCmd .. w .. "x" .. h
		else
			iCmd = iCmd .. " --enc-mode 0"
		end

		if encMode:match("main") then	--Main stream
			local cfg = _G.encoder.stream[0].video
			local self = _M.encoder_objs["00"]

			local w, h = CalculateImageSize( "main", cfg, MAIN_BUFFER_MAX_WIDTH, MAIN_BUFFER_MAX_HEIGHT)
			
			iCmd = iCmd .. " -" .. MAIN_ENC_BUFFER .. " --bmaxsize=" .. MAIN_BUFFER_MAX_WIDTH .. "x" .. MAIN_BUFFER_MAX_HEIGHT .. " --bsize="

			local offx, offy
			local crop_w, crop_h

			if _M.current_video_input.crop then
				crop_w = _M.current_video_input.crop_width
				crop_h = _M.current_video_input.crop_height
			else
				crop_w = _M.current_video_input.width
				crop_h = _M.current_video_input.height
			end

			if _M.current_video_input.crop then
				--The bug fixing: if offset + X or Y exceed the video width, it should be error.
				offx = _M.current_video_input.crop_x or 0
				offy = _M.current_video_input.crop_y or 0

				if (offx + crop_w) > _M.current_video_input.width then
					offx = _M.current_video_input.width - crop_w
				end
				
				if (offy + crop_h) > _M.current_video_input.height then
					offy = _M.current_video_input.height - crop_h
				end

				if offx < 0 then
					crop_w = crop_w + offx
					offx = 0
				end
				if offy < 0 then
					crop_h = crop_h + offy
					offy = 0
				end
				crop_w = crop_w & 0xfffffffc
				crop_h = crop_h & 0xfffffffc
			end
			
			--This need restrict the width/height, because of the input buffer size can not be any values.
			w = RESTRICT_WIDTH( w, crop_w )
			h = RESTRICT_HEIGHT( h, crop_h )
			self.informations.video.width = w
			self.informations.video.height = h
			
			iCmd = iCmd .. w .. "x" .. h
			if offx and offy then
				iCmd = iCmd .. " --binoffset=" .. offx .. "x" .. offy
			end

			iCmd = iCmd .. " --binsize=" .. crop_w .. "x" .. crop_h
			--iCmd = iCmd .. " --binsize=" .. _M.current_video_input.width .. "x" .. _M.current_video_input.height
				
			local rmf = _M.current_video_input.rmf or "0"
			if rmf == "90" or rmf == "270" then
				iCmd = iCmd .. " -A --smaxsize=" .. MAIN_BUFFER_MAX_HEIGHT .. "x" .. MAIN_BUFFER_MAX_WIDTH
			else
				iCmd = iCmd .. " -A --smaxsize=" .. MAIN_BUFFER_MAX_WIDTH .. "x" .. MAIN_BUFFER_MAX_HEIGHT
			end
		end

		if encMode:match("sub") then --Sub stream
			local cfg = _G.encoder.stream[1].video
			local self = _M.encoder_objs["01"]
			--local mainOne = _M.encoder_objs["00"]	--Who the main.
			
			bufferUses[ SUB_ENC_BUFFER ] = "enc"
			
			local w, h = CalculateImageSize( "sub", cfg, SUB_BUFFER_MAX_WIDTH, SUB_BUFFER_MAX_HEIGHT, DEFAULT_SUB_WIDTH, DEFAULT_SUB_HEIGHT)
			iCmd = iCmd .. " -" .. SUB_ENC_BUFFER .. " --btype=enc --bmaxsize=" .. SUB_BUFFER_MAX_WIDTH .. "x" .. SUB_BUFFER_MAX_HEIGHT .. " --bsize="

			local mainOne = _M.encoder_objs["00"]	--Who the main.

			--NOTE: The sub stream size restriction is not same as main buffer
			w = RESTRICT_WIDTH(w, mainOne.informations.video.width)
			h = RESTRICT_WIDTH(h, mainOne.informations.video.height)
			
			--Note the binsize is excact like the main buffer picture size (not always be buffer size)
			-- The input size must be x4
			iCmd = iCmd .. w .. "x" .. h  .. " --binsize=" .. ((mainOne.informations.video.width+3)&0xfffffffc) .. "x" .. ((mainOne.informations.video.height+3)&0xfffffffc)
			
			self.informations.video.width = w
			self.informations.video.height = h
			local rmf = _M.current_video_input.rmf or "0"
			if rmf == "90" or rmf == "270" then
				iCmd = iCmd .. " -B --smaxsize=" .. SUB_BUFFER_MAX_HEIGHT .. "x" .. SUB_BUFFER_MAX_WIDTH
			else
				iCmd = iCmd .. " -B --smaxsize=" .. SUB_BUFFER_MAX_WIDTH .. "x" .. SUB_BUFFER_MAX_HEIGHT
			end
		end

		-- ..............................SNAP.............................................
		--if encMode:match("snap") then --Snap stream
		if true then --Always enable snapping.
			--local cfg = _G.encoder.stream[2].video
			local self = _M.encoder_objs["02"]
			local mainOne = _M.encoder_objs["00"]	--Who the main.

			if _G.encoder.snap_type == "main" then	--Use the main buffer as input
				self.encode_Buffer = MAIN_ENC_BUFFER
				self.encode_BufferNum = MAIN_ENC_BUFFER_NUM
				self.max_width = MAIN_BUFFER_MAX_WIDTH
				self.max_height = MAIN_BUFFER_MAX_HEIGHT

				self.informations.video.width = mainOne.informations.video.width
				self.informations.video.height = mainOne.informations.video.height

				local rmf = _M.current_video_input.rmf or "0"
				if rmf == "90" or rmf == "270" then
					iCmd = iCmd .. " -C --smaxsize=" .. MAIN_BUFFER_MAX_HEIGHT .. "x" .. MAIN_BUFFER_MAX_WIDTH
				else
					iCmd = iCmd .. " -C --smaxsize=" .. MAIN_BUFFER_MAX_WIDTH .. "x" .. MAIN_BUFFER_MAX_HEIGHT
				end				
			
			elseif _G.encoder.snap_type == "sub" then --Use the sub buffer as input.
				self.encode_Buffer = SUB_ENC_BUFFER
				self.encode_BufferNum = SUB_ENC_BUFFER_NUM
				self.max_width = SUB_BUFFER_MAX_WIDTH
				self.max_height = SUB_BUFFER_MAX_HEIGHT
				
				if _G.encoder.stream[1].enabled == 1 then --Directly use
					local subOne = _M.encoder_objs["01"]
					self.informations.video.width = subOne.informations.video.width
					self.informations.video.height = subOne.informations.video.height
				else --So need calculate sub buffer size.
					local cfg = _G.encoder.stream[1].video
					bufferUses[ SUB_ENC_BUFFER ] = "enc"
			
					iCmd = iCmd .. " -" .. SUB_ENC_BUFFER .. " --btype=enc --bmaxsize=" .. SUB_BUFFER_MAX_WIDTH .. "x" .. SUB_BUFFER_MAX_HEIGHT .. " --bsize="
					local w, h = CalculateImageSize("preview", cfg, SUB_BUFFER_MAX_WIDTH, SUB_BUFFER_MAX_HEIGHT, DEFAULT_SUB_WIDTH, DEFAULT_SUB_HEIGHT)
					--NOTE: The sub stream size restriction is not same as main buffer
					w = RESTRICT_WIDTH(w, mainOne.informations.video.width) 
					h = RESTRICT_WIDTH(h, mainOne.informations.video.height)
					self.informations.video.width = w
					self.informations.video.height = h
			
					--Note the binsize is excact like the main buffer picture size (not always be buffer size)
					-- The input size must be x4
					iCmd = iCmd .. w .. "x" .. h  .. " --binsize=" .. ((mainOne.informations.video.width+3)&0xfffffffc) .. "x" .. ((mainOne.informations.video.height+3)&0xfffffffc)
				end

				local rmf = _M.current_video_input.rmf or "0"
				if rmf == "90" or rmf == "270" then
					iCmd = iCmd .. " -C --smaxsize=" .. SUB_BUFFER_MAX_HEIGHT .. "x" .. SUB_BUFFER_MAX_WIDTH
				else
					iCmd = iCmd .. " -C --smaxsize=" .. SUB_BUFFER_MAX_WIDTH .. "x" .. SUB_BUFFER_MAX_HEIGHT
				end				
				
			else --Preview snapping.
				bufferUses[ SNAP_ENC_BUFFER ] = "enc"
				iCmd = iCmd .. " -" .. SNAP_ENC_BUFFER .. " --btype=enc --bmaxsize=" .. SNAP_BUFFER_MAX_WIDTH .. "x" .. SNAP_BUFFER_MAX_HEIGHT .. " --bsize="
				self.encode_Buffer = SNAP_ENC_BUFFER
				self.encode_BufferNum = SNAP_ENC_BUFFER_NUM
				self.max_width = SNAP_BUFFER_MAX_WIDTH
				self.max_height = SNAP_BUFFER_MAX_HEIGHT

				local w, h = CalculateImageSize("preview", {scale="default"}, SNAP_BUFFER_MAX_WIDTH, SNAP_BUFFER_MAX_HEIGHT, DEFAULT_SNAP_WIDTH, DEFAULT_SNAP_HEIGHT )

				--NOTE: The snap stream size restriction is not same as main buffer
				w = RESTRICT_WIDTH(w, mainOne.informations.video.width)
				h = RESTRICT_WIDTH(h, mainOne.informations.video.height)
				self.informations.video.width = w
				self.informations.video.height = h

				-- The input size must be x4
				iCmd = iCmd .. w .. "x" .. h  .. " --binsize=" .. ((mainOne.informations.video.width+3)&0xfffffffc) .. "x" .. ((mainOne.informations.video.height+3)&0xfffffffc)

				local rmf = _M.current_video_input.rmf or "0"
				if rmf == "90" or rmf == "270" then
					iCmd = iCmd .. " -C --smaxsize=" .. SNAP_BUFFER_MAX_HEIGHT .. "x" .. SNAP_BUFFER_MAX_WIDTH
				else
					iCmd = iCmd .. " -C --smaxsize=" .. SNAP_BUFFER_MAX_WIDTH .. "x" .. SNAP_BUFFER_MAX_HEIGHT
				end
			end --Preview snapping.
		end
		-- ..............................SNAP......................................
		
		for k,v in pairs(bufferUses) do
			if v == "off" then
				iCmd = iCmd .. " -" .. k .. " --btype=off"
			end
		end
	
		local rmf = _M.current_video_input.rmf or "0"
		if rmf == "90" or rmf == "270" then
			iCmd = iCmd .. " --enc-rotate-possible=1"
		else
			iCmd = iCmd .. " --enc-rotate-possible=0"
		end
		print( "** Initialize:", iCmd )
		os.execute( EXT_TOOLS .. iCmd )

		encModeUpdateCounter = UPDATE_STREAM_ENCMODE_PERIOD	--Update it immediately.

		for k,v in pairs(_M.encoder_objs) do
			v:Start()	--Try start all.
		end
--[[
	elseif signalChanged then
		--Update encoder
		for k,v in pairs(_M.encoder_objs) do
			v:Stop()	--Stop streaming first, and restart encoding.
			v:_ActualStart()	--Re-set the encoder parameters.
		end
]]
	else
		for k,v in pairs(_M.encoder_objs) do
			if v.status == "starting" or v.status:match("retry") then	--Start the pending encoder objects.
				v:_ActualStart()	--Re-set the encoder parameters.
			end
		end

		encModeUpdateCounter = encModeUpdateCounter + 1
	end

	--Period notify the streamer to update the encMode
	if encModeUpdateCounter >= UPDATE_STREAM_ENCMODE_PERIOD	then
		if GetStreamerProxy() then
			--Use _NOTIFY to avoid wait Ack
			iStreamDefaultAppCfgProxy._NOTIFY:Set( { ["streamer.stream_mode"] = encMode } )
		end

		encModeUpdateCounter = 0
	end

	--Twice again.
	if GetStreamerProxy() then
		local stream_id, opt
		for stream_id, opt in pairs(dynamic_bitrate_set_cache) do
			--NOTE: Use _NOTIFY to avoid wait ACK
			iStreamDefaultAppCfgProxy._NOTIFY:SetEnv( { 
				["stream[" .. stream_id .. "].bitrateLimit"] = opt.bitrate,
				["stream[" .. stream_id .. "].enable_dynamic_bitrate"] = opt.dyn_br
			} )
		end
		dynamic_bitrate_set_cache = {} --Clear.
	end
end

-------------

--Update encoder parameters.
function _M.NotifyUpdateEncoder( idr )
	local k,v
	for k,v in pairs(_M.encoder_objs) do
		if not idr or idr == "" or idr == "all" or idr == "*" or idr == k then
			v:_UpdateConfig("all")	--Re-set the encoder parameters.
		end
	end
end

function _M.NotifyStartEncoder( idr )
	local k,v
	for k,v in pairs(_M.encoder_objs) do
		if not idr or idr == "" or idr == "all" or idr == "*" or idr == k then
			v:Start()
		end
	end
end

function _M.NotifyStopEncoder( idr )
	local k,v
	for k,v in pairs(_M.encoder_objs) do
		if not idr or idr == "" or idr == "all" or idr == "*" or idr == k then
			v:Stop()
		end
	end
end

function _M.NotifyReset()
	_M.firstRun = true	--Mark first run, so when checking, all encoders should stop and restart.
end

function _M.CheckStreamStatus( idr )
	idr = string.format( "%02d", tonumber(idr) )
	local svc_obj = _M.encoder_objs[idr]
	if not svc_obj then
		return false, 0, 0, 0
	else
		if svc_obj.status ~= "started" then
			return false, 0, 0, 0
		elseif _M.current_video_input.rmf == "90" or _M.current_video_input.rmf == "270" then
			return true, svc_obj.informations.video.height, svc_obj.informations.video.width, true, tonumber(_M.current_video_input.rmf)
		else
			return true, svc_obj.informations.video.width, svc_obj.informations.video.height, false, tonumber(_M.current_video_input.rmf) or 0
		end
	end
end

return _M
