require "kiloview.configure_helper"
local fmtUtil = require "formatUtil"
local mo_encoder = require "encoder"

--------------
local function getStreamEnable( owner, pObj )
	local rootObj = owner:Root()
	if rootObj then
		local thisStream = pObj:Parent():Name()
		if thisStream == "2" then
			pObj:InternalSetInt(1)	--Always enable the snap/JPEG stream.
		elseif rootObj.encode_mode == "main" then
			if thisStream == "0" then
				pObj:InternalSetInt(1)
			else
				pObj:InternalSetInt(0)
			end
		elseif rootObj.encode_mode == "main+sub" then
			pObj:InternalSetInt(1)
		else
			pObj:InternalSetInt(0)
		end
	else
		pObj:InternalSetInt(0)
	end
end

--------------
local function getScale( owner, pObj )
	if owner.scale_width <= 0 or owner.scale_height <= 0 then
		pObj:InternalSetString("default")
	else
		pObj:InternalSetString( owner.scale_orig )
	end
end

local function setScale( owner, pObj, value )
	if not value or value == "" or value == "default" or value=="0x0" then
		owner.scale_width = 0	--Restore default
		owner.scale_height = 0
		owner.scale_orig = "default"
	else
		owner.scale_orig = value
		local w,h = value:match("(%d+)x(%d+)")
		--local w,h = fmtUtil.GetSizeFromFormat(value)
		if w and h then
			owner.scale_width = tonumber(w) --& 0xfffffff0
			owner.scale_height = tonumber(h) --& 0xfffffff8
		end
	end
end

--------------
local function getFrameMode( owner, pObj )
	if owner.framerate <= 0 then
		pObj:InternalSetString("default")
	elseif owner.framerate == 0.5 then
		pObj:InternalSetString("half")
	else
		pObj:InternalSetString("custom")
	end
end

local function setFrameMode( owner, pObj, value )
	if not value or value == "" or value == "default" or value=="0" then
		owner.framerate = 0
	elseif value == "half" then
		owner.framerate = 0.5
	end
end

--------------

local function onEncModeChanged( owner, pObj )
	mo_encoder.NotifyReset()
end

--------------

local function onStreamConfigChanged( obj )
	local p = obj:Parent()
	local s = p and p:Name() or "all"
	if s ~= "all" then
		s = "0" .. s	--00,01,02, etc.
	end

	mo_encoder.NotifyUpdateEncoder(s)
end

--------------

local function getCodec( owner, pObj )
	local thisStream = pObj:Parent():Parent():Name()
	if thisStream == '2' then
		pObj:InternalSetString( "MJPEG" )
	end --Else use default!
end

local function getProfile( owner, pObj )
	local thisStream = pObj:Parent():Parent():Name()
	if thisStream == '2' then
		pObj:InternalSetString( "default" )
	end --Else use configured.
end

--------------

Encoders_Template = OBJECT({
	encode_mode = { type="string", value="main" },
	snap_type = { type="string", value="preview" }, --Snap type: preview / main / sub.
	
	stream = ARRAY(
		{ 0,1,2 },

		OBJECT({
			enabled = { type="integer",  noPersist=true, writeEnable=false, onRead = getStreamEnable },
			video = OBJECT({
				scale = { type="string", noPersist=true, onRead = getScale, onWrite = setScale },
				scale_orig = { type="string", value="default" }, --Keep the original value setting.
				scale_width = { type="integer", value=0 },
				scale_height = { type="integer", value=0 },
				codec = { type="string", value="H264", onRead = getCodec },
				br_ctrl = { type="string", value="cbr" },
				bitrate = { type="integer", value=2000000 },
				dynamic_bitrate = { type="integer", value=0 },
				reduce_framerate = { type="integer", value=0 },
				profile = { type="string", value="high", onRead = getProfile },
				avclevel = { type="integer", value=40 },
				gop_size = { type="integer", value=60 },
				gop = { type="string", value="IP" },				
				framerate_mode = { type="string", noPersist = true, onRead = getFrameMode, onWrite = setFrameMode },
				framerate = { type="double", value=0.5 },
				quality = { type="integer", value=80 },
				ref_frames = { type="integer", value=1 },
				qp_max = { type = "integer", value = 51},
				qp_min = { type = "integer", value = 18},
				chrome = { type="integer", value=0 }
			}, onStreamConfigChanged),

			audio = OBJECT({
				codec = { type="string", value="AAC" },
				br_ctrl = { type="string", value="cbr" },
				bitrate = { type="integer", value=64000 },
			}, onStreamConfigChanged)
		})
	)
})



