require "osd2"
local MAX_AREAS = 16
local _dbg = require "coaf.debug"

local RGN
local _did = pcall(require, "himpp")
if _did then
	RGN = HiMPP.RGN
end

--NOTE: Assign the default font for current products.
KM_OSD2.setDefaultFont("/usr/share/fonts/gbsn00lp.ttf")

local _M = {
	fontLocation = "/usr/share/fonts",
	imageLocation = "/data/osd_images",

	fontMap = { --Total 8 items.
		"default.ttf",
		"default.ttf",
		"default.ttf",
		"default.ttf",
		"default.ttf",
		"default.ttf",
		"default.ttf",
		"default.ttf"
	},

	colorType = "rgba",

	colorMap = {
		{ 0, 0, 0, 255 },
		{ 255, 255, 255, 255 },
		{ 255, 0, 0, 255 },
		{ 0, 0, 255, 255 },
		{ 0, 255, 0, 255 },
		{ 255, 255, 0, 255 },
		{ 255, 0, 255, 255 },
		{ 0, 255, 255, 255 }
	},

	allAdapters = {}
}

--------------------------------------
local osdmeta = {}

function osdmeta:Reset()
	local area_id, area_detail
	for area_id, area_detail in pairs(self.areas) do
		area_detail.surface = nil	--Destroy it.
	end
	collectgarbage()
end

function osdmeta:ChangeSize(w,h,rotate)
	if self.width ~= w or self.height ~= h or self.rotate ~= rotate then
		self.width = w
		self.height = h
		self.rotate = rotate
		self.osdd:ChangeSize(w,h,rotate)
	end
end

function osdmeta:Destroy()
	local i
	for i=1, #_M.allAdapters do
		if _M.allAdapters[i] == self then
			table.remove( _M.allAdapters, i )
			break
		end
	end

	self:Reset()
	self.osdd = nil
	collectgarbage()
end

function osdmeta:Update()
	local i, j
	local shift_y
	local area, srh_area
	local repaint

	for i=0, MAX_AREAS-1 do
		area = self.areas[i]
		if area and area.surface then
			--repaint = false
			repaint = area.last_changed
			area.last_changed = nil

			if area.related then
				shift_y = 0
				for j=0, i-1 do
					srh_area = self.areas[j]
					if srh_area and srh_area.posi_type == area.posi_type and srh_area.surface then
						shift_y = shift_y + srh_area.h
					end
				end

				shift_y = area.y + shift_y*area.related
				if area.x ~= area.render_x or shift_y ~= area.render_y then
					area.render_x = area.x
					area.render_y = shift_y
					repaint = true
				end
			else
				if area.x ~= area.render_x or area.render_y ~= area.y then
					area.render_x = area.x
					area.render_y = area.y
					repaint = true
				end
			end

			if repaint then
				self.osdd:RenderSurface( area.surface, { x = area.render_x, y = area.render_y, z = i } )
				--@@4
				if self.afterChangedAreaPosition then
					self:afterChangedAreaPosition( i, area.render_x, area.render_y )
				end
			end
		end
	end
end

function osdmeta:ForceRefresh()
	self.osdd:ForceRefresh()
end

function osdmeta:DropArea(areaId)
	if self.areas[ areaId ] then
		self.areas[areaId].surface = nil
		--alternative destroy. --@@1
		if self.afterDestroyArea then
			self:afterDestroyArea(areaId)
		end
		self.areas[areaId] = nil
		collectgarbage()
	end
end

function osdmeta:EnableArea(areaId, en)
	if self.areas[areaId] then
		self.areas[areaId].enable = en
		if self.areas[areaId].surface then
			self.areas[areaId].surface:SetEnable(en)
		end
	end
end

function osdmeta:ChangeAreaType(areaId, aType)
	local area_detail = self.areas[areaId]
	if not area_detail then
		area_detail = {
			id = area_id,
			enable = true,
			area_type = nil,
			surface = nil,
			posi_type = nil,
			x = 0,
			y = 0,
			render_x = nil,
			render_y = nil,
			w = 0,
			h = 0
		}

		self.areas[areaId] = area_detail
	end

	aType = aType:lower()
	if aType == "date" or aType == "time" or aType == "clock" or aType == "datetime" then
		aType = "datetime"
	elseif aType == "image" then
		aType = "image"
	else
		aType = "text"
	end

	if aType ~= area_detail.area_type or not area_detail.surface then
		area_detail.last_changed = true
		area_detail.area_type = aType
		if area_detail.surface then
			area_detail.surface = nil
			--alternative destroy. --@@1
			if self.afterDestroyArea then
				self:afterDestroyArea(areaId)
			end
			collectgarbage()
		end

		local flag = 0
		--@@2
		if self.getSurfaceCreateFlag then
			flag = self:getSurfaceCreateFlag( aType, areaId )
		end
		area_detail.surface = self.osdd:CreateSurface( aType, flag )
		if not area_detail.surface then
			_dbg.Error( "OSD", "Fail to create surface for area:", areaId, ", module:", self.name )
		end
	end
end

function osdmeta:SetAreaFont( areaId, fontOpts )
	local area_detail = self.areas[areaId]
	if area_detail and area_detail.surface and type(fontOpts) == "table" then
		local fontName = _M.fontLocation .. "/" .. _M.fontMap[fontOpts.font + 1]
		area_detail.surface:SetFont({
			font = fontName,
			size = fontOpts.size,
			bold = fontOpts.bold,
			italic = fontOpts.italic,
			outline = fontOpts.outline
		})
	end
end

function osdmeta:SetAreaFontColor( areaId, colorId )
	local area_detail = self.areas[areaId]
	if area_detail and area_detail.surface then
		local colorItem = _M.colorMap[ colorId and colorId+1 or 1 ]
		if colorItem then
			if _M.colorType == "yuv" or _M.colorType == "yuva" then
				area_detail.surface:SetFontColorYUV( colorItem[1], colorItem[2], colorItem[3], colorItem[4] )
			else
				area_detail.surface:SetFontColorRGB( colorItem[1], colorItem[2], colorItem[3], colorItem[4] )
			end
		end
	end
end

function osdmeta:SetAreaOutlineColor( areaId, colorId )
	local area_detail = self.areas[areaId]
	if area_detail and area_detail.surface then
		local colorItem = _M.colorMap[ colorId and colorId+1 or 1 ]
		if colorItem then
			if _M.colorType == "yuv" or _M.colorType == "yuva" then
				area_detail.surface:SetOutlineColorYUV( colorItem[1], colorItem[2], colorItem[3], colorItem[4] )
			else
				area_detail.surface:SetOutlineColorRGB( colorItem[1], colorItem[2], colorItem[3], colorItem[4] )
			end
		end
	end
end

function osdmeta:SetAreaLineHeight( areaId, h )
	local area_detail = self.areas[areaId]
	if area_detail and area_detail.surface then
		area_detail.surface:SetLineHeight(h or 0)
	end
end

function osdmeta:SetAreaText( areaId, text )
	local area_detail = self.areas[areaId]
	if area_detail and area_detail.surface then
		area_detail.surface:UpdateText(text)
	end
end

function osdmeta:TellAreaSize( areaId )
	local area_detail = self.areas[areaId]
	if area_detail and area_detail.surface then
		return area_detail.surface:TellSize()
	else
		return 0,0
	end
end

function osdmeta:ChangeAreaSize( areaId, w, h )
	local area_detail = self.areas[areaId]

	if self.restrictAreaSize then
		w, h = self:restrictAreaSize(areaId, w, h)
	end

	if area_detail then
		--@@5
		if area_detail.w ~= w or area_detail.h ~= h then
			area_detail.w = w
			area_detail.h = h
			--@@3
			if self.beforeResizeArea then
				self:beforeResizeArea( areaId, w, h )
			end
		end

		if area_detail.surface then
			area_detail.surface:Resize(w,h)
		end

		return w, h
	else
		return w, h
	end
end

function osdmeta:SetAreaPositionType( areaId, pt )
	local area_detail = self.areas[areaId]
	if area_detail then
		area_detail.posi_type = pt
		if pt == "left-top" then
			area_detail.related = 1
		elseif pt == "top-center" then
			area_detail.related = 1
		elseif pt == "right-top" then
			area_detail.related = 1
		elseif pt == "left-bottom" then
			area_detail.related = -1
		elseif pt == "bottom-center" then
			area_detail.related = -1
		elseif pt == "right-bottom" then
			area_detail.related = -1
		elseif pt == "center" then
			area_detail.related = -1
		else
			area_detail.related = nil
		end
	end
end

function osdmeta:SetAreaPosition( areaId, x, y )
	local area_detail = self.areas[areaId]
	if area_detail then
		area_detail.x = x
		area_detail.y = y
	end
end

function osdmeta:LoadImageByID(areaId, imageId, current_win_w, current_win_h )
	if not areaId or not imageId then
		_dbg.Error("OSD", "Invalid arguments for loading image by ID.")
		return nil
	end

	local area_detail = self.areas[areaId]
	if not area_detail or not area_detail.surface or area_detail.area_type ~= "image" then
		_dbg.Error("OSD", "No image surface created for loading image by ID. area=", areaId, ", image-id=", imageId )
		return nil
	end

    local thisAreaW, thisAreaH
	local tp_x, tp_y
    local imageFile
    local alpha

    local imageDescFile = _M.imageLocation .. "/" .. imageId .. "/images.desc"
    local done, func = pcall( loadfile, imageDescFile )
    if not done then
		_dbg.Error( "OSD", "Invalid image id while loading image, area=", areaId, "image-id=", imageId )
        return nil
	end

    local desc_table
    done, desc_table = pcall(func)
    if not done or type(desc_table) ~= "table" then
		_dbg.Error( "OSD", "Fail to loading image desc, area=", areaId, "image-id=", imageId )
        return nil
    end

	--TODO: now ignore the window size matching.
	current_win_w = current_win_w or 3840
	current_win_h = current_win_h or 2160

    local i, knowSize
    for i=1, #desc_table do
        if (not desc_table[i].width or current_win_w >= desc_table[i].width) and
           (not desc_table[i].height or current_win_h >= desc_table[i].height) then
           imageFile = _M.imageLocation .. "/" .. imageId .. "/" .. (desc_table[i].file or "default.raw")
           if desc_table[i].transparent then
               tp_x = desc_table[i].transparent[1]
               tp_y = desc_table[i].transparent[2]
           end
           alpha = desc_table[i].alpha
           if type(desc_table[i].size) == "table" then
               thisAreaW = desc_table[i].size[3] --NOTE: 3/4 is the restricted size (x32 : x16)
               thisAreaH = desc_table[i].size[4]
               if not thisAreaW then
                   thisAreaW = desc_table[i].size[1]
               end
               if not thisAreaH then
                   thisAreaH = desc_table[i].size[2]
               end
               knowSize = true
           end
           break
        end
    end

    if not imageFile then
		_dbg.Error( "OSD", "Fail to loading image by ID since no image file found! area=", areaId, "image-id=", imageId )
        return nil
    end

    if not knowSize then
		_dbg.Error( "OSD", "TODO: unknow the image file size! area=", areaId, "image-id=", imageId )
        return nil
	end

	local w,h = self:ChangeAreaSize(areaId, thisAreaW, thisAreaH)

	if not area_detail.surface:LoadFile({ filename = imageFile, w = thisAreaW, h = thisAreaH, alpha = alpha or 255, format = "amba.ayuv" }) then
		_dbg.Error( "OSD", "Fail to load image for OSD burning! area=", areaId, "image-id=", imageId )
		return nil
	else
		_dbg.Info( "OSD", "Image loaded. area=", areaId, "image-id=", imageId, "file=", imageFile, "size:", thisAreaW, thisAreaH )
		return w or thisAreaW, h or thisAreaH
	end
end

function osdmeta:LoadImageFile(areaId, filename, w, h, alpha, format)
	--TODO:
end

osdmeta.__index = osdmeta

--#####################################################

local function newRegionOsd( attachChn, regionType, baseRegionId, w, h)
	local region_osdd = KM_OSD2.Device.createNew( "hi_region", { width=w, height=h } )
	if not region_osdd then
		_dbg.Error( "OSD", "Fail to create hi_region OSD device!\n" )
		return nil
	end

	local region_osd = {
		osd_type = "hi_region",
		regionType = regionType or "OVERLAYEX",
		chnObj = attachChn,
		osdd = region_osdd,
		base_id = baseRegionId,
		allRegions = {},
		areas = {},
		width = w,
		height = h,
		rotate = 0
	}

	function region_osd:afterChangedAreaPosition(areaId, x, y)
		if self.allRegions[areaId] and self.areas[areaId] and self.areas[areaId].surface then
			x, y = self.osdd:GetRegionPosition( self.areas[ areaId ].surface )
			self.allRegions[areaId]:DetachFromChn( self.chnObj )
			self.allRegions[areaId]:AttachToChn( self.chnObj, {
				bShow = true,
				enType = self.regionType,
				attr = {
					stPoint = { x&0xfffffffe, y&0xfffffffe },
					u32FgAlpha = self.regionType == "OVERLAY" and 128 or 255,
					u32BgAlpha = 0,
					u32Layer = areaId,
					stQpInfo = {
						bQpDisable = false
					},
					stInvertColor = {
						bInvColEn = false,
						stInvColArea = {0,0}
					}
				}
			})
		end
	end

	function region_osd:afterDestroyArea(areaId)
		local region = self.allRegions[areaId]
		if region then
			region:DetachFromChn( self.chnObj )
			region:Destroy()
			if self.areas[areaId] then
				self.areas[areaId].render_x = nil
				self.areas[areaId].render_y = nil
			end
			self.allRegions[areaId] = nil
		end
	end

	function region_osd:getSurfaceCreateFlag(areaType, areaId)
		return self.base_id + areaId
	end

	function region_osd:beforeResizeArea(areaId, w, h)
		local region = self.allRegions[areaId]
		if region then
			region:DetachFromChn( self.chnObj )
			region:Destroy()
			if self.areas[areaId] then
				self.areas[areaId].render_x = nil
				self.areas[areaId].render_y = nil
			end
		end

		region = RGN.GetRegion( self.base_id + areaId )
		region:Create({
			type = self.regionType,
			attr = {
				enPixelFmt = "RGB_1555",
				u32BgColor = 0,
				stSize = { w, h }
			}
		})

		self.allRegions[areaId] = region
	end

	function region_osd:restrictAreaSize(areaId, w,h)
		return (w+15)&0xfff0, (h+7)&0xfff8
	end

	return region_osd
end

--#####################################################

local staticAmbaOSDD = nil

local function newAmbaOsd(streamId)
	if not staticAmbaOSDD then
		staticAmbaOSDD = KM_OSD2.Device.createNew( "amba_iav", "extra" )
		if not staticAmbaOSDD then
			_dbg.Error( "OSD", "Can't create Amba OSD device!" )
			return nil
		end
	end

	local amba_osd = {
		osd_type = "amba_iav",
		osdd = staticAmbaOSDD,
		stream_id = streamId,
		areas = {},
		width = 0,
		height = 0,
		rotate = 0,
		surface_recorded = {}
	}

	function amba_osd:getSurfaceCreateFlag(areaType, areaId)
		areaId = areaId or 0

		--NOTE: for the new Extra working mode, suggest to use area 0,1,2 for image, and
		--      >=3 for text/date-time.
		local idx
		if areaType == "image" then
			local i
			for i=0,2 do
				if not self.surface_recorded[i] then
					self.surface_recorded[i] = areaId
					idx = i
					break
				end
			end
		end

		if not idx then
			local i
			for i=3,MAX_AREAS-1 do
				if not self.surface_recorded[i] then
					self.surface_recorded[i] = areaId
					idx = i
					break
				end
			end
			if not idx and areaType ~= "image" then --Try check front 0-2 areas is usable?
				for i=0,2 do
					if not self.surface_recorded[i] then
						self.surface_recorded[i] = areaId
						idx = i
						break
					end
				end
			end
		end

		return (self.stream_id << 16) | (idx or 0) 
	end
	
	function amba_osd:afterDestroyArea(areaId)
		local i
		for i=0, MAX_AREAS-1 do
			if self.surface_recorded[i] and self.surface_recorded[i] == areaId then
				self.surface_recorded[i] = nil
			end
		end
	end

	function amba_osd:restrictAreaSize(areaId, w,h)
		return (w+3)&0xfffc, (h+1)&0xfffe
	end

	return amba_osd
end

--#####################################################

local function newFbOsd(fbDev, w, h)
	local fb_osdd = KM_OSD2.Device.createNew( "fb", { device = fbDev, width=w, height=h } )
	if not fb_osdd then
		_dbg.Error( "OSD", "Can't create FB OSD device!" )
		return nil
	end

	local fb_osd = {
		osd_type = "fb",
		osdd = fb_osdd,
		areas = {},
		width = w,
		height = h,
		rotate = 0
	}

	function fb_osd:getSurfaceCreateFlag(areaType, areaId)
		return areaId + 1 -- >=1, To use 16 units of pallet.
	end

	return fb_osd
end

--#####################################################

function _M.createNew( adpType, opts )
	local osdObj
	if adpType == "fb" then
		local dev, w, h
		if type(opts) == "string" then
			dev = opts
			w = 0
			h = 0
		elseif type(opts) == "table" then
			dev = opts.device
			w = opts.width or 0
			h = opts.height or 0
		end
		if not dev then
			_dbg.Error( "OSD", "Invalid FB device for 'fb' OSD adapter!" )
			return nil
		end
		osdObj = newFbOsd( dev, w, h )

	elseif adpType == "amba_iav" then
		local streamId
		if type(opts) == "number" then
			streamId = opts
		elseif type(opts) == "table" then
			streamId = opts.id
		end
		if not streamId then
			_dbg.Error( "OSD", "Invalid stream-id for 'amba_iav' OSD adapter!" )
			return nil
		end

		osdObj = newAmbaOsd( streamId )

	elseif adpType == "hi_region" and RGN then
		local attachChn
		local baseRegionId
		local overlayType
		local w
		local h
		if type(opts) == "table" then
			attachChn = opts.channel
			baseRegionId = opts.base_id
			overlayType = opts.overlay_type
			w = opts.width or 0
			h = opts.height or 0
		end

		if not attachChn or not baseRegionId then
			_dbg.Error( "OSD", "Invalid attaching channel or base region-id for 'hi_region' OSD adapter!" )
			return nil
		end
		osdObj = newRegionOsd(attachChn, overlayType, baseRegionId, w, h)
	else
		_dbg.Error( "OSD", "Unsupported OSD adapter type of '" .. (adpType or "nil") .. "'" )
		return nil
	end

	if not osdObj then
		_dbg.Error( "OSD", "Fail to create OSD adapter, with type:", adpType )
		return nil
	end

	setmetatable( osdObj, osdmeta )
	table.insert( _M.allAdapters, osdObj )
	return osdObj
end

--#####################################################
function _M.ForceRefresh()
	if staticAmbaOSDD then
		staticAmbaOSDD:ForceRefresh()
	end
end

function _M.PeriodRender()
	--First render Amba-iav OSD, since it's static.
	if staticAmbaOSDD then
		staticAmbaOSDD:RenderSurface(nil)
	end
--[[
	local _i, osdObj
	for _i, osdObj in pairs(_M.allAdapters) do
		if osdObj.osd_type ~= "amba_iav" then
			osdObj.osdd:RenderSurface(nil)
		end
	end
--]]
end

--#####################################################

return _M

