local checker = require "statusChecker"

local _M = {}

--The G1 device IO/LEDs control parameters
_G.IOLED_OPTIONS = 
{
	WITH_FPGA = true,	--With FPGA
	FPGA_DEV_ADDR = 0xf4,	--FPGA I2C device address
	WITH_FPGA_LEDS	= true,	--With FPGA LEDs
	------
	RESET_PIN = 9,	--GPIO9 as RESET button
	------
	GPIO_LEDS = {
		["signal"] = 58,	--GPIO signal LED
		["running"] = 57	--GPIO running LED
	},
	------
	FPGA_LEDS = {
		["wifi"] = 0x10,	--Register address [ all same bellow ]
		["g4"] = 0x12,
		["stream"] = 0x14,
		["audio_source"] = 0x16,

		--Dump, for user controls.
		["led1"] = 0x10,
		["led2"] = 0x12,
		["led3"] = 0x14,
		["led4"] = 0x16
	},
	------
	--Rolling options: When pressed (but no arrived reset time) RESET button, rolling LEDs
	ROLLING_SEQ = { "wifi", "g4", "stream", "audio_source" },
	
	ROLLING_LANG_STATUS = {
		[0x01] = "RED:ON",		--Default language is CN
		[0x04] = "GREEN:ON",	--Default language is EN
		[0x00] = "YELLOW:ON"	--Other languages
	},
	
	ROLLING_MIN_NUMS = 1,	--How many leds at least show when rolling (Multiple languages support should +1)
	------
	ACTION_STATUS = {
		["default"] = {
			["signal"] = "OFF",
			["running"] = "BLINK",
			["wifi"] = "OFF",
			["g4"] = "OFF",
			["stream"] = "OFF",
			["audio_source"] = "OFF"
		},
	
		["restore"] = {
			["signal"] = "OFF",
			["running"] = "OFF",
			["wifi"] = "YELLOW:BLINK",
			["g4"] = "YELLOW:BLINK",
			["stream"] = "YELLOW:BLINK",
			["audio_source"] = "YELLOW:BLINK"
		},
	
		["reboot"] = {
			["signal"] = "OFF",
			["running"] = "OFF",
			["wifi"] = "YELLOW:BLINK",
			["g4"] = "YELLOW:BLINK",
			["stream"] = "YELLOW:BLINK",
			["audio_source"] = "YELLOW:BLINK"
		},
		
		["reset"] = {
			["signal"] = "BLINK",
			["running"] = "BLINK",
			["wifi"] = "GREEN:BLINK",
			["g4"] = "GREEN:BLINK",
			["stream"] = "GREEN:BLINK",
			["audio_source"] = "GREEN:BLINK"
		}
	},
	
	RESET_HOLD = 5000	--Hold RESET button more than this value(milliseconds), should do restore factory action!
}

require "oem"
if _G.WITHOUT_CONVERT then
	_G.IOLED_OPTIONS.FPGA_DEV_ADDR = 0xE4
	_G.IOLED_OPTIONS.FPGA_I2C_DEV = "/dev/i2c-0"
end
local ctrl_module = nil

function _M.Check(period_ms)
	if not ctrl_module then
		ctrl_module = require( "controlModule" )
	end
	
	if not ctrl_module then
		return
	end
	
	if not checker then
		return
	end
	
	--Signal checking
	-- local signal = checker.CheckSignal()
	-- if signal then
	-- 	ctrl_module.LedControl( "signal", "ON" )
	-- else
	-- 	ctrl_module.LedControl( "signal", "OFF" )
	-- end
	
	--Running modules
	local modules = checker.CheckRunningModules()
	if not modules then
		ctrl_module.LedControl( "running", "OFF" )
	else
		if modules["AVIO"] and modules["CODEC"] and modules["STREAMER"] then
			if modules["WEB"] then
				ctrl_module.LedControl( "running", "BLINK-FAST" )
			else
				ctrl_module.LedControl( "running", "BLINK" )
			end
		else
			ctrl_module.LedControl( "running", "ON" )
		end
	end

	if ctrl_module.userControl then return end

	--Wifi checking
	local wifiName, workMode, status = checker.CheckWifiStatus()
	if not wifiName then
		ctrl_module.LedControl( "wifi", "OFF" )
	else
		if workMode == "STATION" then
			local ledStatus = "OFF"
			if status == "IDLE" then
				ledStatus = "RED:BLINK"
			elseif status == "CONNECTING" then
				ledStatus = "GREEN:BLINK"
			elseif status == "CONNECTED" or status == "INTERNET_OK" then
				ledStatus = "GREEN:ON"
			elseif status == "ERROR" then
				ledStatus = "RED:ON"
			else
				ledStatus = "YELLOW:ON"
			end
			ctrl_module.LedControl( "wifi", ledStatus )
		else
			ctrl_module.LedControl( "wifi", "YELLOW:BLINK" )
		end
	end

	--4G checking
	local g4Name, mode, status = checker.Check4GStatus()
	if not g4Name then
		ctrl_module.LedControl( "g4", "OFF" )
	else
		local ledStatus = "OFF"
		if status == "IDLE" then
			ledStatus = "RED:BLINK"
		elseif status == "CONNECTING" then
			ledStatus = "GREEN:BLINK"
		elseif status == "CONNECTED" or status == "INTERNET_OK" then
			ledStatus = "GREEN:ON"
		elseif status == "ERROR" then
			ledStatus = "RED:ON"
		else
			ledStatus = "YELLOW:ON"
		end
		ctrl_module.LedControl( "g4", ledStatus )
	end
	
	--Streaming checking
	local status = checker.CheckStreaming()
	local ledStatus
	if status == "OFF" then
		ledStatus = "OFF"
	elseif status == "CONNECTING" then
		ledStatus = "GREEN:BLINK"
	elseif status == "CONNECTED" then
		ledStatus = "YELLOW:ON"
	elseif status == "PUSHING" then
		ledStatus = "GREEN:ON"
	elseif status == "STOPPED" then
		ledStatus = "YELLOW:BLINK"
	else
		ledStatus = "RED:ON"
	end
	ctrl_module.LedControl( "stream", ledStatus )

	--Audio source checking
	local source = checker.CheckAudioSource()
	if source == "DIGITAL" or source == "HDMI" or source == "SDI" then
		ledStatus = "GREEN:ON"
	elseif source == "LINE" or source == "ANALOG" or source == "MIC" then
		ledStatus = "YELLOW:ON"
	elseif source == "NONE" or source == "OFF" then
		ledStatus = "OFF"
	else
		ledStatus = "RED:ON"
	end
	ctrl_module.LedControl( "audio_source", ledStatus )
	
end

-----------

return _M
