local checker = require "statusChecker"

local _M = {}

--The G1 device IO/LEDs control parameters
_G.IOLED_OPTIONS = 
{
	WITH_FPGA = false,	--With FPGA
	FPGA_DEV_ADDR = 0xf4,	--FPGA I2C device address
	WITH_FPGA_LEDS	= true,	--With FPGA LEDs
	------
	RESET_PIN = 9,	--GPIO9 as RESET button
	------
	GPIO_LEDS = {
		--["signal"] = 58,	--GPIO signal LED
		["running"] = 81	--GPIO running LED
	},
	------
	--Rolling options: When pressed (but no arrived reset time) RESET button, rolling LEDs
	ROLLING_SEQ = { "running" },
	
	ROLLING_LANG_STATUS = {
		[0x01] = "RED:ON",		--Default language is CN
		[0x04] = "GREEN:ON",	--Default language is EN
		[0x00] = "YELLOW:ON"	--Other languages
	},
	
	ROLLING_MIN_NUMS = 1,	--How many leds at least show when rolling (Multiple languages support should +1)
	------
	ACTION_STATUS = {
		["default"] = {
			--["signal"] = "OFF",
			["running"] = "BLINK"
		},
	
		["restore"] = {
			--["signal"] = "OFF",
			["running"] = "OFF"
		},
	
		["reboot"] = {
			--["signal"] = "OFF",
			["running"] = "OFF"
		},
		
		["reset"] = {
			--["signal"] = "BLINK",
			["running"] = "BLINK"
		}
	},
	
	RESET_HOLD = 5000	--Hold RESET button more than this value(milliseconds), should do restore factory action!
}

local ctrl_module = nil

function _M.Check(period_ms)
	if not ctrl_module then
		ctrl_module = require( "controlModule" )
	end
	
	if not ctrl_module then
		return
	end
	
	if not checker then
		return
	end
	
	--Signal checking
	-- local signal = checker.CheckSignal()
	-- if signal then
	-- 	ctrl_module.LedControl( "signal", "ON" )
	-- else
	-- 	ctrl_module.LedControl( "signal", "OFF" )
	-- end
	
	--Running modules
	local modules = checker.CheckRunningModules()
	if not modules then
		ctrl_module.LedControl( "running", "OFF" )
	else
		if modules["AVIO"] and modules["CODEC"] and modules["STREAMER"] then
			if modules["WEB"] then
				ctrl_module.LedControl( "running", "BLINK-FAST" )
			else
				ctrl_module.LedControl( "running", "BLINK" )
			end
		else
			ctrl_module.LedControl( "running", "ON" )
		end
	end

	if ctrl_module.userControl then return end

end

-----------

return _M
