local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

local config = require "arp_config"

local _M = {
	last_arp_table = nil
}

function _M.AddARP(ip,mac)
	if ip and mac then
		os.execute("/usr/sbin/arp -s " .. ip .. " " .. mac )
		return true
	else
		_dbg.Err("ARP","AddARP: Invalid IP/MAC")
		return false
	end
end

function _M.RemoveARP(ip)
	if ip then
		os.execute("/usr/sbin/arp -d " .. ip)
		return true
	else
		_dbg.Err("ARP","RemoveARP: Invalid IP/MAC")
		return false
	end
end

function _M.Update(arp_table)
	if type(arp_table) ~= "table" then
		return false
	end
	local previous_arp_table = _M.last_arp_table or {}
	local new = {}
	local drop = {}
	for ip,mac in pairs(arp_table) do
		if not previous_arp_table[ip] or previous_arp_table[ip] ~= mac then
			new[ip] = mac
		end
	end
	for ip,mac in pairs(previous_arp_table) do
		if not arp_table[ip] then
			drop[ip] = mac
		end
	end
	for ip, mac in pairs(drop) do
		_M.RemoveARP(ip)
	end
	for ip, mac in pairs(new) do
		_M.AddARP(ip, mac)
	end

	_M.last_arp_table = arp_table
end


_G.arp_config = config.Load()

function _M.Init()
	_M.Update(_G.arp_config)
end

function _M.GetList()
    return _G.arp_config
end

function _M.DeleteOne(ip)
	if _M.RemoveARP(ip) then
		_G.arp_config[ip] = nil
		_G.arp_config:__TRIGGER(true)
		return true
	end
	return false
end

function _M.AddOne(ip, mac)
    if type(ip) ~= "string" or type(mac) ~= "string" then
        return false, "参数类型有误！"
    end

    if #mac > 17 or #ip > 15 then
        return false, "非法的参数长度！"
    end

    if not ip:match("%d+%.%d+%.%d+%.%d+") then
        return false, "无效的ip地址！"
    end

    if not mac:match("%w%w:%w%w:%w%w:%w%w:%w%w:%w%w") then
        return false, "无效的mac地址！"
    end

    for i=1, #mac do
        if mac:sub(i, i) ~= ":" and not tonumber(mac:sub(i, i), 16) then
            return false, "无效的mac字符！"
        end
    end

    if _G.arp_config[ip] then
        return false, "重复的ip地址！"
	end

	if _M.AddARP(ip, mac) then
		_G.arp_config[ip] = mac
		_G.arp_config:__TRIGGER(true)	
		return true
	end
	return false
end

return _M
