#!/usr/bin/env lua
--The hostapd service monitor, for service a controlable WIFI sharing.
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

local nm = require "mod.networkManager"
local nu = require "kiloview.netUtil"
local getset = require "kiloview.getSetHelper"
local dhcp_service = require "dhcp_service"
local rndis_if = require "rndis_if"
local periphery = require( "periphery" )
require "features"

require "hostap_config"
require "configuredb"

local pendingTryFirstWifiInit = true
---------------------------------------------------
--NOTE: Must be global here.
_G.hostapm = CONFIGS.InstanceOf( HOSTAPM_Template, "hostapm" )

---------------------------------------------------

-- gpio11是专门为G系列产品控制wifi模块而修改的 目前只有G1-s使用到了 1是拉高 0是拉低
local function initGPIO11Status()
	if _G.OEM_TYPE == "G1-s" or _G.OEM_TYPE == "G1" or _G.OEM_TYPE == "G2" then
		local gpio11 = periphery.GPIO(11,"out")
		if gpio11 then
			gpio11:write(1)
			gpio11 = nil
		end
	end
end

local function checkGPIO11Status()
	if _G.OEM_TYPE == "G1-s" or _G.OEM_TYPE == "G1" or _G.OEM_TYPE == "G2" then
		local gpio11 = periphery.GPIO(11,"out")
		if gpio11 then
			gpio11:write(0)
			gpio11 = nil
		end
	end
end

function natExisted(iprange)
	-- print("iprange", iprange)
	local file = io.popen( "/usr/sbin/iptables -L POSTROUTING -t nat -n", "r" )
	if file then
		line = file:read("l")
		while line ~= nil do
			-- print(line)
			if line:match(iprange) then
				file:close()
				return true
			end
			line = file:read("l")
		end
		file:close()
		return false
	else
		return false
	end
end

function g__UpdateLocalLinkNAT()
	local req_ip = hostapm.localLink.ip
	local req_mask = hostapm.localLink.netmask

	local r1,r2,r3,r4 = req_ip:match( "(%d+)%.(%d+)%.(%d+).(%d+)" )
	if r1 and r2 and r3 and r4 then
		if nu.ip4MaskToPrefix(req_mask) == 24 then
			local ipRange = r1 .. "." .. r2 .. "." .. r3 .. ".0/24"
			if not natExisted(ipRange) then
				os.execute( "/usr/sbin/iptables -t nat -A POSTROUTING -s " .. ipRange .. " -j MASQUERADE" ) --Add NAT rule.
			end
		else
			print( "WARNING: MASK not 24!" )
		end
	else
		print( "WARNING: Invalid LocalLink IP" )
	end
end

local HOSTAPM_SVC = {
	device = "",
	deviceType = "wifi",
	enable = false,
	hostAp = "",
	shareKey = "12345678",
	shareDevice = "",
	ipPrefix = 24,
	ipRange = "192.168.250.",
	wifiChannel = 6,
	hideSsid = 0,
	started = false,
	__init = false,

	Identifier = "HOSTAMP",
	FriendlyName = "HostAP Monitor",
	Version = "2.0",
	ConfigPrefix = "hostapm",
}


function HOSTAPM_SVC:Activate()
	--Nothing
end

function HOSTAPM_SVC:Start()
end

function HOSTAPM_SVC:Stop()
end

function HOSTAPM_SVC:Reset()
--TODO: Nothing
end

function HOSTAPM_SVC:Terminate()
	os.exit(0)
end

function HOSTAPM_SVC:Control(action,params)
	if action == "activate" then
		self:Activate()
		return 0, {}
	elseif action == "start" then
		self:Start()
		return 0, {}
	elseif action == "stop" then
		self:Stop()
		return 0, {}
	elseif action == "Reset" then
		self:Reset()
		return 0, {}
	elseif action == "terminate" then
		self:Terminate()
		return 0, {}
	elseif action == "update-alter-modems" then
		g__UpdateAlterModems()
		return 0, {}
	else
		return -1, {}
	end
end

function HOSTAPM_SVC:Set(params)
	local cnt = getset.SetConfigureValues( self.ConfigPrefix, params )
	return cnt
end

function HOSTAPM_SVC:SetEnv(params)
	local cnt = getset.SetValues( "_ENV", params )
	return cnt
end

function HOSTAPM_SVC:Get(inParams)
	local cnt = getset.GetValues( self.ConfigPrefix, inParams )
	return cnt,inParams
end

function HOSTAPM_SVC:GetEnv(inParams)
	local cnt = getset.GetValues( "_ENV", inParams )
	return cnt,inParams
end

function HOSTAPM_SVC:RestoreDefault()
	--TODO:
	return 0
end

function HOSTAPM_SVC:LoadConfigureFile(filename)
	CONFIGS.BeginUpdate()
	local r = pcall( dofile, filename )
	CONFIGS.Submit()
	if r then
		return 1
	else
		return 0
	end
end

--------------------------------------------------

-- Update the NetworkManager.conf for unmanagement device.
function HOSTAPM_SVC:_UpdateNMConfig(enableDevice, specDevice)
	specDevice = specDevice or self.device
	if enableDevice then
		if not specDevice or specDevice == "" then
			print( "ERROR: Invalid device, so could not enable the device when updating NetworkManager configure." )
			return false
		end
	end

	if specDevice:match("wlan0") then
		print("INFO: Not change the wlan 0")
		return true
	end

	local nmcfg = io.open( "/etc/NetworkManager/NetworkManager.conf", "r" )
	if nil == nmcfg then
		print( "ERROR: Could not open the NetworkManager configure file" )
		return false
	end

	local wholefile = ""
	local fnd_keyfile = false
	--local fnd_kiloview_instru = false
	local fnd_unmanaged_devices = false

	local line = nmcfg:read("l")
	while line ~= nil do
		if wholefile ~= "" then wholefile = wholefile .. "\n" end

		if line:match("^%s-#") then	--Is comment
			wholefile = wholefile .. line
			--if fnd_keyfile and line:match("~kiloview%-instruction:") then
			--	fnd_kiloview_instru = true
			--end
		elseif line:match("%[keyfile%]") then
			wholefile = wholefile .. line
			fnd_keyfile = true
			--fnd_kiloview_instru = false
		elseif line:match("%[.-%]") then	--New section
			wholefile = wholefile .. line
			fnd_keyfile = false
			--fnd_kiloview_instru = false
		elseif line:match("unmanaged%-devices=") then
			--print( fnd_keyfile, fnd_kiloview_instru )
			--if fnd_keyfile and fnd_kiloview_instru then
			if fnd_keyfile then
				line = line:gsub("unmanaged%-devices=","")
				line = line .. ";"
				local allUmDevices = {}
				local item
				local alreadyExist = false
				for item in line:gmatch("(.-)[;,]") do
					if item ~= "" and not item:match("wlan0") then
						if enableDevice and item == ("interface-name:"..specDevice) then
							alreadyExist = true
							table.insert( allUmDevices, item )
						elseif not enableDevice and item == ("interface-name:"..specDevice) then
							; --Nothing to do (drop it)
						elseif not enableDevice and specDevice:match("[%.%?%*]") and item:match("interface%-name:"..specDevice) then
							; --Nothing to do (drop it)
						else
							table.insert( allUmDevices, item )
						end
					end
				end

				if enableDevice and not alreadyExist then
					table.insert( allUmDevices, "interface-name:" .. specDevice )
				end

				wholefile = wholefile .. "unmanaged-devices=" .. table.concat( allUmDevices, ";" )

				fnd_unmanaged_devices = true
			else
				wholefile = wholefile .. line
			end
		else
			wholefile = wholefile .. line
		end

		line = nmcfg:read("l")
	end

	nmcfg:close()

	if not fnd_unmanaged_devices and enableDevice then
		wholefile = wholefile .. "\n[keyfile]\nunmanaged-devices=interface-name:" .. specDevice .. "\n\n"
	end	

	--print( wholefile )
	nmcfg = io.open( "/etc/NetworkManager/NetworkManager.conf", "w" )
	if not nmcfg then
		print( "ERROR: Could not update the NetworkManager configure file!" )
		return false
	end

	nmcfg:write(wholefile)
	nmcfg:close()
	os.execute( "/bin/sync" ) --Don't loss the cache.
	
	print( "NetworkManager configure file updated for unmanage device '" .. specDevice .. "'" )
	return true
end

--Internal start the hostapd service.
function HOSTAPM_SVC:_InternalStart()
	--Step1: Find the device in NetworkManager, and unmanage it.
	--self:_UpdateNMConfig(true)
	--Step2: Wait it be unmanaged.
	local cc=0
	local unmanaged = false
	while cc < 180 do
		local devInfo = nm.GetDevicesBy( self.device )
		if devInfo and devInfo[1] then
			if devInfo[1].State == 10 then
				unmanaged = true
				break
			end
		end

		coaf.Sleep(0.1)
		cc = cc + 1
	end

	if not unmanaged then
		print( "ERROR: Could not unmanage the device before start the sharing." )
		return false
	else
		print( "The device '" .. self.device .. "' is unmanged." )
	end

	--Step3: Updating the /etc/hostapd/hostapd.conf (if WIFI type)
	if self.deviceType == "wifi" then
		print( "Configuring the hostapd..." )

		os.execute( "/usr/bin/killall hostapd" )	--Kill it first.
		--os.execute( [[ccc=0; while [ "`pidof hostapd`" != "" -a $ccc -lt 20 ]; do usleep 100000; ccc=`expr ccc + 1`; done]] )
		os.execute( "/bin/sleep 1" )
		os.execute( "/bin/mkdir -p /etc/hostapd" )
		local cfg = io.open( "/etc/hostapd/hostapd.conf", "w" )
		if not cfg then
			print( "ERROR: Could not configure the hostapd!" )
			return false
		end

		local command = "/usr/sbin/iw dev wlan1 info | /bin/grep channel | /usr/bin/awk '{print$2}'"
        local file_channel = io.popen(command)
        local wifi_Channel = file_channel:read("*a")
        if wifi_Channel ~= "" then
                self.wifiChannel = tonumber(wifi_Channel)
        end

		local hw_mode = 'g'
		if tonumber(self.wifiChannel) > 14 then
			hw_mode = 'a'
		end

		cfg:write( "ctrl_interface=/var/run/hostapd\n" )
		cfg:write( "ctrl_interface_group=1\n" )
		cfg:write( "interface=" .. self.device .. "\n" )
		cfg:write( "driver=nl80211\n" )
		cfg:write( "ssid=" .. self.hostAp .. "\n" )
		cfg:write( "channel=" .. self.wifiChannel .. "\n" )
		cfg:write( "hw_mode=" .. hw_mode .. "\n" )
		cfg:write( "ignore_broadcast_ssid=" .. self.hideSsid .. "\n" )
		cfg:write( "auth_algs=1\n" )
		cfg:write( "wpa=3\n" )
		cfg:write( "wpa_passphrase=" .. self.shareKey .. "\n" )
		cfg:write( "wpa_key_mgmt=WPA-PSK\n" )
		cfg:write( "wpa_pairwise=TKIP\n" )
		cfg:write( "rsn_pairwise=CCMP\n" )

		cfg:close()

		--Step4: Start the hostapd service
		os.execute( "/usr/sbin/hostapd -B /etc/hostapd/hostapd.conf" ) --Start the hostapd.
		print( "hostapd started." )
	end

	--Step4: Configure it's IP address.
	local mask = nu.ip4PrefixToMask( self.ipPrefix )
	local addr = self.ipRange .. "254"	-- x.x.x.254
	os.execute( "/sbin/ifconfig " .. self.device .. " " .. addr .. " netmask " .. mask .. " up" )
	print( "IP address configured." )

	--Step5: Start DNSMASQ for DHCP service.
	--Bug fixed: If no this directory, DHCP service should not start.
	--os.execute("/bin/mkdir -p /var/lib/misc")
	
	dhcp_service.StartDHCPService(self.device, addr, 24, {
		dynamic_start = self.ipRange .. "10",
		dynamic_end = self.ipRange .. "250",
		gw = addr,
		dns = {addr, "8.8.8.8"}
	})

	--[[
	local cfg = io.open( "/etc/dnsmasq.conf", "w" )
	if not cfg then
		print( "WARNING! Could not open DNSMASQ configure file, so the DHCP service may not correct!" )
	else
		cfg:write( "listen-address=" .. addr .. ",127.0.0.1\n" )
		cfg:write( "dhcp-range=" .. self.ipRange .. "10," .. self.ipRange .. "250,24h\n" )
		cfg:write( "dhcp-option=3," .. addr .. "\n" )	--3 is the gateway.
		cfg:write( "dhcp-option=6," .. addr .. ",8.8.8.8\n" )	--6 is the DNS.
		cfg:close()

		os.execute( "/usr/bin/killall dnsmasq" )	--Kill it first.
		os.execute( "/usr/sbin/dnsmasq" )	--Restart it for updating.

		print( "DHCP/DNS service started DONE." )
	end
	--]]

	--Step6: Enable the NAT
	os.execute( "/sbin/sysctl -w net.ipv4.ip_forward=1" )
	os.execute( "/usr/sbin/iptables -t nat -D POSTROUTING 1; while [ $? -eq 0 ]; do /usr/sbin/iptables -t nat -D POSTROUTING 1; done" )		--Remove all POSTROUTING rules first.
	os.execute( "/usr/sbin/iptables -t nat -A POSTROUTING -s " .. self.ipRange .. "0/" .. self.ipPrefix .. " -j MASQUERADE" ) --Add NAT rule.
	g__UpdateLocalLinkNAT()
	print( "Open network sharing DONE. [" .. self.device .. " -> ANYWHERE, with DHCP/DNS and NAT]" )
	self.started = true
	return true
end

--Init
function HOSTAPM_SVC:Init()
	if self.__init then return; end

	initGPIO11Status()

	hostapm:LoadFile( "/data/configs/hostapm.conf" )
	hostapm:ResetChanges()
	--hostapm:SaveFile()

	self.device = hostapm.share.device
	if self.device == "wlan0" then
		hostapm.share.device = "wlan1"
		self.device = "wlan1"
		hostapm:SaveFile()
	end
	self.deviceType = hostapm.share.deviceType
	self.enable = hostapm.share.enable and hostapm.share.enable == "true"
	self.hostAp = hostapm.share.hostAp
	self.shareKey = hostapm.share.shareKey
	self.shareDevice = hostapm.share.shareDevice
	self.ipPrefix = hostapm.share.ipPrefix
	self.ipRange = hostapm.share.ipRange
	self.wifiChannel = hostapm.share.wifiChannel
	self.hideSsid = hostapm.share.hideSsid or 0
	
	local r1,r2,r3 = self.ipRange:match( "(%d+)%.(%d+)%.(%d+)" )
	if r1 and r2 and r3 then
		self.ipRange = r1 .. "." .. r2 .. "." .. r3 .. "."
		if not self.ipPrefix or self.ipPrefix < 1 or self.ipPrefix > 24 then
			self.ipPrefix = 24
		end
	else
		print( "WARNING: Invalid IP range specified, assume it to be 192.168.250.0/24" )
		self.ipRange = "192.168.250."
		self.ipPrefix = 24
	end

	self.__init = true

	if _G.OEM_TYPE == "G1-s" or _G.OEM_TYPE == "G1"  or _G.OEM_TYPE == "G2" then
		coaf.NewTimer(checkGPIO11Status,5,60):start()
	end
	
	if self.enable then
		self:_UpdateNMConfig(true)
		if self.device and self.device ~= "" and self.shareDevice and self.shareDevice ~= "" then	--Try start the hostapd
			coaf.ScheduleDelayedTask( function()  
				self:_InternalStart()
			end, 10 )
			--self:_InternalStart()
		end
	else
		self:StopShare()
	end
	
	g__UpdateLocalLink()
	g__UpdateWifiSpec()	--Update wifi special settings (e.g. country code)
	--g__UpdateAlterModems()
end

function HOSTAPM_SVC:get_Device()
	return self.device
end

function HOSTAPM_SVC:get_Enable()
	return self.enable
end

function HOSTAPM_SVC:get_Started()
	return self.started
end

function HOSTAPM_SVC:get_SharedAddress()
	return self.ipRange .. "254"
end

function HOSTAPM_SVC:get_ShareAddressRange()
	return self.ipRange .. "10 - " .. self.ipRange .. "250"
end

function HOSTAPM_SVC:get_ShareIPRange()
	return self.ipRange
end

function HOSTAPM_SVC:get_HostAP()
	return self.hostAp
end

function HOSTAPM_SVC:get_HideSSID()
	return self.hideSsid
end

function HOSTAPM_SVC:get_ShareKey()
	return self.shareKey
end

function HOSTAPM_SVC:get_WifiChannel()
	return self.wifiChannel
end

function HOSTAPM_SVC:get_ShareDevice()
	return self.shareDevice
end

function HOSTAPM_SVC:StartShare(device, deviceType, shareDevice, hostAp, shareKey, ipRange, ipPrefix, wifiChannel,hideSsid)
	local changed = false

	--print( device, deviceType, shareDevice, hostAp, shareKey, ipRange, ipPrefix )

	if not self.__init then self.Init() end

	local prev_device = self.device

	if nil ~= device and device ~= self.device then
		self.device = device
		hostapm.share.device = device
		changed = true
	end

	if nil ~= deviceType and deviceType ~= self.deviceType then
		self.deviceType = deviceType
		hostapm.share.deviceType = deviceType
		changed = true
	end

	if nil ~= shareDevice and shareDevice ~= self.shareDevice and shareDevice ~= "" then
		self.shareDevice = shareDevice
		hostapm.share.shareDevice = shareDevice
		changed = true
	end

	if nil ~= hostAp and hostAp ~= self.hostAp then
		self.hostAp = hostAp
		hostapm.share.hostAp = hostAp
		changed = true
	end

	if nil ~= shareKey and shareKey ~= self.shareKey then
		self.shareKey = shareKey
		hostapm.share.shareKey = shareKey
		changed = true
	end

	if nil ~= ipRange and ipRange ~= self.ipRange then
		local r1,r2,r3 = ipRange:match( "(%d+)%.(%d+)%.(%d+)" )
		if r1 and r2 and r3 then
			ipRange = r1 .. "." .. r2 .. "." .. r3 .. "."
		else
			print( "WARNING: Invalid IP range specified, assume it to be 192.168.250.0/24" )
			ipRange = "192.168.250."
		end

		self.ipRange = ipRange
		hostapm.share.ipRange = ipRange
		changed = true
	end

	if nil ~= ipPrefix and ipPrefix ~= self.ipPrefix then
		if not ipPrefix or ipPrefix < 1 or ipPrefix > 24 then
			ipPrefix = 24
		end

		self.ipPrefix = ipPrefix
		hostapm.share.ipPrefix = ipPrefix
		changed = true
	end

	if nil ~= wifiChannel and wifiChannel ~= self.wifiChannel then
		self.wifiChannel = wifiChannel
		hostapm.share.wifiChannel = wifiChannel
		changed = true
	end

	if nil ~= hideSsid and hideSsid ~= self.hideSsid then
		self.hideSsid = hideSsid
		hostapm.share.hideSsid = hideSsid
		changed = true
	end
	
	if not self.enable then
		self.enable = true
		hostapm.share.enable = "true"
		changed = true
	end

	if changed then
		print( "Stopping previous shared..." )
		self:_InternalStop()
		self:_UpdateNMConfig(false,prev_device)

		print( "Updating the hostapd monitor parameters..." )
		hostapm:SaveFile()
		hostapm:ResetChanges()
	end

	if not self.started then
		self:_UpdateNMConfig(true)
		if self.device and self.device ~= "" and self.shareDevice and self.shareDevice ~= "" then	--Try start the hostapd
			self:_InternalStart()
		end
	end

	return self.started
end

function HOSTAPM_SVC:_InternalStop()
	if self.started then
		--Step1: Stop NAT.
		os.execute( "/usr/sbin/iptables -t nat -D POSTROUTING 1; while [ $? -eq 0 ]; do /usr/sbin/iptables -t nat -D POSTROUTING 1; done" )		--Remove all POSTROUTING rules
		--Step2: Stop DNS/DHCP service.
		os.execute( "/usr/bin/killall dnsmasq" )

		--Step3: Stop hostapd
		if self.deviceType == "wifi" then
			os.execute( "/usr/bin/killall hostapd" )
		end

		--Change the WIFI device to be managed.
		self.started = false
	end
end

function HOSTAPM_SVC:StopShare()
	if not self.__init then return false; end

	local changed = false
	if self.enable then
		self.enable = false
		hostapm.share.enable = "false"
		changed = true
	end

	if changed then
		print( "Updating the hostapd monitor parameters..." )
		hostapm:SaveFile()
		hostapm:ResetChanges()
	end

	self:_InternalStop()
	self:_UpdateNMConfig(false)
	return true
end

----------------------------------------------------
local function ParseLink( dev )
	local file, line
	file = io.popen( "/sbin/ifconfig " .. dev, "r" )
	if file then
		line = file:read("l")
		if line and line:match(dev) then
			line = file:read("l") --Next line, should be ip-address configures.
			local ip,mask = line:match( "inet addr:([0-9%.]+)%s+Bcast:[0-9%.]+%s+Mask:([0-9%.]+)" )
			file:close()
			if ip and mask then
				return ip,mask
			else
				return nil
			end
		else
			file:close()
			return nil
		end
	else
		return nil
	end
end

----------------------------------------------------
local function CheckRealtekDevices(filter)
	filter = filter or "wlan"
	local rslt = {}
	
	local lfs = require "lfs"
	for item in lfs.dir("/proc/net") do
		if item:match("rtl8.*") then
			for ff in lfs.dir("/proc/net/" .. item) do
				local attr = lfs.attributes( "/proc/net/" .. item .. "/" .. ff)
				if ff:match(filter) and attr and attr.mode == "directory" then
					table.insert(rslt, "/proc/net/" .. item .. "/" .. ff)
				end
			end
		end
	end
	return rslt
end

local compatibleWlanDevices = {
	{ "0cf3", "9378" }
}

local function CheckCompatibleWlanDevices(filter)
	filter = filter or "wlan"
	local rslt = {}
	
	local f = io.popen( "/bin/ls -d /sys/class/net/" .. filter .. "*", "r" )
	if not f then
		return nil
	end
	
	local vid, pid
	local item = f:read("l")
	while item do
		vid = nil
		pid = nil
		local vf = io.open( item .. "/device/idVendor", "r" )
		local pf = io.open( item .. "/device/idProduct", "r" )
		if vf then
			vid = vf:read("l")
			vf:close()
		end

		if pf then
			pid = pf:read("l")
			pf:close()
		end

		if vid and pid then
			local i
			for i=1, #compatibleWlanDevices do
				if vid:match( compatibleWlanDevices[i][1] ) and pid:match( compatibleWlanDevices[i][2] ) then
					table.insert( rslt, item:match(".+/(.*)") )
				end
			end
		end

		item = f:read("l")
	end
	
	f:close()
	return rslt
end

----------------------------------------------------
-- To update Wi-Fi special configure (loca
----------------------------------------------------

function g__UpdateWifiSpec()
	local devs = CheckRealtekDevices("wlan")
	if not devs or #devs < 1 then
		return
		--[[
		devs = CheckCompatibleWlanDevices("wlan")
		if not devs or #devs < 1 or not pendingTryFirstWifiInit then
			return
		end
		pendingTryFirstWifiInit = false
		local maxWlanNum = 0
		local i
		for i=1, #devs do
			local num = devs[i]:match(".-([0-9]+)")
			if num and tonumber(num) > maxWlanNum then
				maxWlanNum = tonumber(num)
			end
		end
		for i=1, #devs do
			os.execute( "/usr/sbin/iw dev " .. devs[i] .. " interface add wlan" .. (maxWlanNum+1) .. " type managed" )
			maxWlanNum = maxWlanNum + 1
		end
		]]
	else
		local i
		local file
		local pattern
		local driver
		local country_code = hostapm.wifiSpecConfig.country_code or "0x0c"
		for i=1, #devs do
			local skipDo = false
			file = io.open( devs[i] .. "/chan_plan".."", "r" )
			pattern = "/proc/net/(.-)/wlan%d"
            driver = string.match(devs[i], pattern)
			if driver == 'rtl88x2cu' then
				if file then
					file:close()
					local file_chplan = io.popen("/bin/cat " .. devs[i] .. "/chan_plan".."| /bin/grep chplan | /bin/grep -v /bin/grep | /usr/bin/awk -F ':' '{printf$2}'")
					local code = file_chplan:read("*a")
					if code then 
						if tonumber(code) == tonumber(country_code) then
							skipDo = true
						--else
							--country_code = code
						end
					else
						skipDo = true
					end
				else
					return
				end
			else
				if file then
					local line = file:read("l")
					file:close()
					local _nouse,code = line:match("(.-):(.+)$")
					if code then
						if tonumber(code) == tonumber(country_code) then
							skipDo = true
						end
					else
						skipDo = true
					end
				else
					return
				end
			end

			if not skipDo then
				file = io.open( devs[i] .. "/chan_plan", "w" )
				if file then
					print( "Update chan_plan of " .. devs[i] .. ":" .. country_code )
					file:write( country_code )
					file:close()
				end
			end
--[[
			local roam_flag = hostapm.wifiSpecConfig.roaming == 1 and "0x07" or "0x03"
			file = io.open( devs[i] .. "/roam_flags", "w" )
			if file then
				print( "Set roaming flags of " .. devs[i] .. ":" .. roam_flag )
				file:write( roam_flag )
				file:close()
			end

			if roam_flag == "0x07" then --Started roaming, so increment the rescan interval!
				wifiCorun.rescanInterval = math.floor(hostapm.wifiSpecConfig.roaming_scan_intv / 1000)
				print( "Speed up wifi rescan interval to: ", wifiCorun.rescanInterval )
			else
				wifiCorun.rescanInterval = wifiCorun.DEFAULT_RESCAN_INTERVAL
			end

			local options = hostapm.wifiSpecConfig.roaming_rssi_thres .. " " .. hostapm.wifiSpecConfig.roaming_scan_intv .. " " .. hostapm.wifiSpecConfig.roaming_scan_intv
			file = io.open( devs[i] .. "/roam_param", "w" )
			if file then
				print( "Set roaming params of " .. devs[i] .. ":" .. options )
				file:write( options )
				file:close()
			end
]]
		end
		--pendingTryFirstWifiInit = false
		--NOTE: TODO: Let it always check.
	end
end

----------------------------------------------------
-- To update local link address (since NetworkManager may discard configuration)
----------------------------------------------------

function g__UpdateLocalLink()
	--Seek the local-link interface 'eth0:0' and update it's configurations.
	local device = hostapm.localLink.device
	local subdevice = device .. ":" .. hostapm.localLink.subdevice
	--1, Check the interface exists?
	local file
	local carrier = false
	file = io.open( "/sys/class/net/" .. device .. "/carrier", "r" )
	if file then
		local line = file:read("l")
		if line == "1" then
			carrier = true
		end
		file:close()
	end
	
	if not carrier then	--No carrier, no care.
		return
	end
	
	--2, If disabled, shutdown it.
		
	if hostapm.localLink.enable ~= "true" then --Disable it.
		--local ip,mask = ParseLink(subdevice)
		--if ip then
		--	os.execute( "/sbin/ifconfig " .. subdevice .. " down" )
		--end
		return
	end
	
	local ip,mask = ParseLink(subdevice)
	--3, if ip/mask not match, force change.
	local req_ip = hostapm.localLink.ip
	local req_mask = hostapm.localLink.netmask
	if nu.ipToU32(req_ip) == 0 then --Invalid ip, no changing.
		return
	end

	if nu.ipToU32(req_mask) == 0 then --Invalid mask, make it as default
		req_mask = "255.255.255.0"
	end
	
	if ip ~= req_ip or mask ~= req_mask then
		--print( "***Update local-link: " .. req_ip .. "/" .. req_mask )
		os.execute( "/sbin/ifconfig " .. subdevice .. " " .. req_ip .. " netmask " .. req_mask .. " up 2>/dev/null" ) 
		g__UpdateLocalLinkNAT()
	end
end

----------------------------------------------------
local function write_usb_new_id(class, opts)
	--TODO: Check class.
	local file = io.open( "/sys/bus/usb-serial/drivers/option1/new_id", "w" )
	if file then
		print( "*** Update special usb modem:", opts )
		file:write(opts)
		file:close()
	end
end

function g__UpdateAlterModems()
	os.execute( "/sbin/modprobe option" )

	local file = io.open( "/data/configs/option-modems.conf", "r" )
	if file then
		local line = file:read("l")
		while line do
			if not line:match("^%s*#") then --not be comment
				local pid, vid = line:match("^%s*(%x+)%s+(%x+)")
				if pid and vid then
					write_usb_new_id( "usb-serial", line )
				end
			end
			line = file:read("l")
		end
		file:close()
	end
end

function g__UpdateBuiltinModems()
	os.execute( "/sbin/modprobe option" )

	local file = io.open( "/etc/option-modems.conf", "r" )
	if file then
		local line = file:read("l")
		while line do
			if not line:match("^%s*#") then --not be comment
				local pid, vid = line:match("^%s*(%x+)%s+(%x+)")
				if pid and vid then
					write_usb_new_id( "usb-serial", line )
				end
			end
			line = file:read("l")
		end
		file:close()
	end
end

local period_timer
----------------------------------------------------
local function periodCheck()
	rndis_if.Check()
	g__UpdateLocalLink()
	if pendingTryFirstWifiInit then
		g__UpdateWifiSpec()
	end
end
----------------------------------------------------
local function delayLoadModems()
	g__UpdateBuiltinModems()
	g__UpdateAlterModems()                                                                                               
end 

local function initA1ModemsInfo()
	write_usb_new_id( "usb-serial", "19d2 0536" )
end
----------------------------------------------------

local _M = {}

function _M.Init(svc)
	print( "Hostapd Monitor service running..." )
	os.execute( "/usr/bin/killall dnsmasq" )
	svc:ExposeObject( "/raw/HOSTAP", HOSTAPM_SVC )
	HOSTAPM_SVC:Init()

	period_timer = coaf.NewTimer( periodCheck, 0.01, 2 )	--After 0.01 second, and per 2 seconds check.
	if period_timer then
		period_timer:start()
	end

	if _G.OEM_TYPE == "A1_plus" then
		initA1ModemsInfo()
	end

	coaf.Sleep(5)
	delayLoadModems()
end

return _M

