#!/usr/bin/env lua
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local thread = require "coaf.thread"

-- Change current dir to app location
coaf.ChDir(coaf.GetAppDir(arg))

local socket = require "cosocket.core"

socket.default_close_on_exec = true

--local client = require "coaf.client"
--local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"

local roll = require "coaf.log.rollfile"

local withLog = false
local _x
for _x=1, #arg do
	if arg[_x] == "--log" then
		withLog = true
		break
	end
end

if withLog then
	-- ** initialize the log file **
	local roll = require "coaf.log.rollfile"
	os.execute( "/bin/mkdir -p /data/log/" )
	local logF = roll.Create("/data/log/nm_daemon.log")
	if not logF or not logF:init() then
		_dbg.Error("NM_DAEMON", "Fail to open log file: /data/log/nm_daemon.log, so the log messages should be output to console.")
		withLog = false
	else
		_dbg.AddLogger("logfile", logF, nil, nil)
	end
	-- ** log **
end

coaf.Init( "nm_daemon" )

_G.print = function(...)
	_dbg.Info1( "NM_DAEMON", ... )
end

if withLog then
	local stdio = require "coaf.log.stdio"
	stdio.SetDefaultFilterName( "NM_DAEMON" )
	stdio.Start()
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-- CoAF setup above.
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

-- DBus client setup.
local dbusCli = require "kiloview.dbusClients"
dbusCli.Init( "SYSTEM" )

-- CoAF serivce initialization.
service.InitSocket()
--client.InitSocket( socket, adapter )

----------------------------------------

local NM = require "mod.networkManager"
local WIFI = require "mod.wifiManager"
local MODEM = require "mod.modemManager"
local NET_STATUS = require "net_status"
local PORT_FWD = require "port_fwd"
local AVAHI = require "mod.avahiManager"
local arp = require "arp"
local static = require "nm_static"

local hostap_m
local cosvc_NM
local period_timer
local nm_period_timer
----------------------------------------
local function periodTask()
	NET_STATUS.Check() --Check all interfaces
end

local function nmperiodTask()
	NM.Steps()
end
----------------------------------------
local function Main()
	NM.Init()
	WIFI.Init()
	NET_STATUS.Init()
	PORT_FWD.Init()
	AVAHI.Init()
	arp.Init()
	static.Init()

	hostap_m = require "hostap_m"

	cosvc_NM = service.New( "nm_daemon", "unix:/var/run/nm_daemon.sock" )
	if cosvc_NM then
		_dbg.Info( "NM_DAEMON", "CoService of 'nm_daemon' is created." )
		cosvc_NM:CreateCoServerProxy()
		--NOTE: These are the raw methods for network management accessing.
		--      This is not suggested use in future.
		cosvc_NM:ExposeObject( "/raw/NETWORK_MANAGER", NM )
		cosvc_NM:ExposeObject( "/raw/WIFI", WIFI )
		cosvc_NM:ExposeObject( "/raw/MODEM", MODEM )
		cosvc_NM:ExposeObject( "/NET_STATUS", NET_STATUS )
		cosvc_NM:ExposeObject( "/PORT_FWD", PORT_FWD )
		cosvc_NM:ExposeObject( "/AVAHI", AVAHI)
		cosvc_NM:ExposeObject( "/ARP", arp )
		cosvc_NM:ExposeObject( "/STATIC", static )
	else
		_dbg.Error( "NM_DAEMON", "Fail created CoService!" )
	end

	--First update scan.
	WIFI.UpdateScan()

	period_timer = thread.NewTimer( periodTask, 0.01, 1 )
	if period_timer then
		period_timer:start()
	end

	nm_period_timer = thread.NewTimer( nmperiodTask, 5, 5 )
	if nm_period_timer then
		nm_period_timer:start()
	end

	hostap_m.Init( cosvc_NM )
	AVAHI.start()
end

----------------------------------------

coaf.Start(Main)

if AVAHI then
	AVAHI:stop()
end
PORT_FWD.Destroy()

_dbg.Info( "NM_DAEMON", "Network daemon service is terminated." )

