local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local dhcp_service = require "dhcp_service"
local nu = require "kiloview.netUtil"
require "SimpleNetHelper"

local _M = {
	already_configured = nil,
	dont_too_fast_count = 0,
	force_update = true
}

-- The the first line from a specified file.
local function file_line( filename )
	local f = io.open( filename, "r" )
	if f then
		local r = f:read("*l")
		f:close()
		return r
	else
		return nil
	end
end

local function check_rndis_interface()
	--local item
	local lfs = require "lfs"
	for item in lfs.dir("/sys/class/net") do
		if item:match("usb%d+") then
			local f = io.popen("/usr/bin/readlink -f /sys/class/net/" .. item, "r" )
			if f then
				local path = f:read("*l")
				f:close()
				if path and path:match(".*/.*%.udc/gadget/.*") then
					return item
				end
			end
		end
	end
	return nil
end


function _M.Check()
	if _M.dont_too_fast_count == 0 then
		local rndis_if = check_rndis_interface()
		if _M.force_update or rndis_if ~= _M.already_configured then
			_M.force_update = false
			if _M.already_configured then
				dhcp_service.StopDHCPService(_M.already_configured)
				os.execute("/sbin/ifconfig " .. _M.already_configured .. " down")
				_M.already_configured = nil
			end

			if rndis_if and _G.hostapm.rndisLink.enable == "true" then
				local ip = _G.hostapm.rndisLink.ip
				local mask = _G.hostapm.rndisLink.netmask
				if nu.ipToU32(ip) == 0 then --Invalid ip
					ip = "10.10.10.10"
				end
				if nu.ipToU32(mask) == 0 then --Invalid ip
					mask = "255.255.255.0"
				end

				local prefix = nu.ip4MaskToPrefix(mask)
				if not prefix or prefix < 1 or prefix > 31 then
					prefix = 24
					mask = "255.255.255.0"
				end

				local tok1, tok2 = ip:match("([%d%.]+)%.(%d+)$")
				local dh_start
				local dh_end

				tok2 = tonumber(tok2) or 0
				if tok2 > 100 then
					dh_start = tok1 .. ".1"
					dh_end = tok1 .. ".100"
				else
					dh_start = tok1 .. ".101"
					dh_end = tok1 .. ".254"
				end

				os.execute("/sbin/ifconfig " .. rndis_if .. " " .. ip .. " netmask " .. mask .. " up")
				dhcp_service.StartDHCPService(rndis_if, ip, prefix, {
					dynamic_start = dh_start,
					dynamic_end = dh_end
				})
				_M.already_configured = rndis_if
			end
			
		end
	end

	_M.dont_too_fast_count = _M.dont_too_fast_count + 1
	if _M.dont_too_fast_count > 2 then
		_M.dont_too_fast_count = 0
	end
end

function _M.UpdateRndisLink()
	_M.dont_too_fast_count = 0
	_M.force_update = true
end

return _M