--[==[

The Co-Socket for CoAF (Co-Application Framework)

The cosocket module is similar to lua-socket, but, it works
asynchronously. If you are calling cosocket APIs in a 
coroutine context and called API may be blocked, it 
should yield current coroutine, util unblocked or
timeouted, it returns.

The cosocket module depends 'lua-ev' module.

Copyright (C) 2018, KILOVIEW Electronics, Co., Ltd.

]==]

local _M = {}

--Original lua socket.
local socket = require "socket"
local tcp_m = require "patch_socket.tcp"		--get TCP patch module.

-- No overrided functions/properties, use default.
function _M.__index(m, key)
	return socket[key]
end

-----------------------------------------------------------------------------
-- Fixed async methods.
-----------------------------------------------------------------------------

function _M.tcp()
	local tcp, err = socket.tcp()
	if not tcp then
		return tcp,err
	end
	return tcp_m.build_socket(tcp)
end

function _M.tcp6()
	local tcp, err = socket.tcp6()
	if not tcp then
		return tcp,err
	end
	return tcp_m.build_socket(tcp)
end

-----------------------------------------------------------------------------
-- Exported auxiliar functions (Copied from lua-socket and changed)
-----------------------------------------------------------------------------
function _M.connect(address, port, laddress, lport, family)
	--TODO: How to process laddress and lport ?

	local sock,err
	if family == "inet6" then
		sock,err = _M.tcp6()
	else
		sock,err = _M.tcp()
	end
	if not sock then return nil, err end
	local x
	x,err = sock:connect(address,port)
	if not x then
		sock:close()
		return nil, err
	else
		return sock
	end
end

function _M.connect4(address, port, laddress, lport)
    return _M.connect(address, port, laddress, lport, "inet")
end

function _M.connect6(address, port, laddress, lport)
    return _M.connect(address, port, laddress, lport, "inet6")
end

function _M.bind(host, port, backlog)
	if host == "*" then host = "0.0.0.0" end
	
	local addrinfo, err = socket.dns.getaddrinfo(host);
	
    if not addrinfo then return nil, err end
    local sock, res
    err = "no info on address"
    for i, alt in base.ipairs(addrinfo) do
        if alt.family == "inet" then
            sock, err = _M.tcp()
        else
            sock, err = _M.tcp6()
        end
        if not sock then return nil, err end
        sock:setoption("reuseaddr", true)
        res, err = sock:bind(alt.addr, port)
        if not res then 
            sock:close()
        else 
            res, err = sock:listen(backlog)
            if not res then 
                sock:close()
            else
                return sock
            end
        end 
    end
    return nil, err
end

return setmetatable(_M, _M)

