local busOpr = require "busOprs"
local timedatectl = require "kiloview.systemd-timedate"

local args = {...}
local request = args[1]

local function getTimeZoneInfo(location, zone)
	if location == "UTC" and zone == "0" then
		return "UTC"
	elseif location == "China" then
		return "Asia/Chongqing"
	end
	local file = io.popen("/usr/bin/timedatectl list-timezones | /bin/grep " .. location .. " | /usr/bin/head -1","r")
	local ret = nil
	if file then
		local line = file:read("a")
		if line and line ~= "" then
			ret = line:match("(%w+/?%w+/?%w+)")
		end
	end
	if not ret then
		file = io.popen("/usr/bin/find /usr/share/zoneinfo/ -name " .. location .. " | /usr/bin/head -1","r")
		if file then
			local line = file:read("a")
			if line and line ~= "" then
				ret = line:match("/usr/share/zoneinfo/(%w+/?%w+/?%w+)")
			end
		end
	end
	if file then
		file:close()
	end
	return ret
end

local function matchTimeZone(str)
	local rule = {
		"(%w+)StandardTime(%d+)",
		"(%w+)StandardTime(%-%d+)",
		"(%w+)StandardTimeMexico(%d+)",
		"(%w+)StandardTimeMexico(%-%d+)",
		".+(UTC).+[^%w+](%-%d+)$",
		".+(UTC).+[^%w+](%d+)$",
		"^(UTC)(%d+)$",
		"^(UTC)(%-%d+)$",
	}
	local i,v 
	for i,v in ipairs(rule) do
		local a,b = string.match(str,v)
		if a and b then
			return getTimeZoneInfo(a,b)
		end
	end
	return nil--speZoneArr[str]
end

local function GetCoAFProxy( objPath, service, loc, timeout )
	local client = require "coaf.client"
	local proxy
	local cli
	cli = client.New( service, loc or "local", nil, timeout or 3 ) --3 seconds timeout.
	if not cli then
		_dbg.Error( "WEB", "Fail to create connection to service: " .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "WEB", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end

local function setStatus(info)
	local proxy = GetCoAFProxy( "/SYS_STATUS", "web_daemon", nil, 5 )
	if proxy then
		proxy.SetTimeUpdateStatus(info)
		proxy:Destroy()
	end
end

if not request then
	error( "Invalid request argument passed!" )
end

if request.DateTimeType == tt__SetDateTimeType__NTP then
	timedatectl.SetNTP( "true" )	--No address so only start NTP service.
	setStatus("1")
else
	--TODO: Now ignored timezone settting.
	
	if request.UTCDateTime and request.UTCDateTime.Date and request.UTCDateTime.Time then
		-- local timeStr = request.UTCDateTime.Date.Year .. "-" .. 
		-- 				request.UTCDateTime.Date.Month .. "-" ..
		-- 				request.UTCDateTime.Date.Day .. " " ..
		-- 				request.UTCDateTime.Time.Hour .. ":" ..
		-- 				request.UTCDateTime.Time.Minute .. ":" ..
		-- 				request.UTCDateTime.Time.Second
		
		timedatectl.SetNTP( "false" )
		local zone_name = matchTimeZone(request.TimeZone.TZ)
		if zone_name then
			local done,err = timedatectl.SetTimezone(zone_name)
			if done then
				setStatus("1")
			else
				setStatus("-1")
			end
		elseif request.TimeZone.TZ == "UTC" or string.match(request.TimeZone.TZ,"(/)") then
			local done,err = timedatectl.SetTimezone( request.TimeZone.TZ )
			if done then
				setStatus("1")
			else
				setStatus("-1")
			end
		elseif string.match(request.TimeZone.TZ,"(GMT)") then
			setStatus("1")
		else
			setStatus("-1")
			-- timedatectl.SetTimezone( timeinfo["zone"] )	--Set it back.
		end

		local timeinfo = timedatectl.GetTimeDateInfo()
		local timediff = math.floor(timeinfo["zone_offset"] / 100) * 3600
		---指定日期的时间戳
		local timeStr = os.date("%Y-%m-%d %H:",os.time({day=request.UTCDateTime.Date.Day, 
			month=request.UTCDateTime.Date.Month, year=request.UTCDateTime.Date.Year,
			hour=request.UTCDateTime.Time.Hour}) + timediff)..request.UTCDateTime.Time.Minute..":"..request.UTCDateTime.Time.Second
		timedatectl.SetTime(timeStr)
	end
end

return SVC_OK

--[==[ *ABSTRACT* The 'request' arguments:
{
	DateTimeType: (!)
	(
		tt__SetDateTimeType__Manual: 0,
		tt__SetDateTimeType__NTP: 1,
	)
	DaylightSavings: (!)
	(
		xsd__boolean__false_: 0,
		xsd__boolean__true_: 1,
	)
	TimeZone: 
	{
		TZ: <char*> (!)
	}
	UTCDateTime: 
	{
		Time: (!)
		{
			Hour: <int> (!)
			Minute: <int> (!)
			Second: <int> (!)
		}
		Date: (!)
		{
			Year: <int> (!)
			Month: <int> (!)
			Day: <int> (!)
		}
	}
}
]==]

--[==[ *ABSTRACT* The 'response' arguments:
{
}
]==]


