local _M = {}
---------------------------------------------------
-- SIP CoAF bus service operations
---------------------------------------------------

local sip_svc_methods = {}
sip_svc_methods.__index = sip_svc_methods

function sip_svc_methods:get_ServiceType()
	return "SIP/CLIENT"
end

function sip_svc_methods:ServiceName()
	return "SIP client of stream " .. self.appId .. "-" .. self.streamId
end

function sip_svc_methods:get_Enabled()
	local thisConfig = _G.streamer.stream[self.cfgId].Dynamic_services[self.dynamicIndex].SIP
	if not thisConfig then
		return false
	else
		return _G.streamer.stream[self.cfgId].enabled ~= 0 and thisConfig.enabled ~= 0
	end
end

function sip_svc_methods:get_ServiceUri()
	local stream_sip = require "stream.sip"
	local thisConfig = _G.streamer.stream[self.cfgId].Dynamic_services[self.dynamicIndex].SIP
	--or _G.streamer.stream[self.cfgId].enabled == 0 or thisConfig.enabled == 0 
	if not thisConfig then
		return "offline"
	else
		if stream_sip.sip_ep_details.callId then
			if stream_sip.sip_ep_details.callee then
				return stream_sip.sip_ep_details.callee
			else
				return "{Incoming call}"
			end
		else
			return "offline"
		end
	end
end

function sip_svc_methods:Enable(en)
	local thisConfig = _G.streamer.stream[self.cfgId].Dynamic_services[self.dynamicIndex].SIP
	if not thisConfig then
		return false
	else
		CONFIGS.BeginUpdate()
		thisConfig.enabled = en and 1 or 0
		CONFIGS.Submit()
		return thisConfig.enabled ~= 0 and _G.streamer.stream[self.cfgId].enabled ~= 0
	end
end

function sip_svc_methods:Update()
	local streams = require "streams"
	streams.UpdateStreamService( self.streamId, "Dynamic_" .. string.format( "%02d", self.dynamicIndex ), self.dynamicIndex )
end

function sip_svc_methods:GetStatus()
	local stream_sip = require "stream.sip"
	if not stream_sip.sip_ep_details.callId then
		return 1, { ["stream.status"] = "offline" }
	else
		--TODO: More details.
		return 1, { ["stream.status"] = "online" }
	end
end

--------------------------------------------

function _M.ServiceMethods()
	return sip_svc_methods
end

return _M
