require "kmCore"
pcall( require, "sys_setting")
local _config = require "klnk_config"
local _thread = require "coaf.thread"
local _coaf = require "coaf.core"
local _client = require "coaf.client"
local _dbg = require "coaf.debug"
local _pl  = require "pl.pretty"

require "klnk_forward"

local _M = {
    forward = nil, --端口转发
    fwd_enabled = false,
    link_config  = {},
    media_source = {main = nil, sub = nil},
    ifname_state = {},
    klink_service = nil,
    klink_start = false,
    dynamic_bitrate = true, --默认启动动态调节码率
    thread_id  = nil,
    runing     = true,
    changed    = false,
    main_bitrate = 4000000, main_scale = 1.0,
    sub_bitrate = 2000000, sub_scale = 1.0
}

local function get_coaf_proxy( objPath, service, loc, timeout )
	local proxy
	local cli
	cli = _client.New( service, loc or "local", nil, timeout or 3 ) --3 seconds timeout.
	if not cli then
		_dbg.Error( "BOND", "Fail to create connection to service: " .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "BOND", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end
local function get_serial_number()
	if type(_G.KV) == "table" and type(_G.KV.SERIAL_HISTORIES) == "table" and #_G.KV.SERIAL_HISTORIES > 0 then
		return _G.KV.SERIAL_HISTORIES[ #_G.KV.SERIAL_HISTORIES ].serial
	else
		return "0000000000"
	end
end

local function init_ifname()
	os.execute("/bin/echo 1 > /proc/sys/net/ipv4/ip_forward")
	os.execute("/bin/echo 0 > /proc/sys/net/ipv4/conf/all/rp_filter")
	-- os.execute("/bin/echo 0 > /proc/sys/net/ipv4/conf/wwan0/rp_filter")
    -- os.execute("/bin/echo 0 > /proc/sys/net/ipv4/conf/wwan1/rp_filter")
end

local function init_wwanx_ifname(ifname)
    os.execute("/bin/echo 0 > /proc/sys/net/ipv4/conf/"..ifname.."/rp_filter")
end
local function update_bitrate(stream, bitrate)
    if bitrate > 0 then
        local cmd = "/usr/local/bin/test_encode"
        if stream == "main" then
            if _M.main_scale ~= 0.0 then bitrate = math.floor(bitrate/_M.main_scale) end
            cmd = cmd.." -A"
        else
            if _M.sub_scale ~= 0.0 then bitrate = math.floor(bitrate/_M.sub_scale) end
            cmd = cmd.." -B"
        end
        cmd = cmd.." --bitrate " .. bitrate
        print(" update_bitrate ", cmd )
        os.execute( cmd )
    end
end

-- local function get_dev_name(devname)
--     if string.sub(devname,1,string.len("modem"))=="modem" then
--         local nm_proxy = get_coaf_proxy( "/raw/NETWORK_MANAGER", "nm_daemon" )
--         if not nm_proxy then
--             return nil, "Not nm_proxy"
--         end        
--         local r,data = nm_proxy.GetPPPbyName(devname)
--         nm_proxy:Destroy()
--         if r and data then
--             return data
--         else
--             return nil
--         end
--     else
--         return devname
--     end
-- end

local function get_dev_name(ifname)
    local Iface = ""
    if ifname == "modem1" or ifname == "modem2" then
        local f = io.open("/tmp/" .. ifname .. ".conf", "r")
        if f then
            local v = {}
            local str = f:read("l")
            while str do
                local key, value = str:match("(.+)=(.+)")
                v[key] = value
                str = f:read("l")
            end
            f:close()
            if v.Iface then
                Iface = v.Iface 
            end
        end
    else
        Iface = ifname
    end
    return Iface
end

local function update_ifname_state()
    local count = 0
    local ifnameState = {}

    for index, info in pairs(_M.link_config.polydevs) do
        if info.ifname == "modem1" or info.ifname == "modem2" then
            count = count + 1
            table.insert(ifnameState, count, {id = info.id,
                                    ifname = info.ifname,
                                    actual = get_dev_name(info.ifname),
                                    state = 0})
        end
        
        
    end
    for index, info in pairs(_M.link_config.polydevs) do
        if info.ifname ~= "modem1" and info.ifname ~= "modem2" then
            count = count + 1
            table.insert(ifnameState, count, {id = info.id,
                                        ifname = info.ifname,
                                        actual = info.ifname,
                                        state = 0})
        end
    end
    _M.ifname_state = ifnameState
    _dbg.Info(" klnk update ifname state =====> " )
    _pl.dump(_M.ifname_state)
end

-- local function get_codec_bitrate()
--     local proxy = get_coaf_proxy("/Codec/Encoders", "codec")
--     if proxy then
--         local is_ok, skip, data = proxy:Get({
--             ["encoder.stream[0].video.bitrate"] = 1,
--             ["encoder.stream[0].video.bitrateScale"] = 1,
--         })
--         proxy:Destroy()
--         -- _pl.dump(data)
--         if data["encoder.stream[0].video.bitrate"] then
--             _M.main_bitrate = data["encoder.stream[0].video.bitrate"]
--         end
--         if data["encoder.stream[0].video.bitrateScale"] then
--             _M.main_scale = data["encoder.stream[0].video.bitrateScale"]
--         end
--         print("KLNK get_codec_bitrate：".._M.main_bitrate.." scale:".._M.main_scale)
--     end
-- end

local function start()
    --检查参数
    if _M.klink_service 
        or _M.link_config.poly_enabled==0 
        or _M.link_config.addr=="" 
        or _M.link_config.port==0 
        or _M.link_config.authcode=="" 
        or next(_M.ifname_state) == nil then
        return ""
    end

    local ifname_ok = 0
    for index, info in pairs(_M.ifname_state) do
        if info.ifname and info.actual ~="" then
            ifname_ok = 1
            break
        end
    end
    --检查网卡是否准备好
    if ifname_ok == 0 then
        return ""
    end
    
    _dbg.Info(" klnk service start, Serial ", get_serial_number() )
    _pl.dump(_M.link_config)

    --启动klnk服务
    _M.klink_service  = KM_STREAMING.KLNKService.createNew(0, "media")
    if _M.fwd_enabled then
        --启动端口转发
        _M.forward  = KLNK_FORWARD.createNew();     
    end

    --添加授权设备
    _M.klink_service:AddAllowSerial(get_serial_number(), _M.link_config.authcode);
    --添加媒体源
    if _M.media_source.main then
        _M.klink_service:AddMediaSource(_M.media_source.main, {serial=get_serial_number(), stream=1})
    end
    if _M.media_source.sub then
        _M.klink_service:AddMediaSource(_M.media_source.sub, {serial=get_serial_number(), stream=2})
    end
    --配置网卡
    init_ifname()
    --客户端模式启动服务
    for index, info in pairs(_M.ifname_state) do
        if info.ifname and info.actual ~="" then
            -- if string.sub(info.ifname,1,#"modem")=="modem" then
                -- init_wwanx_ifname(info.actual)
            -- end
            if _M.klink_start then
                _M.klink_service:PathAdd(info.actual)
                if _M.fwd_enabled then
                    _M.forward:PathAdd(info.actual)
                end
            else
                _M.klink_service:StartClient(  { port=_M.link_config.port, 
                                                ip=_M.link_config.addr, 
                                                ifname=info.actual, 
                                                alpn="klnk", 
                                                encrypt=0, 
                                                serial=get_serial_number(), 
                                                key=_M.link_config.authcode,
                                                jitter=_M.link_config.timeout} )
                if _M.fwd_enabled then
                    _M.forward:StartClient(  { port=_M.link_config.port + 1, 
                                                ip=_M.link_config.addr, 
                                                ifname=info.actual, 
                                                alpn="klnk", 
                                                serial=get_serial_number(), 
                                                key=_M.link_config.authcode})
                end
                _M.klink_start = true
            end
        end
    end
    return "ok"
end

local function stop()
    if _M.klink_service then
        _M.klink_service:Stop()
        _M.klink_service = nil
        _M.klink_start   = false
        --关闭聚合服务的时候，更新下最新的编码配置
        update_bitrate("main", _M.main_bitrate);
    end
    if _M.forward then
        _M.forward:Stop()
        _M.forward = nil
    end
    collectgarbage()
end

--轮询线程，处理动态码率调节
local function loop()
    local main_max_rate = 0
    local main_rate_scale = 0.0
    local sub_max_rate = 0
    local sub_rate_scale = 0.0
    local ifname_check = 100
    local serial = get_serial_number()
    while _M.runing do
        if ifname_check > 10 then --间隔10秒检查下，网卡是否已经启动
            for _, info in pairs(_M.ifname_state) do
                if info.actual == "" then
                    local wwanx = get_dev_name(info.ifname)
                    _dbg.Info(" klnk service loop, Serial ", serial, " get dev name :", wwanx )
                    if wwanx and wwanx ~= "" then
                        info.actual = wwanx --保存实际的网卡名称
                        -- init_wwanx_ifname(wwanx)
                        if _M.klink_service then
                            _M.klink_service:PathAdd(wwanx)
                        end
                        if _M.forward then
                            _M.forward:PathAdd(wwanx)
                        end
                    end
                end
            end
            ifname_check = 0;
        end

        if _M.changed then
            stop()
            _M.changed = false
        end
        --读取消息，防止内存增长
        if _M.forward then
            local a,b = _M.forward:RecvMsgpack({timeout = 100})
        end
        
        if _M.klink_service then
            if _M.dynamic_bitrate then
                if main_rate_scale ~= _M.main_scale or main_max_rate ~= _M.main_bitrate then
                    _M.klink_service:SetDynamicBitrate(serial, _M.main_bitrate, _M.main_scale, 1)
                    main_rate_scale  = _M.main_scale
                    main_max_rate    = _M.main_bitrate
                end
                if sub_rate_scale ~= _M.sub_bitrate or sub_max_rate ~= _M.sub_scale then
                    _M.klink_service:SetDynamicBitrate(serial, _M.sub_bitrate, _M.sub_scale, 2)
                    sub_rate_scale  = _M.sub_scale
                    sub_max_rate    = _M.sub_bitrate
                end
            end
        else
            start();
            main_rate_scale = 0.0
            main_max_rate   = 0
            sub_rate_scale  = 0.0
            sub_max_rate    = 0
        end
        _coaf.Sleep(0.5)
        ifname_check = ifname_check + 1
    end
end

function _M.init()
    _dbg.Info(" klnk service init, Serial ", get_serial_number() )
    _M.link_config = _config.Load() --加载配置文件
    _pl.dump(_M.link_config)

    update_ifname_state()

    -- 创建监听线程
    _M.thread_id = _thread.New( loop )
    if _M.thread_id then
        _M.thread_id:Start( )
    end
end

function _M.update_media_source(mediaSource, name)
    if mediaSource then print("update_media_source ------> "..name) end
    if name == "main" then
        _M.media_source.main = mediaSource;
    else
        _M.media_source.sub = mediaSource;
    end
    --媒体源更新的时候要更新klnk service
    if _M.klink_service then
        if name == "main" then
            if _M.media_source.main then
                _M.klink_service:AddMediaSource(_M.media_source.main, {serial=get_serial_number(), stream=1})
            else
                _M.klink_service:DelMediaSource({serial=get_serial_number(), stream=1})
            end
        else
            if _M.media_source.sub then
                _M.klink_service:AddMediaSource(_M.media_source.sub, {serial=get_serial_number(), stream=2})
            else
                _M.klink_service:DelMediaSource({serial=get_serial_number(), stream=2})
            end
        end
    end
end

local function deepcompare(t1,t2,ignore_mt)
    local ty1 = type(t1)
    local ty2 = type(t2)
    if ty1 ~= ty2 then return false end
    -- non-table types can be directly compared
    if ty1 ~= 'table' and ty2 ~= 'table' then return t1 == t2 end
    -- as well as tables which have the metamethod __eq
    local mt = getmetatable(t1)
    if not ignore_mt and mt and mt.__eq then return t1 == t2 end
    for k1,v1 in pairs(t1) do
        local v2 = t2[k1]
        if v2 == nil or not deepcompare(v1,v2) then return false end
    end
    for k2,v2 in pairs(t2) do
        local v1 = t1[k2]
        if v1 == nil or not deepcompare(v1,v2) then return false end
    end
    return true
end

function _M.SetBitrate(bitrate, scale, stream)
    if not bitrate or not scale or not stream then
        return false
    end
    print("KLNK SetBitrate bitrate："..bitrate.." scale:"..scale.. " stream:"..stream)
    if stream == "main" then
        _M.main_bitrate = bitrate
        _M.main_scale   = scale
    else
        _M.sub_bitrate = bitrate
        _M.sub_scale   = scale
    end
    return true
end
-------------------------------web 调用-------------------------------
function _M.SetNetLink(addr, port, authcode, timeout, devs, poly_enabled)
    if not addr or not port or not authcode or not timeout or not devs or not poly_enabled then
        return false, "Invalid arguments"
    end
    _dbg.Info( "#######", addr, port, authcode, poly_enabled)
    _pl.dump(devs)
    if type(addr) ~= "string" then
        return false, "Invalid addr"
    end

    port = tonumber(port)
    if not port then
        return false, "Invalid port"
    end

    if type(devs) ~= "table" then
        devs = {
            { id = 1, ifname = "modem1" },
            { id = 2, ifname = "modem2" },
        }
    end

    poly_enabled = tonumber(poly_enabled)
    if not poly_enabled or ( poly_enabled ~= 1 and poly_enabled ~= 0) then
        return false, "Invalid enable arg"
    end
    
    if not deepcompare(_M.link_config.polydevs, devs) 
        or _M.link_config.addr ~= addr 
        or _M.link_config.port ~= port 
        or _M.link_config.authcode ~= authcode 
        or _M.link_config.poly_enabled ~= poly_enabled 
        or _M.link_config.timeout ~= timeout then
        _M.changed = true
    end
    _M.link_config.addr         = addr
    _M.link_config.port         = port
    _M.link_config.authcode     = authcode
    _M.link_config.timeout      = timeout
    _M.link_config.poly_enabled = poly_enabled
    _M.link_config.polydevs     = devs
    _M.link_config:__TRIGGER(true)

    update_ifname_state()

    return true
end

function _M.GetNetLink()
    return _M.link_config
end

local function cutdownstr(String,End)
    if not String then
        return ""
    end
    if string.sub(String,-string.len(End))==End then
        return string.sub(String, 1, string.len(String) - string.len(End))
    else
        return String
    end
end

local function toFixed( num, digits )
    local value = tonumber(num)
    digits = digits or 0
    return string.format("%." .. tonumber(digits) .. "f", value)
end

local function byteConvert(bytes)
    if not bytes then
        return ""
    end

    local conTAB = { 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y' }
    local exp = math.floor(math.log(bytes)/math.log(2))
    if exp < 1 then
        exp = 0
    end

    local i = math.floor(exp / 10)
    bytes = bytes / math.pow(2, 10 * i);

    if string.len(tostring(bytes)) > string.len(toFixed(bytes,2)) then
        bytes = toFixed(bytes,2);
    end
    bytes = cutdownstr(tostring(bytes),".0")
    return bytes .. conTAB[i+1]
end

function _M.GetBondingStatus()
    local ret = {}
    local links = {}
    local linkcount = 0
    local workercount = 0
    local sendtotal = 0
    local recvtotal = 0
    for _, dev in pairs(_M.ifname_state) do
        dev.state = 0
    end
    if _M.klink_service then
        local conns = _M.klink_service:PathLocalState(get_serial_number()) --只能拿到链路连接成功的网络状态
        for _, info in pairs(conns) do
            --和设置的网络设备比对
            for _, dev in pairs(_M.ifname_state) do
                if dev.actual == info.ifname then
                    dev.state = 1
                    break;
                end
            end
            linkcount = linkcount + 1
            workercount = workercount + 1
            table.insert(links, linkcount, {
                id = linkcount,
                link = info.ifname,
                rtt = info.rtt,
                send = info.tx_all,
                recv = info.rx_all,
                lostrate =  info.lost,
                sendrate = info.tx,
                recvrate = info.rx,
                state = info.state,
            })
            sendtotal = sendtotal + info.tx_all
            recvtotal = recvtotal + info.rx_all
        end
    end
    for _, dev in pairs(_M.ifname_state) do
        if dev.state==0 then
            linkcount = linkcount + 1
            table.insert(links, linkcount, {
                id = linkcount,
                link =  dev.ifname,
                send = "unknown",
                recv = "unknown",
                rtt = "unknown",
                lostrate = "unknown",
                sendrate = "unknown",
                recvrate = "unknown",
                state = "unknown",
            })
        end
    end

    ret.server = _M.link_config.addr
    ret.linkcount = linkcount
    ret.workercount = workercount
    ret.enable = _M.link_config.poly_enabled
    ret.sendtotal = byteConvert(sendtotal)
    ret.recvtotal = byteConvert(recvtotal)
    -- print("sendtotal: "..ret.sendtotal.." recvtotal:"..ret.recvtotal)
    if _M.link_config.poly_enabled == 0 then
        ret.state = "disabled"
    else
        if _M.klink_service then
            local serial = get_serial_number()
            -- 1 不在线， 2 在线， 3 连接中， 4 鉴权失败
            local status = _M.klink_service:GetState(serial);
            if status == 2 then
                ret.state = "connected"
                ret.links = links
            elseif status == 3 or status == 1 then
                ret.state = "connecting"
            elseif status == 4 then
                ret.state = "authError"
            end
        else
            ret.state = "connecting"  
        end
    end

    ret.config = _M.link_config

    return ret
end

local function GetPolyStats(serial)
    local ret = {
        link = "",
        send = "",
        recv = "",
        rtt = "",
        lostrate = "",
        sendrate = "",
        recvrate = "",
    }

    for _, dev in pairs(_M.ifname_state) do
        dev.state = 0;
    end

    local linkcount = 0
    local conns = _M.klink_service:PathLocalState(serial) --只能拿到链路连接成功的网络状态
    for _, info in pairs(conns) do
        --和设置的网络设备比对
        for _, dev in pairs(_M.ifname_state) do
            if dev.actual == info.ifname then
                dev.state = 1;
                break;
            end
        end
        -- info.ifname     网卡名称，eth0/wwan0/wwan1
        -- info.state      链路连接状态 IDLE/CONNECTING/CONNECTED/BROKEN/DEAD/UNKNOWN
        -- info.rtt        ms
        -- info.rx       实时接收率，kbps
        -- info.tx       实时发送率，kbps
        -- info.lost       实时丢包率
        -- info.tx_all   总发送数据量，kB
        -- info.rx_all   总接收数据量，kB
        -- info.lost_all   丢包总数字节
        -- info.last_tx_time  字符串格式，单位微妙
        -- info.last_rx_time  字符串格式，单位微妙
        -- info.create_time     字符串格式，单位微妙
        -- info.pacing_rate       kbps
        -- info.bw_estimate       预估带宽 kbps
        -- info.cwin       滑动窗口大小
        -- print("ifname:"..info.ifname.." state:"..info.state..
        -- 		" rtt:"..info.rtt.." tx:"..info.tx.." rx:"..info.rx.." lost:"..info.lost..
        -- 		" tx_all:"..info.tx_all.." rx_all:"..info.rx_all.." lost_all:"..info.lost_all..
        -- 		" pacing_rate:"..info.pacing_rate.." bw_estimate:"..info.bw_estimate.." cwin:"..info.cwin)
        linkcount = linkcount + 1
        ret.link  = ret.link .. info.ifname .. " | "
        ret.rtt   = ret.rtt .. (info.rtt or 0) .. " | "
        ret.send  = ret.send .. byteConvert(info.tx_all or 0) .. " | "
        ret.recv  = ret.recv .. byteConvert(info.rx_all or 0) .. " | "
        ret.lostrate =  ret.lostrate .. (info.lost or 0) .. " | "
        ret.sendrate = ret.sendrate .. byteConvert(info.tx or 0) .. " | "
        ret.recvrate = ret.recvrate .. byteConvert(info.rx or 0) .. " | "
    end

    for _, dev in pairs(_M.ifname_state) do
        if dev.state==0 then
            ret.link = ret.link .. dev.actual .. " | "
            ret.rtt = ret.rtt .. " - | "
            ret.send = ret.send .. " - | "
            ret.recv = ret.recv .. " - | "
            ret.lostrate =  ret.lostrate .. " - | "
            ret.sendrate = ret.sendrate .. " - | "
            ret.recvrate = ret.recvrate ..  " - | "
        end
    end

    if linkcount > 0 then
        ret.link = cutdownstr(ret.link, " | ")
        ret.rtt = cutdownstr(ret.rtt, " | ")
        ret.send = cutdownstr(ret.send, " | ")
        ret.recv = cutdownstr(ret.recv, " | ")
        ret.lostrate = cutdownstr(ret.lostrate, " | ")
        ret.sendrate = cutdownstr(ret.sendrate, " | ")
        ret.recvrate = cutdownstr(ret.recvrate, " | ")
    end
    return ret
end

function _M.GetPolyStatus()
    local ret = {}
    if _M.link_config.poly_enabled == 0 then
        ret.state = "disabled"
    else
        if _M.klink_service then
            local serial = get_serial_number()
            -- 1 不在线， 2 在线， 3 连接中， 4 鉴权失败
            local status = _M.klink_service:GetState(serial);
            print("######GetPolyStatus. " .. status)
            if status == 2 then
                ret.state = "connected"
                ret.stats = GetPolyStats(serial)
            elseif status == 3 or status == 1 then
                ret.state = "connecting"
            elseif status == 4 then
                ret.state = "authError"
            end
        else
            ret.state = "connecting"  
        end
    end
    return ret
end

return _M