local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
require "kmCore"

----------------------------------------------------

local _M = {
	server = nil
}

----------------------------------------------------

local kmp_meta = {}

function kmp_meta:AddSession( idr, mediaSource, channel, options )
	if not mediaSource or not channel then
		_dbg.Error( "KMP", "Invalid KMP session arguments (MediaSource/Channel ID ...). Stream:", self.stream_id, ", IDR:", idr )
		return nil
	end

	if self.channel then
		_dbg.Warn( "KMP", "The KMP session is already created. Stream:", self.stream_id, ", IDR:", idr )
		return self.channel
	end

	if _M.server:AddMediaChannel( channel, mediaSource ) 
	then
		_dbg.Info( "KMP", "Session of stream '" .. idr .. "' created/updated. [channel=" .. channel .. "]"  )
		local timeShift = _M.server:GetTimeShift()
		if options and options.timeShift and options.timeShift > timeShift then
			_M.server:SetTimeShift( options.timeShift )
		end
		self.channel = channel
	else
		_dbg.Error( "KMP", "Session of stream '" .. idr .. "' add fail! Channel id:", channel )
	end
	return self.channel
end

function kmp_meta:RemoveSession()
	if self.channel then
		_M.server:RemoveMediaChannel( self.channel )
		self.channel = nil
		collectgarbage()
	end

	return true
end

function kmp_meta:StartSession()
	_M.server:Start()
end

function kmp_meta:StopSession()
	--TODO: Nothing.
end

function kmp_meta:SetTimeShift(shift)
	_M.server:SetTimeShift(shift)
	return true
end

function kmp_meta:Destroy()
	self:RemoveSession()
end

kmp_meta.__index = kmp_meta

----------------------------------------------------

function _M.GetService( stream_id, options )
	stream_id = tonumber(stream_id)
	stream_id = string.format( "%02d", stream_id )
	if not options then
		options = {
			bind = "*",
			port = 8580
		}
	end

	if not _M.server then
		_M.server = KM_STREAMING.KMP.PullingServer.createNew( "PullingServer", options.bind, options.port )
		if not _M.server then
			_dbg.Error( "KMP", "Could not create KMP server at:", options.bind .. ":" .. options.port, "! This effects stream:", stream_id )
			return nil
		else
			_dbg.Info( "KMP", "Created a KMP server instance, at:", options.bind .. ":" .. options.port )
			_M.server:SetTimeShift(500)
		end
	end

	local service = {
		stream_id = stream_id
	}

	return setmetatable( service, kmp_meta )
end

----------------------------------------------------
return _M

