local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local pl = require "pl.pretty"

require "kmCore"

local _M = {
	each_pusher_session_limits = 3,
	pusher_groups = {
		["00"] = {},
		["01"] = {}
	},
}

local rtp_pusher_meta = {}

function rtp_pusher_meta:AddSession( idr, mediaSource, options )
	if not mediaSource then
		_dbg.Error( "RTP-PUSH", "Invalid RTP pusher arguments MediaSource Stream:", self.stream_id )
		return nil
	end

    if not options.video_ip and not options.video_port then
        _dbg.Error("RTP-PUSH", "Invalid Rtp pusher options, not video_ip and not video_port")
        return nil
    end

    options.audio_ip = options.audio_ip or options.video_ip
    options.audio_port = options.audio_port or options.video_port
	options.idr = idr

	self.session = self.group.group:AddSession( mediaSource, {
        url = string.format("rtp://%s:%s", options.video_ip, options.video_port)
    } )

	if not self.session then
		_dbg.Error( "RTP-PUSH", "Fail to create RTP pusher session for Stream:", self.stream_id, options.video_ip, options.video_port )
		return nil
	end

    self.savedOptions = options
    self.savedMediaSource = mediaSource
	self.formalUrl = string.format("rtp://%s:%s", options.video_ip, options.video_port)
	_dbg.Info( "RTP-PUSH", "Created RTP push session, for Stream:", self.stream_id, ", URL:", string.format("rtp://%s:%s", options.video_ip, options.video_port) )
	return self.sessions
end

function rtp_pusher_meta:RemoveSession()
	if self.session then
		self.group.group:RemoveSession(self.session)
		_dbg.Info( "RTP-PUSH", "RTP push sessions is destroied, for Session:", self.session, ", URL:", self.formalUrl )
	end

	self.session = nil
	self.formalUrl = nil
	self.savedOptions = nil
	self.savedMediaSource = nil
	collectgarbage()
	return true
end

function rtp_pusher_meta:StartSession()
	--TODO
	return true
end

function rtp_pusher_meta:StopSession()
	if self.sessions then
		self.group.group:RemoveSession()
		_dbg.Info( "RTP-PUSH", "RTP push session is stopped, for Stream:", self.stream_id, ", URL:", self.formalUrl )
		collectgarbage()
	end
	return true
end

function rtp_pusher_meta:Destroy()
	self:RemoveSession()
end

rtp_pusher_meta.__index = rtp_pusher_meta

----------------------------------------------

function _M.GetStreamer( stream_id )
	stream_id = tonumber(stream_id)
	stream_id = string.format( "%02d", stream_id )

	local fetched_group
	local pgroups = _M.pusher_groups[ stream_id ]
	if not pgroups then
		_dbg.Error( "RTP-PUSH", "Invalid stream id of:", stream_id )
		return nil
	end

	for i=1, #pgroups do
		if pgroups[i].counter < _M.each_pusher_session_limits then
			pgroups[i].counter = pgroups[i].counter + 1
			fetched_group = pgroups[i]
			break
		end
	end

    if not fetched_group then
        local idr = "rtppush:" .. stream_id .. ":" .. #pgroups + 1
		local one = KM_STREAMING.RTP.SERVICE.createNew( idr )
		if not one then
			_dbg.Error( "RTP-PUSH", "Fail to create rtp pusher for stream:", stream_id )
			return
		end
		_dbg.Info( "RTP-PUSH", "A new rtp pusher group is created, for stream:", stream_id, ", IDR:", idr )
		fetched_group = {
			counter = 1,
			group = one
		}
		table.insert( pgroups, fetched_group )
    end

	local pusher = {
		stream_id = stream_id,
		group = fetched_group
	}
	return setmetatable( pusher, rtp_pusher_meta )
end

----------------------------------------------
return _M


