local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

local CONFIG_LOCATION = "/data/configs"

local _M = {
}

function _M.GetRebootTimerInfo()
	return cfg_timer
end

function _M:LoadConfigureFile(filename, ltime, sysconf)
	os.execute("/bin/cp " .. filename .. " /data/configs/")
	os.execute("/bin/cp -d " .. ltime .. " /etc/")
	os.execute("/bin/cp -Rf " .. sysconf .. " /etc/systemd/")
	return 1
end

-- ADD                                                                                                                            
local function stringFormatArray(str,split)                                                                                       
        split = split or ","                                                                                                      
        local t_str = str .. split                                                                                                
        local item                                                                                                                
        local ret = {}                                                                                                            
        local matchs = "(.-)%".. split .. "+"                                                                                     
        for item in t_str:gmatch(matchs) do                                                                                       
                if item ~= "" then                                                                                                
                        table.insert(ret, item)                                                                                   
                end                                                                                                               
        end                                                                                                                       
        return ret                                                                                                                
end                                                                                                                               


local function dateTransformat(date) -- "2000-01-01"                                                                              
        local _s,_e,t_y,t_m,t_d = string.find(date,"(%d+)-(%d+)-(%d+)")                                                           
        if not _s then                                                                                                            
                local ret = stringFormatArray(date,"-")                                                                           
                t_y = ret[1]                                                                                                      
                t_m = ret[2]                                                                                                      
                t_d = ret[3]                                                                                                      
        end                                                                                                                       
        return t_y,t_m,t_d                                                                                                        
end                                                                                                                               
                                                                                                                                  
local function localDateDomp(date,hour,min)                                                                                       
        local yy,mm,dd = dateTransformat(date)                                                                                    
        time = os.time({year = yy, month = mm, day = dd, hour = hour, minute = min, second = "00"})                               
        return time > os.time(),time,mm,dd                                                                                        
end                                                                                                                               
                                                                                                                                  
local function checkDate(date,time)                                                                                               
        local _year,_mon,_day = dateTransformat(date)                                                                             
        local n_year,n_mon,n_day                                                                                                  
        n_year = os.date("%Y",time)                                                                                               
        n_mon = os.date("%m",time)                                                                                                
        n_day = os.date("%d",time)                                                                                                
        if n_year == _year and n_mon == _mon and n_day == _day then                                                               
                return true                                                                                                       
        end                                                                                                                       
        return false                                                                                                              
end                                    


function _M.SetRebootTimerInfo(enable,date,hour,minute,policy,week,day,month,days)                          
        _dbg.Info("SetRebootTimerInfo:",enable,date,hour,minute,policy,week,day,month,days)                                         
                                                                                                                                  
        cfg_timer.enable = enable                                                                                                 
        cfg_timer.policy = policy                                                                                                 
        cfg_timer.date = date                                                                                                     
        cfg_timer.hour = hour                                                                                                     
        cfg_timer.minute = minute                                                                                                 
        cfg_timer.week = week                                                                                                     
        cfg_timer.day = day                                                                                                       
        cfg_timer.month = month                                                                                                   
        cfg_timer.days = days                                                                                                     
        cfg_timer.rebootDate = cfg_timer.rebootDate or "1970-1-1"                                                                 
        if cfg_timer.rebootDate ~= "1970-1-1" then                                                                                
                cfg_timer.rebootDate = "1970-1-1"                                                                                 
        end                                                                                                                       
        cfg_timer:__TRIGGER(true)                                                                                                 
        _M.GetRebootTimerInfo()                                                                                                   
        return true                                                                                                               
end 


function _M.loadConfig(enable,date,hour,minute,policy,week,day,month,days)                                  
        _dbg.Info("InitRebootTimerInfo:",enable,date,hour,minute,policy,week,day,month,days)                
        cfg_timer.enable = enable                                                                           
        cfg_timer.policy = policy                                                                           
        cfg_timer.date = date                                                                               
        cfg_timer.hour = hour                                                                               
        cfg_timer.minute = minute                                                                           
        cfg_timer.week = week                                                                               
        cfg_timer.day = day                                                                                 
        cfg_timer.month = month                                                                             
        cfg_timer.days = days                                                                               
        cfg_timer:__TRIGGER(true)                                                                           
        _M.GetRebootTimerInfo()                                                                             
        return true                                                                                         
end                                                                                                         
                                                                                                            
                                                                     
function _M.Init()                                                                                                                              
        _dbg.Info("RebootTimer  Init")                                                                                                          
        _M.loadConfig(cfg_timer.enable,cfg_timer.date,cfg_timer.hour,cfg_timer.minute,cfg_timer.policy,cfg_timer.week,cfg_timer.day,            
        cfg_timer.month,cfg_timer.days)                                                                                                         
end                                                                                                                                             

-- coaf timer                                                                                                                                   
function _M.rebootTimer()                                                                                                                       
        _dbg.Info("SetRebootTimerInfo:===","working!")                                                                                          
                                                                                                                                                
        if tonumber(cfg_timer.enable ) ~= 1 then                                                                                                
                return                                                                                                                          
        end                                                                                                                                     
        local r,s_time = localDateDomp(cfg_timer.date, cfg_timer.hour, cfg_timer.minute)                                                        
        if r then                                                                                                                               
                return                                                                                                                          
        end                                                                                                                                     
        local time = os.time()                                                                                                                  
        local n_hour,n_minute                                                                                                                   
        n_hour = os.date("%H",time)                                                                                                             
        n_minute = os.date("%M",time)                                                                                                           
                                                                                                                                                
        if n_hour+0 ~= cfg_timer.hour+0 or n_minute+0 ~= cfg_timer.minute+0 then                                                                
                return                                                                                                                          
        end                                                                                                                                     
        if checkDate(cfg_timer.rebootDate,time) then                                                                                            
                return                                                                                                                          
        end                                                                                                                                     
        local canReboot = false                                                                                                                 
        if cfg_timer.policy == "day" then                                                                                                       
                local d = cfg_timer.day+0                                                                                                       
                if d > 1 then                                                                                                                   
                        if math.modf((time - s_time) / 86400) % d == 0 then                                                                     
                                canReboot = true                                                                                                
                        end                                                                                                                     
                else                                                                                                                            
                        if os.date("%Y",time)+0 == os.date("%Y",s_time)+0 and os.date("%m",time)+0 == os.date("%m",s_time)+0 and                
                                os.date("%d",time)+0 == os.date("%d",s_time)+0 then                                                             
                                if d == 0 then                                                                                                  
                                        canReboot = true                                                                                        
                                end                                                                                                             
                        else                                                                                                                    
                                if d == 1 then                                                                                                  
                                        canReboot = true                                                                                        
                                end                                                                                                             
                        end                                                                                                                     
                 end
          elseif cfg_timer.policy == "mon" then
		local months = stringFormatArray(tostring(cfg_timer.month))
		local days = stringFormatArray(tostring(cfg_timer.days))
		local n_mon = os.date("%m",time)+0
		local n_day = os.date("%d",time)+0
		local i,v 
		local in_month = false
		for i,v in pairs(months) do
			if n_mon == v+0 then
				in_month = true
				break
			end
		end
		if in_month then
			for i,v in pairs(days) do
				if n_day == v+0 then
					canReboot = true
					break
				end
			end
		end
          elseif cfg_timer.policy == "week" then
		local weeks = stringFormatArray(tostring(cfg_timer.week))
		local n_week = os.date("%w",time)+0
		local i,v 
		for i,v in pairs(weeks) do
			if n_week == v+0 then
				canReboot = true
				break
			end
		end
	end
	if canReboot then
		cfg_timer.rebootDate = os.date("%Y-%m-%d",time)
		cfg_timer:__TRIGGER(true)
		os.execute("/sbin/reboot")
	end
end


return _M