------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=BASIC}: 添加HTTP登录用户
##ARGUMENTS{GET,POST}: 
    @@Username{STR}: 要添加的用户名
	@@Password{STR}: 添加用户的密码
    	
##RETURNS: 
	++Data{OBJ}:
		**Reason{STR}: 如果返回错误码为405，Reason说明添加用户失败的原因：
			%%reserved: 使用了系统保留的用户名(如admin)
			%%invalid: 无效的用户名
			%%exist: 指定的用户已存在
			%%full: 超出了系统最多支持的用户数
		~~
	~~

##NOTES:
>>{BOLD} 安全性说明：
>> 调用该API必须通过HTTP的身份认证。身份认证用户名和密码同Web登录。

##__OTHERS:
>> 在本页面的API Call测试中，给定的用户名为admin，密码为admin（指定HTTP Request header中Authorization:"Basic YWRtaW46YWRtaW4="）。如果需要修改，请在下面指定：
<script type="text/javascript">
ajaxAuth = "Basic YWRtaW46YWRtaW4=";
function change_auth(val)
{
	ajaxAuth = val;
	alert('DONE');
}
</script>
>> 修改Authorization值: <input type="text" id="test_auth" size="28" value="Basic YWRtaW46YWRtaW4=" /> <input type="button" value="修改" onclick='change_auth($("#test_auth")[0].value);' />
<br /><br />
]]
local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Username or not args.Password then
	cgilua.put([[
	{
		"Result":400,
		"Status":"ERROR: Invalid arguments"
	}]] )
end

local r, code, reason = rpc.Call( "SERVER.AddUser", args.Username, args.Password )
if r and code then
	cgilua.put([[
	{
		"Result":200,
		"Status":"OK"
	}]] )
else
	cgilua.put([[
	{
		"Result":405,
		"Status":"ERROR: Fail to add user",
		"Data":{
			"Reason":"]] .. (reason or "unknown") .. [["
		}
	}]] )
end
