------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=WWAN}: 添加一个无线宽带(WWan)的连接配置
##ARGUMENTS{POST}: 
  @@DeviceObject: 可选参数。如果指定，则为网络接口的ObjectID（参见 ^^REF{getNetInterface.lua}）；如果没有指定，则为默认的WWan接口
  @@Title{*}: 连接名称
  @@Type{STR,*}: 指定WWan网络类型
    %%gsm: 网络为GSM/HSPA/LTE类型
    %%cdma: 网络为CDMA类型
  ~~

  @@AutoConnect{*}: 自动连接选项
    %%1: 表示当在范围内找到这个热点，将自动连接
    %%0: 不自动连接，需要手动进行连接（参见 ^^REF{connectWWan.lua}）
  ~~

  @@G=GSM: 对于Type为gsm类型，需要指定以下参数：
  @@GSM_APN{G=GSM,*}:    网络APN
  @@GSM_Number{G=GSM,*}: 拨号号码
  @@GSM_User{G=GSM}:     用户名
  @@GSM_Password{G=GSM}: 密码
  @@GSM_Pin{G=GSM}:      SIM卡PIN

  @@G=CDMA: 对于Type为cdma类型，需要指定以下参数：
  @@CDMA_Number{G=CDMA,*}:  拨号号码
  @@CDMA_User{G=CDMA}:      用户名
  @@CDMA_Password{G=CDMA}:  密码

##RETURNS{DEFAULT}: 

]]

local WWAN = require "M_WWanSet"
local cgilua = require "cgilua"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid request arguments"
}
]])
	return
end

local objectId = args.DeviceObject
if not objectId or objectId == "" or objectId == "/" then
	local device = WWAN.GetDefaultWWan()
	if not device then
		cgilua.put([[
	{
		"Result":500,
		"Status":"FAIL: Fault to get WWan device"
	}
	]])
		return
	end
	objectId = device.Object
end

local r,msg = WWAN.SetConnection( objectId, true --[[New one]], args )

cgilua.put([[
{
	"Result":]] .. (r and "200" or "400") .. [[,
	"Status":"]] .. (msg or "OK") .. [["
}
]])



