------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=WIFI}: 连接到一个WIFI Connection
##ARGUMENTS{POST}: 
  @@DeviceObject: 可选参数。如果指定，则为网络接口的ObjectID（参见 ^^REF{getNetInterface.lua}）；如果没有指定，则为默认的Wifi接口
  @@Object{*}: 要连接Wifi Connection的Object名称（通过^^REF{listWifiConnection.lua}可以获得）

##RETURNS{DEFAULT}: 

]]

local WIFI = require "M_WifiSet"
local cgilua = require "cgilua"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Object then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid request arguments"
}
]])
	return
end

local objectId = args.DeviceObject
if not objectId or objectId == "" or objectId == "/" then
	local device = WIFI.GetDefaultWifi()
	if not device then
		cgilua.put([[
	{
		"Result":500,
		"Status":"FAIL: Fault to get Wifi device"
	}
	]])
		return
	end
	objectId = device.Object
end

local r,msg = WIFI.ConnectConnection( objectId, args.Object )

cgilua.put([[
{
	"Result":]] .. (r and "200" or "400") .. [[,
	"Status":"]] .. (msg or "OK") .. [["
}
]])


