------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=RECORD}: 删除录像文件

##ARGUMENTS{GET,POST}:
	@@Device{STR,*}: 要删除录像文件的磁盘设备号（请参考^^REF{getRecordList.lua}返回的RecordDevice字段）
	@@SequenceID{STR,*}: 要删除文件所在的录像Sequence ID（请参考^^REF{getRecordList.lua}返回的SequenceID字段）
	@@FileFilter{STR}: 指定要删除的文件名，或者文件名通配符。文件名通配符可以使用*匹配任意（多个）字符，例如：
	REC*表示匹配以REC开头的所有文件（可以匹配 REC00000001.ts, REC00000002.ts, ...）；再如：*0001*表示匹配所有文件名
	中包含有0001的文件（可以匹配 REC00000001.ts, REC00000010.ts, ...）。
	>> 可选参数。如果没有指定，表示删除所有当前Sequence中的文件。

##RETURNS{DEFAULT}: 
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local dev, filter
local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Device or args.Device == "" or not args.SequenceID or args.SequenceID == "" then
	cgilua.put([[
	{
		"Result":400,
		"Status":"ERROR: Invalid device or sequence ID"
	}
]])
	return
end
local filter = args.FileFilter
if not filter or filter == "" then
	filter = "*"
end

local r,cc,info = rpc.Call( "BUS.DiskRecordList", args.Device, args.SequenceID )
if not r or not cc or type(info) ~= "table" then
	cgilua.put([[
	{
		"Result":400,
		"Status":"ERROR: No valid recording list"
	}
]])
	return
end

local path = info[1] and info[1].PATH
if not path then
	cgilua.put([[
	{
		"Result":400,
		"Status":"ERROR: No valid recording files"
	}
]])
	return
end

--os.execute( "/bin/rm -f " .. path .. "/" .. filter )
rpc.Call( "BUS.RemoveRecordFile", path .. "/" .. filter )

cgilua.put([[
{
	"Result":200,
	"Status":"OK"
}
]])

