------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=BASIC}: 获取设备的基本信息
##ARGUMENTS{GET,POST}:
  @@_LANG: 可选参数。如果指定了 '_LANG'参数，则获得信息的语言由此参数决定；否则，默认为中文。
    %%cn: 中文
    %%en: 英文
  ~~

##RETURNS: 
	++Data{OBJ}: 返回的设备信息对象
		**SERIAL_NO{STR}: 设备序列号	
		**DEVICE_NAME{STR}: 设备名称（内建名称）
		**DEVICE_VENDOR{STR}: 设备制造商
		**DEVICE_COPYRIGHT{STR}: 版权信息
		**DEVICE_TYPE{STR}: KILOVIEW内部标定的设备型号
		**OEM_TYPE{STR}: OEM厂商特定的设备型号
		**HARDWARE_VERSION{STR}: 硬件版本号	
		**SOFTWARE_VERSION{STR}: 软件版本号
        **HOST_NAME{STR}: 网络主机名称
        **FRIENDLY_NAME{STR}: 设备友好名称
	~~
]]

local cgilua = require "cgilua"
local mBasic = require "M_Basic"
local rpc = require "kiloview.ringRpc"

require("kiloview.globalConfig")(true,true)

-----------------------------------
--Get device serial no.
-----------------------------------

local function GetSerialNo()
	if type(_G.KV) == "table" and type(_G.KV.SERIAL_HISTORIES) == "table" and #_G.KV.SERIAL_HISTORIES > 0 then
		return _G.KV.SERIAL_HISTORIES[ #_G.KV.SERIAL_HISTORIES ].serial
	else
		return "0000000000"
	end
end

local lang = "cn" --Default using Chinese language.
local REQS = require("M_Request").VerifyAndGetArgs()
if REQS and REQS["_LANG"] then
	lang = REQS["_LANG"]
end

local device_oem_info

if OEM_INFO and OEM_TYPE then
	device_oem_info = OEM_INFO[OEM_TYPE][lang]
end

if _G.FEATURE_NDI_HX and lang == "en" then
	local ndi_name
	if DEVICE_TYPE == "G1" or DEVICE_TYPE == "P1" or DEVICE_TYPE == "A1" or DEVICE_TYPE == "R3C1" then
		ndi_name = "NewTek HD/3G-SDI Encoder"
	else
		ndi_name = "NewTek HDMI Encoder"
	end

	device_oem_info = {
		DEVICE_NAME = ndi_name,
		DEVICE_VENDOR = "NewTek Inc.",
		DEVICE_COPY = "Copyright 2017, All rights reserved",
		DEVICE_URL = "<a href='http://www.newtek.com/' class='oemurl'>http://www.newtek.com/</a>",
		DEVICE_TYPE = "NewTek OEM"
	}
end


local done,hwChk = rpc.Call( "SERVER.CheckHardwareInfo" )

_G.HARDWARE_VERSION = hwChk and hwChk.hardwareVersion or _G.HARDWARE_VERSION or "1.4"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local oemType = _G.OEM_TYPE or "ENC-1"
if _G.FEATURE_NDI_HX then
	if DEVICE_TYPE == "G1" or DEVICE_TYPE == "P1" or DEVICE_TYPE == "A1" or DEVICE_TYPE == "R3C1" then
		oemType = "G1_NewTek"
	else
		oemType = "U2_NewTek"
	end
end

cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data":{
		"SERIAL_NO":"]] .. GetSerialNo() .. [[",	
		"DEVICE_NAME":"]] .. ( device_oem_info and device_oem_info["DEVICE_NAME"] or "Encoder" ) .. [[",	
		"DEVICE_VENDOR":"]] .. ( device_oem_info and device_oem_info["DEVICE_VENDOR"] or "" ) .. [[",	
		"DEVICE_COPYRIGHT":"]] .. ( device_oem_info and device_oem_info["DEVICE_COPY"] or "Copyright 2016." ) .. [[",	
		"DEVICE_TYPE":"]] .. ( device_oem_info and device_oem_info["DEVICE_TYPE"] or "ENC-1" ) .. [[",	
		"OEM_TYPE":"]] .. oemType .. [[",	
		"DEVICE_OEM_TYPE":"]] .. _G.OEM_TYPE .. [[",	
		"HARDWARE_VERSION":"]] .. ( _G.HARDWARE_VERSION or "0.0" ) .. [[",	
		"SOFTWARE_VERSION":"]] .. ( _G.SOFTWARE_VERSION or "3.0.0000" ) .. [[",
		"HOST_NAME":"]] .. mBasic.GetHostName() .. [[",
		"FRIENDLY_NAME":"]] .. (mBasic.GetFriendlyName() or "") .. [["
	}
}
]])

