------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=RECORD}: 获取可移动磁盘列表
>> 获取当前设备上已插入的USB磁盘或SD/MMC存储卡磁盘列表。

##ARGUMENTS{GET,POST}:
无

##RETURNS: 
	++Data{ARR}: 返回所有可用的磁盘列表。数组中的每一项代表一个磁盘：
	++{OBJ}:
		**Device{STR}: 磁盘设备号
		**Type{STR}: 磁盘类型
			%%usb: USB存储设备
			%%sdmmc: SD/MMC存储卡
		~~

		**Filesys{STR}: 磁盘的文件系统类型
			%%ntfs: NTFS文件系统
			%%vfat: FAT32文件系统
		~~
		
		**VolumnName: 磁盘卷标
		**Total{INT}: 磁盘总容量 (MB)
		**Used{INT}: 已使用空间 (MB)
		**Free{INT}: 可用空间 (MB)
		**UsedPercent{NUM}: 占用率百分比
	~~
	~~
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"
local cjson = require "cjson"


local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local _r, polyStatus = rpc.Call( "POLY.GetBondingStatus" )

if not _r or not polyStatus then
	cgilua.put([[
	{
		"Result":400,
		"Status":"ERROR: Get Bonding Status error"
	}
]])
else
	cgilua.put(cjson.encode(polyStatus))
end
