------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=BASIC}: 获取设备的名称配置

##ARGUMENTS: 
  @@_LANG: 可选参数。如果指定了 '_LANG'参数，则获得信息的语言由此参数决定；否则，默认为中文。
    %%cn: 中文
    %%en: 英文
  ~~

##RETURNS: 
	++Data{OBJ}:
		**DEVICE_NAME:  内建的默认设备名称
        **HOST_NAME:    设备主机名称，决定网络访问时所能获得的网络名称
        **FRIENDLY_NAME:设备友好名称，该名称可以决定Onvif探测协议得到的名称
	~~
]]

local cgilua = require "cgilua"
local mBasic = require "M_Basic"
require("kiloview.globalConfig")()

local lang = "cn" --Default using Chinese language.
local REQS = require("M_Request").VerifyAndGetArgs()
if REQS and REQS["_LANG"] then
	lang = REQS["_LANG"]
end

local device_oem_info
if OEM_INFO and OEM_TYPE then
	device_oem_info = OEM_INFO[OEM_TYPE][lang]
end

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )
local ConfigData = mBasic.GetNDIConfig()

cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data":{
		"DEVICE_NAME":"]] .. ( device_oem_info and device_oem_info["DEVICE_NAME"] or "Encoder" ) .. [[",	
		"HOST_NAME":"]] .. mBasic.GetHostName() .. [[",
		"FRIENDLY_NAME":"]] .. ConfigData["streamer.stream[0].NDI_server.device_name"] .. [[",
		"GROUP_NAME":"]] .. ConfigData["streamer.stream[0].NDI_server.group_name"] .. [[",
		"NDI_CONNECTION":"]] .. ConfigData["streamer.stream[0].NDI_server.connection"] .. [[",
		"NDI_MCAST_PREFIX":"]] .. ConfigData["streamer.stream[0].NDI_server.mcast_prefix"] .. [[",
		"NDI_MCAST_MASK":"]] .. ConfigData["streamer.stream[0].NDI_server.mcast_mask"] .. [[",
		"NDI_DISCOVERY_SERVER":"]] .. ConfigData["streamer.stream[0].NDI_server.server_addr"] .. [[",
		"NDI_ENABLE":]] .. ConfigData["streamer.stream[0].NDI_server.enabled"] .. [[
	}
}
]])

