------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=OSD}: 获得主码流/子码流的字符叠加配置

##ARGUMENTS{GET,POST}: 
    @@Stream{*}: 指定码流
        %%main: 主码流
        %%sub: 子码流
    ~~
##RETURNS: 
    ++Data{ARR}: 每一项为一个OSD叠加项，具体内容如下：
        ++{OBJ}:
            **ID{INT}: 当前叠加项的ID，范围 0-3

            **Type: 叠加内容类型：
                %%text: 指定的文字
                %%time: 显示系统时间
                %%none: 关闭叠加
            ~~
            **Position:
                叠加位置。分别表示：
                %%left-top: 左上角
                %%top-center: 顶部居中
                %%right-top: 右上角
                %%left-bottom: 左下角
                %%bottom-center: 底部居中
                %%right-bottom: 右下角
                %%special: 手动指定位置
            ~~

            **X{INT}: 像素为单位，手动指定的水平叠加位置（仅当Position参数为special时有效）
            **Y{INT}: 像素为单位，手动指定的垂直叠加位置（仅当Position参数为special时有效）

            **Font: 字体名称。当前支持的字体有:
                %%default: 默认，英文
                %%song: 中文，宋体
                %%vera: 英文
                %%lucida: 英文
            ~~

            **Size{INT}:    值为0，表示小号字体；1表示中等字体；2表示大号字体；大于2表示指定的文字大小（px）
            **Outline{INT}: 指定在文字的外围加上边框的粗细；0表示不加边框，数值越大，边框越粗。
            **Bold{INT}: 0|1, 字体加粗。0表示不加粗，1表示加粗
            **Italic{INT}: 0|1，斜体。0表示正常，1表示斜体（注：当前尚不支持斜体功能）
            **Text: 叠加文字(UTF-8编码，仅当 Type 为 text 时有效)
        ~~
    ~~
]]


local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Stream or args.Stream == "" then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid arguments"
}
]])
	return
end

local stream = 0
if args.Stream == "sub" then
	stream = 1
end

cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data":[
]])

local i
for i=0, 15 do
	local gets = {
		[ "osd.stream[" .. stream .. "].items[" .. i .. "].osd_type" ] = "",
		[ "osd.stream[" .. stream .. "].items[" .. i .. "].position" ] = "",
		[ "osd.stream[" .. stream .. "].items[" .. i .. "].x" ] = 0,
		[ "osd.stream[" .. stream .. "].items[" .. i .. "].y" ] = 0,
		[ "osd.stream[" .. stream .. "].items[" .. i .. "].font" ] = "",
		[ "osd.stream[" .. stream .. "].items[" .. i .. "].font_color" ] = 1,
		[ "osd.stream[" .. stream .. "].items[" .. i .. "].size" ] = 1,
		[ "osd.stream[" .. stream .. "].items[" .. i .. "].outline" ] = 0,
		[ "osd.stream[" .. stream .. "].items[" .. i .. "].outline_color" ] = 0,
		[ "osd.stream[" .. stream .. "].items[" .. i .. "].line_space" ] = "x0.5",
		[ "osd.stream[" .. stream .. "].items[" .. i .. "].format" ] = "",
		[ "osd.stream[" .. stream .. "].items[" .. i .. "].bold" ] = 0,
		[ "osd.stream[" .. stream .. "].items[" .. i .. "].italic" ] = 0,
		[ "osd.stream[" .. stream .. "].items[" .. i .. "].text" ] = "",
	}

	local r,cc
	r, cc, gets = rpc.Call( "BUS.Get", gets )
	if i > 0 then
		cgilua.put( "				," )
	end
	if not r or not gets then
		cgilua.put( [[
			{
				"ID":]] .. i .. [[,
				"Type":"none",
				"ERROR":"Fail to get arguments"
			}
		]])
	else
		cgilua.put( [[
			{
				"ID":]] .. i .. [[,
				"Type":"]] .. gets[ "osd.stream[" .. stream .. "].items[" .. i .. "].osd_type" ] .. [[",
				"Position":"]] .. gets[ "osd.stream[" .. stream .. "].items[" .. i .. "].position" ] .. [[",
				"X":]] .. gets[ "osd.stream[" .. stream .. "].items[" .. i .. "].x" ] .. [[,
				"Y":]] .. gets[ "osd.stream[" .. stream .. "].items[" .. i .. "].y" ] .. [[,
				"Font":"]] .. gets[ "osd.stream[" .. stream .. "].items[" .. i .. "].font" ] .. [[",
				"FontColor":]] .. gets[ "osd.stream[" .. stream .. "].items[" .. i .. "].font_color" ] .. [[,
				"Size":]] .. gets[ "osd.stream[" .. stream .. "].items[" .. i .. "].size" ] .. [[,
				"Outline":]] .. gets[ "osd.stream[" .. stream .. "].items[" .. i .. "].outline" ] .. [[,
				"OutlineColor":]] .. gets[ "osd.stream[" .. stream .. "].items[" .. i .. "].outline_color" ] .. [[,
				"LineSpace":"]] .. gets[ "osd.stream[" .. stream .. "].items[" .. i .. "].line_space" ] .. [[",
				"Bold":]] .. gets[ "osd.stream[" .. stream .. "].items[" .. i .. "].bold" ] .. [[,
				"Italic":]] .. gets[ "osd.stream[" .. stream .. "].items[" .. i .. "].italic" ] .. [[,
				"Format":"]] .. gets[ "osd.stream[" .. stream .. "].items[" .. i .. "].format" ] .. [[",
				"Text":"]] .. gets[ "osd.stream[" .. stream .. "].items[" .. i .. "].text" ] .. [["
			}
		]])
	end
end

cgilua.put([[
	]
}
]])

