------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------
--[[

##ABSTRACT{G=IMAGE}: 获取视频源图像裁切参数
>> KILOVIEW G系列/M系列编码器产品，允许对视频图像进行高级的前置处理，其中一项为裁剪原始输入视频的部分，然后进行放大／缩小，
从而实现局部图像显示、局部放大／缩小等高级效果。
>> <b>注意：KILOVIEW U系列产品不支持本功能。</b>
>> 本操作可以获取当前视频源图像裁切的状态和参数。

##ARGUMENTS{POST}: 
无

##RETURNS{DEFAULT}:
	++Data{OBJ}:	裁切状态和参数
		**Enable{INT}:	0|1。0表示关闭裁切；1表示打开裁切
		**Left{NUM}:	如果Enable=1，表示裁切位置的左边起始位置（Pixel）
		**Top{NUM}: 如果Enable=1，表示裁切位置的顶部起始位置（Line）
		**Width{NUM}: 如果Enable=1，表示裁切的画面宽度（Pixel）
		**Height{NUM}: 如果Enable=1，表示裁切的画面高度（Line）
	~~
]]

local cgilua = require "cgilua"
require("kiloview.globalConfig")()
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )
if not _G.ENABLE_CONVERTER then
cgilua.put([[
{
	"Result":404,
	"Status":"ERROR: Device does not support source cropping method",
}]])
	return
end

local argPrefix = "avio.video"
local reqParams = {
	[argPrefix .. ".source_crop.enable"] = 0,
	[argPrefix .. ".source_crop.cropX"] = 0,
	[argPrefix .. ".source_crop.cropY"] = 0,
	[argPrefix .. ".source_crop.cropW"] = 0,
	[argPrefix .. ".source_crop.cropH"] = 0
}

local r, cc, gets = rpc.Call( "BUS.Get", reqParams )
if not r or not gets then
cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to get source cropping configurations",
}]])
else
cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data":{
		"Enable":]] .. gets[argPrefix .. ".source_crop.enable"] .. [[,
		"Left":]] .. gets[argPrefix .. ".source_crop.cropX"] .. [[,
		"Top":]] .. gets[argPrefix .. ".source_crop.cropY"] .. [[,
		"Width":]] .. gets[argPrefix .. ".source_crop.cropW"] .. [[,
		"Height":]] .. gets[argPrefix .. ".source_crop.cropH"] .. [[
	}
}]])
end
