------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=WWAN}: 获得当前无线宽带连接(WWan)的详细状态信息
##ARGUMENTS{GET,POST}: 
	@@DeviceObject: 可选参数。如果指定，则为网络接口的ObjectID（参见 ^^REF{getNetInterface.lua}, 类型为MODEM）；如果没有指定，则为默认的无线宽带连接(WWan)接口
	@@_LANG: 可选参数。指定StateReason返回参数的语言，默认为英文。
        %%cn: 中文
        %%en: 英文
    ~~

##RETURNS: 
	++Data{OBJ}:
		**Device: WWan设备名（如ppp0）
		**Object: WWan对象名
		**State: 当前WWan的连接状态，为以下值之一： 
            %%disabled: 被禁用
            %%off: 未连接
            %%connecting: 正在连接
            %%connected: 已连接
            %%error: 错误
        ~~

        **StateReason: 当WWan处于disabled/off/connecting/error状态时，返回处于该状态的原因（字符串，可由请求的_LANG参数来决定语言）
        **StateReasonCode{INT}: StateReason的代码
		**Capabilities: WWan的能力，可能为 CDMA/EVDO, GSM/UMTS/HSPA, LTE 三种值的一个或多个组合（之间用,分隔）,
		**IP: 当前获得的IP地址
		**Prefix{INT}: 以prefix形式表示的子网掩码（如24表示255.255.255.0, 16表示255.255.0.0）
		**Gateway: 默认网关
	~~
]]

local WWAN = require "M_WWanSet"
local cgilua = require "cgilua"
local cjson = require "cjson"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local objectId
local lang = "en"
local args = require("M_Request").VerifyAndGetArgs()
if args then
	objectId = args.DeviceObject
    lang = args._LANG or "en"
end

local device = WWAN.GetDefaultWWan(objectId)
if not device then
	cgilua.put([[
{
	"Result":500,
	"Status":"FAIL: Fault to get WWan device"
}
]])
	return
end

local r = WWAN.GetWWanDetails( device )
if not r then
	cgilua.put([[
{
	"Result":400,
	"Status":"FAIL: Fail to get WWan details"
}
]])

else
	local stateStr = "unknown"
	if device.State == 10 then
		stateStr = "disabled"
	elseif device.State <= 30 or device.State == 120 then
		stateStr = "off"
	elseif device.State < 80 then
		stateStr = "connecting"
	else
		stateStr = "connected"
	end
	
-- 	cgilua.put([[
-- {
-- 	"Result":200,
-- 	"Status":"OK",
-- 	"Data":{
-- 		"Device":"]] .. device.Name .. [[",
-- 		"Object":"]] .. device.Object .. [[",
-- 		"State":"]] .. stateStr .. [[",
--         "StateReason":"]] .. r.StateReason .. [[",
--         "StateReasonCode":]] .. r.StateReasonCode or -1 .. [[,
-- 		"Capabilities":"]] .. r.Capabilities .. [[",
-- 		"IP":"]] .. r.IP .. [[",
-- 		"Prefix":]] .. r.Prefix .. [[,
-- 		"Gateway":"]] .. r.Gateway ..[["
-- 	}
-- }
-- ]])

ret ={
	Result = 200,
	Status = "OK",
	Data = {
		Device = device.Name,
		Object = device.Object,
		State = stateStr,
		StateReason = r.StateReason,
		StateReasonCode = r.StateReasonCode or -1,
		Capabilities = r.Capabilities,
		IP = r.IP,
		Prefix = r.Prefix,
		Gateway = r.Gateway
	}
}

cgilua.put(cjson.encode(ret))

end


