------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=WIFI}: 获取WIFI连接配置参数
##ARGUMENTS{POST}: 
  @@DeviceObject: 可选参数。如果指定，则为网络接口的ObjectID（参见^^REF{getNetInterface.lua}）；如果没有指定，则为默认的Wifi接口
  @@Object{*}: 要获取Wifi连接信息的Connection Object名称（通过^^REF{listWifiConnection.lua}可以获得）

##RETURNS:
  ++Data{OBJ}:
  **DeviceObject: 网络接口的ObjectID
  **Object: Connection Object名称
  **Uuid: 该连接的Uuid
  **MAC: WiFi接口的MAC地址
  **SSID: 连接到的SSID
  **BSSID: 指定连接的BSSID (如未指定，BSSID为空)
  **AutoConnect{INT}: 自动连接选项
    %%1: 表示当在范围内找到这个热点，将自动连接
    %%0: 不自动连接，需要手动进行连接
  ~~
  **Security: WIFI连接的安全性。为以下选项之一：
		%%none-none: 无密码
	    %%none-open: WEP 40/128位密钥(开放式)
	    %%none-share: WEP 40/128位密钥(共享密钥)
	    %%none-open-128: WEP 128位密码句(开放式)
	    %%none-share-128: WEP 128位密码句(共享密钥)
	    %%ieee8021x: IEEE 802.1X动态WEP（暂不支持）
		%%wpa-psk: WPA/PSK或WPA2/PSK
	    %%wpa-none: 面向AdHoc的WPA/WPA2
  ~~
  **Psk: 连接密码

  **DHCP{INT,*}: 
    %%1: 使用DHCP获取地址
    %%0: 手动指定IP地址
  ~~
  **IP: 手动指定的IP地址（如果DHCP为1，该参数无效）
  **Mask: 手动指定子网掩码（如果DHCP为1，该参数无效）
  **Prefix: prefix形式的子网掩码（如果DHCP为1，该参数无效）
  **Gateway: 手动指定默认网关（如果DHCP为1，该参数无效）
  **Dns1: 指定DNS1（如果DHCP为1，该参数无效）
  **Dns2: 指定DNS2（如果DHCP为1，该参数无效）
  ~~
]]

local WIFI = require "M_WifiSet"
local cgilua = require "cgilua"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Object or args.Object == "" then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid request arguments"
}
]])
	return
end

local objectId = args.DeviceObject
if not objectId or objectId == "" or objectId == "/" then
	local device = WIFI.GetDefaultWifi()
	if not device then
		cgilua.put([[
	{
		"Result":500,
		"Status":"FAIL: Fault to get Wifi device"
	}
	]])
		return
	end
	objectId = device.Object
end

local detail,msg = WIFI.GetConnection( objectId, args.Object )

cgilua.put([[
{
	"Result":]] .. (detail and "200" or "400") .. [[,
	"Status":"]] .. (msg or "OK") .. [["
]])

if type(detail) == "table" then
	cgilua.put( [[, "Data": {
]])

	local k,v
	local counter = 0
	for k, v in pairs(detail) do
		if counter > 0 then
			cgilua.put( "," )
		end

		cgilua.put( '"' .. k .. '":' .. ( type(v) == "string" and '"' or '' ) .. v .. ( type(v) == "string" and '"\n' or '\n' ) )
		counter = counter + 1
	end

	cgilua.put( [[
	}
]])
end

cgilua.put([[
}
]])



