------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=IOLED}: 控制LED显示状态
>> 如果需要用户自行控制LED显示，需要先调用 ^^REF{setLEDUserControl.lua} 打开用户控制LED开关。

##ARGUMENTS{GET,POST}: 
	@@ID{STR,*}: 指定要控制的LED ID，定义为：
		%%led1: (从左边开始)第1个LED灯
		%%led2: (从左边开始)第2个LED灯
		%%led3: (从左边开始)第3个LED灯
		%%led4: (从左边开始)第4个LED灯
	~~

	@@Status{STR,*}: 指定LED的状态，可以指定为以下的状态：
		%%OFF: 熄灭
		%%ON: (绿色)常亮，等同于GREEN.ON
		%%BLINK: (绿色)闪烁，等同于GREEN.BLINK
		%%GREEN.ON: 绿色常亮
		%%GREEN.BLINK: 绿色闪烁
		%%RED.ON: 红色常亮
		%%RED.BLINK: 红色闪烁
		%%YELLOW.ON: 黄色常亮
		%%YELLOW.BLINK: 黄色闪烁
	~~

##RETURNS{DEFAULT}: 
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.ID or args.ID == "" or not args.Status or args.Status == "" then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid request arguments"
}
]])
	return
end

args.Status = string.upper(args.Status:gsub("%.",":"))

local r,x = rpc.Call( "BUS.LEDControl", args.ID, args.Status )

cgilua.put([[
{
	"Result":]] .. (r and "200" or "400") .. [[,
	"Status":"]] .. (x and "OK" or "ERROR: Fail to set LED user control") .. [["
}
]])

