------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=WIFI}: 获得WIFI扫描的SSID列表
##ARGUMENTS{GET,POST}: 
	@@Object: 可选参数。如果指定，则为网络接口的ObjectID（参见^^REF{getNetInterface.lua}）；如果没有指定，则为默认的Wifi接口

##RETURNS: 
	++Data{OBJ}:
    	**Device: Wifi设备名 (如wlan0)
		**Object: Wifi对象名
		++SSID_list{ARR}:   每一项为一个扫描到的SSID信息，内容如下：
			++{OBJ}:	
                **SSID: SSID名称,  
				**Strengh{NUM}: 0-100，信号强度
				**Nosafe{NUM}: 0或1。Nosafe为1表示该热点不需要密码（不安全）
			~~
		~~
	~~
]]
local cgilua = require "cgilua"
local WIFI = require "M_WifiSet"

local objectId
local args = require("M_Request").VerifyAndGetArgs()
if args then
	objectId = args.Object
end

local device = WIFI.GetDefaultWifi(objectId)

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

if not device then
	cgilua.put([[
{
	"Result":500,
	"Status":"FAIL: Fault to get Wifi device"
}
]])
else
	local deviceName = device.Name
	local deviceObj = device.Object
	local ssidList = WIFI.GetWifiScanlist( deviceObj )
	if not ssidList then
		cgilua.put([[
{
	"Result":500,
	"Status":"FAIL: Fault to get SSID list"
}
]])
	else
		cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data": {
		"Device":"]] .. deviceName .. [[",
		"Object":"]] .. deviceObj .. [[",
		"SSID_list":[
]] )

		local k,v
		local cc = 0
		for k,v in pairs( ssidList ) do
			if cc > 0 then
				cgilua.put( ", " )
			end
			
			cgilua.put([[
			{ "SSID":"]] .. v.Ssid .. [[",
			"Strength":]] .. math.floor(v.Strength) .. [[,
			"Nosafe":]] .. ( (not v.security or v.security["OPEN"]) and 1 or 0 ) .. [[
			}
			]] )
			
			cc = cc + 1
		end

		cgilua.put([[
		]
	}
}
]])
	end
end
