------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=WWAN}: 列出所有已保存的WWan连接配置列表
##ARGUMENTS{GET,POST}: 
	@@DeviceObject: 可选参数。如果指定，则为网络接口的ObjectID（参见 getNetInterface.lua）；如果没有指定，则为默为的WWan接口

##RETURNS: 
	++Data{OBJ}:
		**Device: WWan设备名(如ppp0)
		**Object: WWan对象名
		++CONNECTION_list{ARR}: 每一项为一个已保存的WWan连接项，内容如下：
			++{OBJ}:
            	**Title: 连接名称  
				**Type: 连接的类型
                    %%gsm: 对于GSM/HSPA/LTE连接，标识为gsm
                    %%cdma:对于cdma类型，标识为cdma
                    %%other: 其它未知的类型
                ~~

				**IsCurrent: 0|1,该连接是否为当前正在使用的连接。1为true，0为false

				**AddressResolve: 地址获取方式
                    %%dhcp: DHCP[对于WWan来说，一般皆为dhcp]
                    %%manual: 手动 
                ~~

				**Object: 连接的对象名称。注意这个名称，这是后续对该连接进行操作的依据，形式为 /org/freedesktop/NetworkManager/Connections/3 。
				**Uuid: 该连接的Uuid。如果需要修改一个WWan连接，该Uuid应该提供作为修改连接的参数

                **GSM_APN:      如果类型为gsm，该值代表GSM网络的APN
                **GSM_Number:   如果类型为gsm，该值代表GSM网络拨号号码
                **GSM_User:     如果类型为gsm，该值代表GSM网络拨号用户名
                **GSM_Password: 如果类型为gsm，该值代表GSM网络拨号用户密码
                **GSM_Pin:      如果类型为gsm，该值为GSM SIM卡PIN
                **CDMA_Number:  如果类型为cdma，该值代表CDMA网络的拨号号码
                **CDMA_User:    如果类型为cdma，该值代表CDMA网络的拨号用户名
                **CDMA_Password:如果类型为cdma，该值代表CDMA网络的拨号用户密码
			~~
		~~
	~~
]]

local cgilua = require "cgilua"
local WWAN = require "M_WWanSet"

local objectId
local args = require("M_Request").VerifyAndGetArgs()
if args then
	objectId = args.DeviceObject
end

local device = WWAN.GetDefaultWWan(objectId)
local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

if not device then
	cgilua.put([[
{
	"Result":500,
	"Status":"FAIL: Fault to get WWan device"
}
]])
	return
end

local conns = WWAN.GetConnections( device.Object )
if not conns then
	cgilua.put([[
{
	"Result":500,
	"Status":"FAIL: Fault to get connection list"
}
]])
	return
end

cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data": {
		"Device":"]] .. device.Name .. [[",
		"Object":"]] .. device.Object .. [[",
		"CONNECTION_list":[
]] )

		local k,v
		local cc = 0
		for k,v in pairs( conns ) do
			if cc > 0 then
				cgilua.put( ", " )
			end
			
			cgilua.put([[
			{ "Title":"]] .. v.title .. [[",
			"Type":"]] .. v.connType .. [[",
			"IsCurrent":]] .. (v.isCurrent and 1 or 0 ) .. [[,
			"AddressResolve":"]] .. v.addrResolv .. [[",
			"Object":"]] .. v.connObj .. [[",
			"Uuid":"]] .. v.uuid .. [[",
            "GSM_APN":"]] .. v.GSM_APN .. [[",
            "GSM_Number":"]] .. v.GSM_Number .. [[",
            "GSM_User":"]] .. v.GSM_User .. [[",
            "GSM_Password":"]] .. v.GSM_Password .. [[",
            "GSM_Pin":"]] .. v.GSM_Pin .. [[",
            "CDMA_Number":"]] .. v.CDMA_Number .. [[",
            "CDMA_User":"]] .. v.CDMA_User .. [[",
            "CDMA_Password":"]] .. v.CDMA_Password .. [["
			}
			]] )
			
			cc = cc + 1
		end

		cgilua.put([[
		]
	}
}
]])

