
------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=AV-SOURCE}: 设置音频增益
##ARGUMENTS{GET,POST}: 
    @@Source: 指定要获设定频增益的信号源名称（来自于 ^^REF{getAudioSource.lua} 中返回的 Sources 列表）
            >> 注：不同产品型号，数字音频源的名称可能不一样，例如HDMI/SDI/...。
            >> 为了软件编程的一致，无论HDMI/SDI/...哪种
            音频源，均可以用DIGITAL来代表；同样，模拟音频源可能为LINE/MIC/...，可以用ANALOG来代表。

			>> <b>注意：</b>当前设备仅支持修改模拟音频输入的增益！对于非模拟音频输入的增益，修改无效。
	
	@@Gain{NUM}: 增益值(db)，当前设备允许的有效值范围从 -17.25 到 6.0

##RETURNS{DEFAULT}: 
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
local source = "LINE"
if args and args.Source then
	source = args.Source
end

local gain = nil

if args and args.Gain then
	gain = tonumber(args.Gain) or 0
end

if not gain then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid request arguments"
}
]])
	return
end

source = source:lower()
local r,x
if source == "line" or source == "analog" then
	r,x = rpc.Call( "BUS.Set", { ["avio.audio.line_gain"] = string.format("%.2f",gain) } )
elseif source == "mic" then
	r,x = rpc.Call( "BUS.Set", { ["avio.audio.mic_gain"] = string.format("%.2f",gain) } )
end

cgilua.put([[
{
	"Result":]] .. (r and "200" or "400") .. [[,
	"Status":"]] .. (x and "OK" or "ERROR: Fail to set audio gain") .. [["
}
]])

