------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[
##ABSTRACT{G=ETHERNET}: 设置Ethernet接口的Failsafe IP地址
>>{BOLD}关于Failsafe IP地址：
>> 在Ethernet的IP地址配置过程中，难免存在以下情况：
>> (1) 忘记手动配置的IP地址
>> (2) 启用DHCP，但DHCP无法得到有效的地址
>> (3) 启用DHCP，虽然得到地址但无法获知该地址是多少
>> 为解决以上问题，您可以为Ethernet配置一个Failsafe（简单、易记）地址。当以上情况发生的时候，您可以通过访问
Failsafe地址继续管理设备。

##ARGUMENTS{GET,POST}: 
	@@DeviceObject: 可选参数。如果指定，则为网络接口的ObjectID（参见^^REF{getNetInterface.lua}）；如果没有指定，则为默认的Ethernet接口
	@@Enable{INT}: Failsafe地址功能开启。0表示关闭，1表示开启
	@@IP{STR}: Failsafe IP地址
	@@Netmask{INT}: Failsafe子网掩码 (如255.255.255.0)

##RETURNS: 
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid arguments"
}
]])
	return
end
local sets = {}

if args.IP and args.IP ~= "" then
	sets["hostapm.localLink.ip"] = args.IP
end

if args.Netmask and args.Netmask ~= "" then
	sets["hostapm.localLink.netmask"] = args.Netmask
end

if args.Enable and tonumber(args.Enable) then
	if tonumber(args.Enable) == 0 then
		sets["hostapm.localLink.enable"] = "false"
	else
		sets["hostapm.localLink.enable"] = "true"
	end
end
local r,x = rpc.Call( "BUS.Set", sets )
if r and x then
	cgilua.put([[
{
	"Result":200,
	"Status":"OK"
}
]])

else
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to set arguments"
}
]])

end

