------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=ENCODING}: 设置MotionJPEG编码/Streaming的参数。

>>    MotionJPEG编码/Streaming在所有KILOVIEW编码产品中均是默认开启的，以辅助于视频抓拍
图像用于预览。但MotionJPEG可以配置成不同的模式，如仅用于预览（图像较小、帧率5fps），或
与主码流相同，或与子码流相同，以供不同的应用场景。

##ARGUMENTS{POST}: 
    @@Type: 设置MotionJPEG的配置模式
        %%preview: 仅预览模式（720x405大小，5fps）
        %%main: 图像大小与主码流相同（帧率可设置）
        %%sub: 图像大小与子码流相同（帧率可设置）
    ~~

    @@Quality{INT}: 1-99, 指定JPEG的编码图像质量(值越大，编码质量更好，对应码率也会升高<b>很多</b>! 要控制码率，建议值不要超过80
    @@Chrome{INT}: 0|1, 0表示彩色，1表示灰度
    @@FrameRateMode: 编码帧率模式
        %%default: full表示全帧率，即输入视频帧率为多少，编码帧率为多少
        %%half: half表示当输入视频的帧率 >=50 时，自动降半帧率
        %%custom: custom表示手动指定帧率
    ~~
    @@FrameRate{NUM}: 指定的编码帧率（如果FrameRateMode为default/half，则不必要指定FrameRate值）
    @@BindAudioChannel{INT}: 绑定的音频编码引擎通道号, -1表示无音频绑定

##RETURNS{DEFAULT}: 
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args then
	cgilua.put([[
{
	"Result":201,
	"Status":"OK: But without any arguments"
}
]])
	return
end

local sets={}
local cc = 0
if args.Type and args.Type ~= "" then
	cc = cc + 1
	sets[ "encoder.snap_type" ] = args.Type
end

if args.Quality and args.Quality ~= "" then
	cc = cc + 1
	sets[ "encoder.stream[2].video.quality" ] = tonumber(args.Quality)
end

if args.Chrome and args.Chrome ~= "" then
	cc = cc + 1
	sets[ "encoder.stream[2].video.chrome" ] = tonumber(args.Chrome)
end

local setFrMode = nil
if args.FrameRateMode and args.FrameRateMode ~= "" then
	cc = cc + 1
	setFrMode = args.FrameRateMode:lower()
	sets[ "encoder.stream[2].video.framerate_mode" ] = setFrMode
end

if args.FrameRate and args.FrameRate ~= "" then
	if setFrMode and setFrMode ~= "full" and setFrMode ~= "half" then
		cc = cc + 1
		sets[ "encoder.stream[2].video.framerate" ] = tonumber(args.FrameRate)
	end
end

if args.BindAudioChannel and args.BindAudioChannel ~= "" then
	cc = cc + 1
	sets[ "streamer.stream[2].audioChannel" ] = tonumber(args.BindAudioChannel)
end

if cc == 0 then
	cgilua.put([[
{
	"Result":201,
	"Status":"OK: But no any arguments specified"
}
]])
	return
end

local r,x = rpc.Call( "BUS.Set", sets )
if not r or not x then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to configure the JPEG streaming"
}
]])
else
	cgilua.put([[
{
	"Result":200,
	"Status":"OK"
}
]])
end

