------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=ONVIF}: 修改Onvif用户信息（及密码）
##ARGUMENTS{GET,POST}: 
	@@Username{*,STR}: 要修改的用户名
	@@Password{STR}: 新密码
 	@@Name{STR}: 用户的友好名称 (为空时表示不修改)
	@@Role{STR}: 用户角色 (为空时，默认角色为 <b>user</b>)
		%%administrator: 管理员
		%%operator: Operator
		%%user: 普通用户
	~~
	>> 注意：admin用户是系统内建的管理员用户，它的用户角色不可以被修改（默认为 administrator）

##RETURNS: 
	++Data{OBJ}:
		**Reason{STR}: 如果返回错误码为405，Reason说明修改用户失败的原因：
			%%invalid: 无效的用户名
			%%notexist: 指定的用户不存在
		~~
	~~

##NOTES:
>>{BOLD} 安全性说明：
>> 调用该API必须通过HTTP的身份认证。身份认证用户名和密码同Web登录。

##__OTHERS:
>> 在本页面的API Call测试中，给定的用户名为admin，密码为admin（指定HTTP Request header中Authorization:"Basic YWRtaW46YWRtaW4="）。如果需要修改，请在下面指定：
<script type="text/javascript">
ajaxAuth = "Basic YWRtaW46YWRtaW4=";
function change_auth(val)
{
	ajaxAuth = val;
	alert('DONE');
}
</script>
>> 修改Authorization值: <input type="text" id="test_auth" size="28" value="Basic YWRtaW46YWRtaW4=" /> <input type="button" value="修改" onclick='change_auth($("#test_auth")[0].value);' />
<br /><br />
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Username or args.Username == "" then
	cgilua.put([[
	{
		"Result":400,
		"Status":"ERROR: Invalid arguments"
	}]] )
	return
end

local MAX_USERS=10

local function changeOnvifUser( user, pass, name, role )
	local sets = {}
	if user == "admin" then
		local valid
		if name and name ~= "" then
			sets[ "onvif.adminName" ] = name
			valid = true
		end

		if pass and pass ~= "" then
			sets[ "onvif.adminPassword" ] = pass
			valid = true
		end
	elseif type(user) ~="string" then
		return false, "invalid"
	else
		local gets = {}
		local i
		
		for i=0, MAX_USERS-1 do
			gets[ "onvif.users[" .. i .. "].username" ] = ""
		end

		local r,cc
		r,cc,gets = rpc.Call( "BUS.Get", gets )
		if not r or not gets then
			return false, "system"
		end

		local validSpace = -1
		for i=0, MAX_USERS-1 do
			if gets[ "onvif.users[" .. i .. "].username" ] == user then
				validSpace = i
				break
			end
		end
		
		if validSpace >= 0 then
			if pass and pass ~= "" then
				sets[ "onvif.users[" .. validSpace .. "].password" ] = pass
			end

			if name and name ~= "" then
				sets[ "onvif.users[" .. validSpace .. "].name" ] = name
			end

			if role and role ~= "" then
				sets[ "onvif.users[" .. validSpace .. "].role" ] = role
			end
		else
			return false, "notexist"
		end
	end
	rpc.Call( "BUS.Set", sets )
	return true
end

local code, reason = changeOnvifUser( args.Username, args.Password, args.Name, args.Role )
if code then
	cgilua.put([[
	{
		"Result":200,
		"Status":"OK"
	}]] )
else
	cgilua.put([[
	{
		"Result":405,
		"Status":"ERROR: Fail to set user",
		"Data":{
			"Reason":"]] .. (reason or "unknown") .. [["
		}
	}]] )
end

