------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------
--[[

##ABSTRACT{G=IMAGE}: 设置视频后级（编码）图像裁切参数
>> 后级图像裁切，是相对于前级裁切（参考 ^^REF{getSourceCrop.lua} / ^^REF{setSourceCrop.lua}）而言的。后级图像裁切对KILOVIEW全系列编码器产品适用。
>> 后级图像裁切，是在视频即将进行编码时，可以对经过前级处理后的图像（对于G/M系列产品）或直接输入的图像（对于U系列产品）进行裁切，从而实现局部
感兴趣区域的编码。如果对于G/M系列产品，这也许和前级裁切功能有所重复，但也可以组合前级裁切完成更高级的图像处理，当然也可以只考虑二者之一的功能开启；对于
U系列产品，这是实现图像裁切功能的手段。

##ARGUMENTS{POST}: 
@@Enable{INT}: 0|1。可选。0表示关闭裁切；1表示打开裁切
@@Left{NUM}:  可选。裁切位置的左边起始位置（Pixel）
@@Top{NUM}:   可选。裁切位置的顶部起始位置（Line）
@@Width{NUM}: 可选。裁切的画面宽度（Pixel）
@@Height{NUM}: 可选。裁切的画面高度（Line）


##RETURNS{DEFAULT}:

##NOTES:
	>> 1. 如果裁切的起始位置(Left,Top)超出实际画面的有效大小，将产生不可预估的结果；
	>> 2. 如果起始位置加指定的宽度/高度超出实际画面的有效大小，实际裁切的宽度／高度将自动进行调整以适应实际画面大小。
	>> 3. 最终画面进行裁切时，Left/Width的值将以4像素为倍数进行近似约束，Top/Height的值将以2行为倍数进行近似约束，所以实际裁切的大小与指定的大小可能存在细微出入，请注意。
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid arguments"
}
]])
	return
end

local argPrefix = "avio.video"
local sets = {}

sets[ argPrefix .. ".out_crop.enable" ] = tonumber(args.Enable)	--May be nil but no care.
sets[ argPrefix .. ".out_crop.cropX" ] = tonumber(args.Left)	--May be nil but no care.
sets[ argPrefix .. ".out_crop.cropY" ] = tonumber(args.Top)	--May be nil but no care.
sets[ argPrefix .. ".out_crop.cropW" ] = tonumber(args.Width)	--May be nil but no care.
sets[ argPrefix .. ".out_crop.cropH" ] = tonumber(args.Height)	--May be nil but no care.

local r,x = rpc.Call( "BUS.Set", sets )
if not r or not x then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to set picture cropping parameters"
}
]])
else
	cgilua.put([[
{
	"Result":200,
	"Status":"OK"
}
]])
end
