------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------
--[[

##ABSTRACT{G=IMAGE}: 设置视频源图像裁切参数
>> KILOVIEW G系列/M系列编码器产品，允许对视频图像进行高级的前置处理，其中一项为裁剪原始输入视频的部分，然后进行放大／缩小，
从而实现局部图像显示、局部放大／缩小等高级效果。
>> <b>注意：KILOVIEW U系列产品不支持本功能。</b>
>> 本操作可以设定当前视频源图像裁切的状态和参数。

##ARGUMENTS{POST}: 
@@Enable{INT}: 0|1。可选。0表示关闭裁切；1表示打开裁切
@@Left{NUM}:  可选。裁切位置的左边起始位置（Pixel）
@@Top{NUM}:   可选。裁切位置的顶部起始位置（Line）
@@Width{NUM}: 可选。裁切的画面宽度（Pixel）
@@Height{NUM}: 可选。裁切的画面高度（Line）


##RETURNS{DEFAULT}:

##NOTES:
	>> 1. 如果裁切的起始位置(Left,Top)超出实际画面的有效大小，将产生不可预估的结果；
	>> 2. 如果起始位置加指定的宽度/高度超出实际画面的有效大小，实际裁切的宽度／高度将自动进行调整以适应实际画面大小。
	>> 3. 最终画面进行裁切时，Left/Width的值将以4像素为倍数进行近似约束，所以实际裁切的大小与指定的大小可能存在细微出入，请注意。
]]

local cgilua = require "cgilua"
require("kiloview.globalConfig")()
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

if not _G.ENABLE_CONVERTER then
cgilua.put([[
{
	"Result":404,
	"Status":"ERROR: Device does not support source cropping method",
}]])
	return
end

local args = require("M_Request").VerifyAndGetArgs()
if not args then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid arguments"
}
]])
	return
end

local argPrefix = "avio.video"
local sets = {}

sets[ argPrefix .. ".source_crop.enable" ] = tonumber(args.Enable)	--May be nil but no care.
sets[ argPrefix .. ".source_crop.cropX" ] = tonumber(args.Left)	--May be nil but no care.
sets[ argPrefix .. ".source_crop.cropY" ] = tonumber(args.Top)	--May be nil but no care.
sets[ argPrefix .. ".source_crop.cropW" ] = tonumber(args.Width)	--May be nil but no care.
sets[ argPrefix .. ".source_crop.cropH" ] = tonumber(args.Height)	--May be nil but no care.

local r,x = rpc.Call( "BUS.Set", sets )
if not r or not x then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to set source cropping parameters"
}
]])
else
	cgilua.put([[
{
	"Result":200,
	"Status":"OK"
}
]])
end
