------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------
--[[

##ABSTRACT{G=IMAGE}: 设置视频前级缩放
>> KILOVIEW G系列/M系列编码器产品，允许对视频图像进行高级的前置处理，其中一项为视频的前级缩放。
>> <b>注意：KILOVIEW U系列产品不支持本功能。</b>
>> 所谓<b>前级缩放</b>，是指视频从输入接口采集到以后，在送入视频编码之前，设备上的Image Processor
可以预先将视频放大或缩小至特定的尺寸。配合 ^^REF{setSourceCrop.lua} 的源视频裁切，可以实现图像的
局部放大/缩小特殊功能；或者即使不需要裁切，仍然可以进行视频的 up/down convension（例如：将输入的
1080p视频转换为720p，或者将1080p60的视频转换为1080p25，……）。

##ARGUMENTS{POST}: 
@@Format: 设定前级缩放的格式：
	%%default:	默认格式，不进行前级缩放。这意味着，输入的视频是什么格式，送入编码就是什么格式；
				>> 如果在前级存在源视频裁切（参见^^REF{getSourceCrop.lua} / ^^REF{setSourceCrop.lua}），
				那么送入编码的视频大小就是裁切后的实际大小。
	%%VIDOUT_1920x1080p60: 缩放至1920x1080@60fps
	%%VIDOUT_1920x1080p59: 缩放至1920x1080@59.94fps
	%%VIDOUT_1920x1080p50: 缩放至1920x1080@50fps
	%%VIDOUT_1920x1080p30: 缩放至1920x1080@30fps
	%%VIDOUT_1920x1080p25: 缩放至1920x1080@25fps
	%%VIDOUT_1280x720p60: 缩放至1280x720@60fps
	%%VIDOUT_1280x720p50: 缩放至1280x720@50fps
	%%VIDOUT_1280x720p30: 缩放至1280x720@30fps
	%%VIDOUT_1280x720p25: 缩放至1280x720@25fps
	%%VIDOUT_640x480p60: 缩放至640x480@60fps
	%%VIDOUT_1024x768p60: 缩放至1024x768@60fps
	%%VIDOUT_1280x1024p60: 缩放至1280x1024@60fps
	%%VIDOUT_1440x900p60: 缩放至1440x900@60fps
	%%...Others: 设备本身还支持一些其它的格式模式，以上列出的是常用的格式。如果需要其它格式，请向技术支持索取。

@@Mode{STR}: 画面显示模式，当Format不为default时，可以指定：
	%%default: 默认（与OUTCTRL_ASPECT相同）
	%%OUTCTRL_ASPECT:	自动拉伸且维持纵横比。如果输入视频与输出视频的比例不同
						>> （例如：输入视频16:9，而输出视频4:3）
						>> 为了维持画面比例，空缺部分将填充黑色。
	%%OUTCTRL_P2P: Point-to-Point（点对点）显示，即输入视频的每一个像素点，对应输出视频的每一个像素点，居中显示。
					这意味着画面不会进行放大或缩小，如果输入视频小于输出视频，那么画面将显示在输出视频的中央，周围
					空缺部分填充黑色；反之，将裁切输入视频的中央部分用于显示。
	%%OUTCTRL_STRETCH: 拉伸至全屏，如果输入/输出视频的比例不同，不会维持比例（因此可能会造成视频比例失真）
~~

##RETURNS{DEFAULT}:
]]

local cgilua = require "cgilua"
require("kiloview.globalConfig")()
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

if not _G.ENABLE_CONVERTER then
cgilua.put([[
{
	"Result":404,
	"Status":"ERROR: Device does not support source scaling method",
}]])
	return
end

local args = require("M_Request").VerifyAndGetArgs()
if not args then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid arguments"
}
]])
	return
end

local argPrefix = "avio.video"
local sets = {}
if args.Format and args.Format ~= "" then
	sets[ argPrefix .. ".spec_out.format" ] = args.Format
end

if args.Mode and args.Mode ~= "" then
	sets[ argPrefix .. ".spec_out.mode" ] = args.Mode
end

local r,x = rpc.Call( "BUS.Set", sets )
if not r or not x then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to set source scaling parameters"
}
]])
else
	cgilua.put([[
{
	"Result":200,
	"Status":"OK"
}
]])
end
