------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=ENCODING}: 添加流媒体服务 / 设置指定码流、指定媒体服务的参数 / 删除一个流媒体服务

>>{BOLD} 媒体服务，是指RTSP/RTMP/HLS/...等服务。
>> KILOVIEW编码器产品，支持多种流媒体服务功能，并且每一项功能均可设定开启/关闭，
或动态添加/删除服务[ 说明：RTSP服务和KMP服务为固定存在的服务，不能添加/删除]。

>>{BOLD} 注意：产品内部会有限定支持服务的数量(当前为8)。
>> 所以，新增媒体服务时，如果到达服务支持数量上限，返回值将为<b>406</b>，错误信息为"LIMIT: ..."

##ARGUMENTS{POST}: 
    @@Stream{*}: 指定要设定媒体服务的码流。
        %%main: 主码流
        %%sub: 子码流
		%%jpeg: Motion JPEG码流
    ~~
    
    @@ID{*}: 媒体服务ID。
        >> 如果修改某个媒体服务参数，该参数应通过 ^^REF{getStreamService.lua} API获取；
        >> 如果要<b>新增</b>一个媒体服务，该值请设定为 <b>"ADD-NEW"</b>

    @@Type: 
        >> 当更改媒体服务类型时，值可选。
		>> 当要求<b>删除</b>一个服务时，该值必须指定为NONE（参见下方描述）
        >> 当<b>新增</b>一个媒体服务时，该值必须指定。
        >> 值可能为:
            %%Rtsp_server: RTSP服务
            %%Kmp_pullingServer: KILOVIEW Media Protocol的拉取服务
            %%Rtmp_push: RTMP推送服务
            %%Rtp_push: RTP推送或组播服务
            %%Kmp_push: KILOVIEW Media Protocol的推送服务
            %%Sproxy_push: 特定OEM客户的推送服务，一般情况下请勿考虑
            %%TS_push: TS/UDP推送或组播服务
            %%HLS: HLS(HTTP Live Streaming)服务
			%%NONE: <b>这是一个特殊值</b>。如果指定Type为NONE，表示<b>移除</b>该服务。注意，对于Rtsp_server/Kmp_pullingServer类型的服务，是不可以移除的。
        ~~

    @@G=Rtsp_server: 如果服务类型为 "Rtsp_server"，可提供本组以下参数：

	@@Rtsp_server.port{INT,G=Rtsp_server}: RTSP服务端口，默认为554
	@@Rtsp_server.session{G=Rtsp_server}: 会话ID, RTSP服务的SessionID
	@@Rtsp_server.ssm{INT,G=Rtsp_server}: 0|1, 是否开启RTSP Multicast服务。1表示开启，0表示关闭。
	@@Rtsp_server.multicast_addr{G=Rtsp_server}: RTSP组播的Multicast地址
	@@Rtsp_server.multicast_video_port{INT,G=Rtsp_server}: multicast视频端口
	@@Rtsp_server.multicast_audio_port{INT,G=Rtsp_server}: multicast音频端口
	@@Rtsp_server.multicast_ttl{INT,G=Rtsp_server}: multicast TTL值

    @@G=Kmp_pullingServer: 如果服务类型为 "Kmp_pullingServer"，可提供本组以下参数：

	@@Kmp_pullingServer.enabled{INT,G=Kmp_pullingServer}: 0|1，是否开启KMP拉流服务。0关闭，1开启。

    @@G=Rtmp_push:  如果服务类型为 "Rtmp_push"，可提供本组以下参数：
	@@Rtmp_push.enabled{INT,G=Rtmp_push}: 0|1, 是否开启RTMP推流。0关闭，1开启。
	@@Rtmp_push.pubPoint{G=Rtmp_push}: RTMP推流的发布地址URL
	@@Rtmp_push.username{G=Rtmp_push}: RTMP推流身份认证用户名
	@@Rtmp_push.password{G=Rtmp_push}: RTMP推流身份认证用户密码
	@@Rtmp_push.timeout{INT,G=Rtmp_push}: RTMP连接超时（单位：秒）
	@@Rtmp_push.reconnect_interval{INT,G=Rtmp_push}: RTMP断开后重连的间隔时间（单位：秒）

    @@G=Rtp_push: 如果服务类型为 "Rtp_push"，可提供本组以下参数：
	@@Rtp_push.enabled{INT,G=Rtp_push}: 0|1, 是否开启Rtp推送功能。0关闭，1开启。
	@@Rtp_push.target_video_IP{G=Rtp_push}: 视频目标推送IP或组播IP
	@@Rtp_push.target_video_port{INT,G=Rtp_push}: 视频目标推送端口
	@@Rtp_push.target_audio_IP{G=Rtp_push}: 音频目标推送IP或组播IP
	@@Rtp_push.target_audio_port{INT,G=Rtp_push}: 音频目标推送端口
    @@Rtp_push.ttl{INT,G=Rtp_push}:  如果目标IP为Multicast地址，指定组播TTL值

    @@G=Kmp_push: 如果服务类型为 "Kmp_push"，可提供本组以下参数：
	@@Kmp_push.enabled{INT,G=Kmp_push}: 0|1, 是否开启KMP推送。0关闭，1开启。
	@@Kmp_push.pubPoint{G=Kmp_push}: KMP推流的发布地址URL
	@@Kmp_push.username{G=Kmp_push}: KMP推流身份认证用户名
	@@Kmp_push.password{G=Kmp_push}: KMP推流身份认证用户密码
	@@Kmp_push.timeout{INT,G=Kmp_push}: KMP连接超时（单位：秒）
	@@Kmp_push.reconnect_interval{INT,G=Kmp_push}: KMP断开后重连的间隔时间（单位：秒）

    @@G=TS_push: 如果服务类型为 "TS_push"，可提供本组以下参数：
	@@TS_push.enabled{INT,G=TS_push}: 0|1, 是否开启TS推送。0关闭，1开启。
    @@TS_push.address{G=TS_push}: 推送目标IP地址或组播IP
    @@TS_push.port{INT,G=TS_push}:推送端口
    @@TS_push.ttl{INT,G=TS_push}: 组播的TTL

    @@G=HLS: 如果服务类型为 "HLS"，可提供本组以下参数：
	@@HLS.enabled{INT,G=HLS}: 0|1, 是否开启HLS服务。0关闭,1开启。
	@@HLS.session{G=HLS}: HLS的会话ID。决定HLS的URL路径。HLS的最终访问路径为 http://&lt;ip address&gt;/&lt;session&gt;/playlist.m3u8
	@@HLS.segmentTime{INT,G=HLS}: HLS切片时长（近似值，单位毫秒），实际切片时长以该值为参照，但可能会因为编码数据的原因有所偏差
	@@HLS.maxSegments{INT,G=HLS}: 在一个m3u8的playlist中，最多同时允许存在的切片数。按HLS规范，该值必须 &gt;=3
}

##RETURNS: 
    ++Data{OBJ}:
        **ID: 返回新增，或当前修改媒体服务的ID。注：仅当成功执行操作时才会返回。
    ~~
]]

local validServices = {
	["SRT_push"] = {
		enabled = "",
		srt_connect_mode = "",
		address = "",
		port = "",
		srt_latency = "",
		advanced_set = "",
		srt_encryption_key_len = "",
		srt_encryption_key = "",
		srt_bandwidth_overhead = "",
		srt_payload_size = "",
	},
	["Rtsp_server"] = {
		[ "port"	] = 0,
		[ "session"	] = "",
		[ "ssm"		] = 0,
		[ "multicast_addr"			] = "",
		[ "multicast_video_port"	] = 0,
		[ "multicast_audio_port"	] = 0,
		[ "multicast_ttl"			] = 0
	},

	["KMP_pullingServer"] = {
		[ "enabled"			   ] = 0,
		[ "timeShift"		   ] = 0,
		[ "channel"			   ] = 0
	},

	["Rtmp_push"] = {
		[ "enabled"			   ] = 0,
		[ "pubPoint"		   ] = "",
		[ "username"		   ] = "",
		[ "password"		   ] = "",
		[ "timeout"			   ] = 0,
		[ "reconnect_interval" ] = 0
	},

	["Rtp_push"] = {
		[ "enabled"			  ] = 0,
		[ "target_video_IP"	  ] = "",
		[ "target_video_port" ] = 0,
		[ "target_audio_IP"	  ] = "",
		[ "target_audio_port" ] = 0,
		[ "ttl"				  ] = 127
	},

	["Kmp_push"] = {
		[ "enabled"			   ] = 0,
		[ "pubPoint"		   ] = "",
		[ "username"		   ] = "",
		[ "password"		   ] = "",
		[ "timeout"			   ] = 0,
		[ "reconnect_interval" ] = 0
	},

	["Sproxy_push"] = {
		[ "enabled"			   ] = 0,
		[ "pubPoint"		   ] = "",
		[ "username"		   ] = "",
		[ "password"		   ] = "",
		[ "timeout"			   ] = 0,
		[ "reconnect_interval" ] = 0
	},

	["TS_push"] = {
		[ "enabled" ] = 0,
		[ "targets[1].address" ] = "",
		[ "targets[1].port" ] = "",
		[ "targets[1].ttl" ] = 127,
	},

	["HLS"] = {
		[ "enabled"	    ] = 0,
		[ "session"	    ] = "",
		[ "segmentTime" ] = 0,
		[ "maxSegments" ] = 0
	},

	["JUNSHI"] = {
		[ "enabled"	   ] = 0,
		[ "host"	   ] = "",
		[ "port"	   ] = 0,
		[ "vender_key" ] = "",
		[ "secure_key" ] = ""
	},
	
	["Omnistream"] = {
		[ "enabled"	   ] = 0,
		[ "address"	   ] = "",
		[ "video_port" ] = 0,
		[ "audio_port" ] = 0,
		[ "error_correction" ] = ""
	},
	--TODO: SRT push
}

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Stream or args.Stream == "" or not args.ID or args.ID == "" then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid arguments"
}
]])
	return
end

local svcId = args.ID
local newSvc = false
if svcId:upper() == "ADD-NEW" then
	newSvc = true
end

local stream = 0
if args.Stream == "sub" then
	stream = 1
elseif args.Stream == "jpeg" then
	stream = 2
end

if newSvc then	--Seek the valid spaces
	if stream == 2 then --JPEG, not allowed now.
		cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Not allowed to add media service in JPEG stream"
}
]])
		return
	end

	if not args.Type or args.Type == "" then
		cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Must specify streaming 'Type' while add new stream service"
}
]])
		return
	elseif args.Type == "Rtsp_server" or args.Type == "Kmp_pullingServer" then
		cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: RTSP server or KMP pulling server is builtin service and could not add it"
}
]])
		return
	elseif args.Type == "Omnistream" then
		cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Omnistream is builtin service and could not add it"
}
]])
		return
	end

	local thisPrefix = "streamer.stream[" .. stream .. "].Dynamic_services"
	local gets = {
		[ thisPrefix .. "[0].serviceType" ] = "",
		[ thisPrefix .. "[1].serviceType" ] = "",
		[ thisPrefix .. "[2].serviceType" ] = "",
		[ thisPrefix .. "[3].serviceType" ] = "",
		[ thisPrefix .. "[4].serviceType" ] = "",
		[ thisPrefix .. "[5].serviceType" ] = "",
		[ thisPrefix .. "[6].serviceType" ] = "",
		[ thisPrefix .. "[7].serviceType" ] = ""
	}
	
	local r,done
	r,done,gets = rpc.Call( "BUS.Get", gets )
	if not r or not done or type(gets) ~= "table" then
		cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to check valid dynamic streams"
}
]])
		return
	else
		local validIndex
		local idx
		local i
		for i=0,7 do
			idx = thisPrefix .. "[" .. i .. "].serviceType"
			if gets[idx] == "" then
				validIndex = i
				break
			end
		end
	
		if validIndex then
			svcId = "Dynamic_services[" .. validIndex .. "]"
		else
			cgilua.put([[
{
	"Result":406,
	"Status":"LIMIT: Stream service count reach limit"
}
]])
			return
		end
	end
else	--Validate the ID.
	if svcId ~= "Rtsp_server" and svcId ~= "Kmp_pullingServer" and svcId ~= "Omnistream" and not svcId:match( "(Dynamic_services)%[[0-9]+%]" ) then
		cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid stream ID"
}
]])
			return
	end
end

if svcId == "Omnistream" then
	svcId = "Dynamic_services[8]"
	svcType = "Omnistream"
	args.Type = "Omnistream"
end

local pfx = "streamer.stream[" .. stream .. "]." .. svcId
local sets = {}
if svcId == "Rtsp_server" or svcId == "Kmp_pullingServer" then
	local k,v
	for k,v in pairs(args) do
		local argId = k:match( svcId .. "%.(.+)$" )
		if argId then
			local argChk = validServices[ svcId ]
			if argChk then
				if type(argChk[argId]) == "number" then
					sets[ pfx .. "." .. argId ] = tonumber(v) --or nil
				elseif type(argChk[argId]) ~= nil then
					sets[ pfx .. "." .. argId ] = v
				end
			end
		end
	end
else
	local svcType = args.Type
	if svcType and svcType ~= "" then
		if svcType == "NONE" then
			sets[ pfx .. ".serviceType"] = ""	--Drop it!
		elseif validServices[svcType] then
			sets[ pfx .. ".serviceType"] = svcType
		end
	end

	local k,v
	for k,v in pairs(args) do
		local dsvcId, argId = k:match( "(.+)%.(.+)$" )
		if dsvcId and argId then
			local argChk = validServices[ dsvcId ]
			if argChk then
				if dsvcId == "TS_push" and argId ~= "enabled" then
					argId = "targets[1]." .. argId
				end
				if type(argChk[argId]) == "number" then
					sets[ pfx .. "." .. dsvcId .. "." .. argId ] = tonumber(v)
				elseif type(argChk[argId]) ~= nil then
					sets[ pfx .. "." .. dsvcId .. "." .. argId ] = v
				end
			end
		end
	end
end

local r, x = rpc.Call( "BUS.Set", sets )
if r and x then
		cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data": {
		"ID":"]] .. svcId .. [["
	}
}
]])
else
		cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to set stream service",
}]])
end
