local upload = require "resty.upload"
local _M = {}

local function checkPartType(res)
	if type(res) == "table" then
		local idr = res[1]:lower()
		if idr == "content-disposition" then
			local fieldname, filename = res[2]:match('name=%"(.*)%".*filename=%"(.*)%"')
			if not fieldname then
				fieldname = res[2]:match('name=%"(.*)%"')
			end
			if not fieldname then
				return nil
			end
			if not filename then
				return "field", fieldname
			else
				return "file", fieldname, filename
			end
		elseif idr == "content-type" then
			return idr, res[2] or ""
		else
			return nil
		end
	else
		return nil
	end
end

function _M.GetUploadFile(args, loc, spec_name_ext)
	loc = loc or "/tmp"
	args = args or {}
	local form,err = upload:new(8192)
	if not form then
		return nil, "start content read fail:" .. (err or "")
	end
	form:set_timeout(5*1000)        --5secs timeout

	local fieldType, fieldName
	local srcFileName
	local curFile

	while true do
		local typ, res, err = form:read()
		if not typ then
			if curFile then
				curFile:close()
			end
			return nil, "read content fail"
		end

		if typ == "header" then
			local ftype,fname,filename = checkPartType(res)
			if ftype == "field" then
				fieldType = ftype
				fieldName = fname
				args[fieldName] = ""
			elseif ftype == "file" then
				fieldType = ftype
				fieldName = fname
				srcFileName = filename
				if not srcFileName or srcFileName == "" then
					if spec_name_ext then
						srcFileName = spec_name_ext
					else
						return nil, "invalid upload filename"
					end
				elseif spec_name_ext then
					if spec_name_ext:sub(1,1) == "." then --Just specified file-ext
						if srcFileName:sub(-#spec_name_ext) ~= spec_name_ext then
							local main_name = srcFileName:match("(.+)%.(.*)$")
							if main_name then
								srcFileName = main_name .. spec_name_ext
							else
								srcFileName = srcFileName .. spec_name_ext
							end
						end
					else
						srcFileName = spec_name_ext
					end
				end
			elseif ftype == "content-type" then
				if fieldType ~= "file" then
					if curFile then
						curFile:close()
					end
					return nil, "wrong Content-Type location"
				else
					local openerrormsg
					curFile, openerrormsg = io.open( loc .. "/" .. srcFileName, "wb" )
					if not curFile then
						return nil, "fail to create file"
					end
				end
			else
				filedType = nil
				fieldName = nil
				if curFile then
					curFile:close()
					curFile = nil
				end
			end
		elseif typ == "body" then
			if fieldType == "field" then
				args[fieldName] = args[fieldName] .. res
			elseif fieldType == "file" then
				if not curFile then
					return nil, "fail to save file"
				end
				curFile:write( res )
			end
		elseif typ == "part_end" then
			filedType = nil
			fieldName = nil
			if curFile then
				curFile:close()
				curFile = nil
			end
		elseif typ == "eof" then
			break
		end
	end

	if curFile then
		curFile:close()
		curFile = nil
	end

	if not srcFileName then
		return nil, "no file"
	end

	return srcFileName
end

return _M


