local rpc = require "kiloview.ringRpc"
local utils = require "wsapi.util"
local chkLang = require "kiloview.checkLang"
require "features"

local module_meta = {}

function module_meta:__index(key)
	if self.extends[key] then
		return self.extends[key]
	end

	if not ngx.ctx.controller_module then
		return nil
	end
	return ngx.ctx.controller_module[key]
end

function module_meta:__newindex(key,val)
	if type(val) == "function" then
		self.extends[key] = val
		return
	end

	if not ngx.ctx.controller_module then
		ngx.ctx.controller_module = {}
	end
	ngx.ctx.controller_module[key] = val
end

--Changed for new web compatible environment: MUST create a new object.
return function()
--{
--------------------------------------------------------
local _M = {
	utils = utils,
	rpc = rpc,
	exports = {},
	extends = {}
}

local kajax = require("kiloview.ajaxCall")()

--------------------------------------------------------
function _M.MasterCall( funcName, ... )
	return rpc.Call( funcName, ... )
end
--------------------------------------------------------
function _M.AjaxExport( funcName, callFunc, mimeType )
	kajax.ExportFunction( funcName, callFunc, mimeType )
end
--------------------------------------------------------
function _M.AjaxProcess()
	return kajax.ProcessingCall()
end
--------------------------------------------------------
function _M.Open( pageName )
	local pageLang = chkLang.Check()
	local langs = require("langs.language")(pageName, pageLang)
	_M.pageLang = pageLang
	_M.langs = langs
	
	local request = request
	if not request then
		request = require("cgilua").SAPI.Request
	end

	local isAdministrator
	local user = request.getUserAndPwd()

	if user == "administrator" or user == "root" then
		isAdministrator = true
	else
		isAdministrator = false
	end

	_M.isAdministrator = isAdministrator
	_M.currentUser = user
end

function _M.CheckPrivileges(...)
	return rpc.Call( "SERVER.CheckPrivileges", _M.currentUser, ... )
end

--------------------------------------------------------
function _M.GenerateScripts(...)
	return kajax.GenerateScripts( _M.pageLang, ... )
end
--------------------------------------------------------

if ngx then --For Nginx(openresty)
	setmetatable(_M, module_meta)
end

return _M

--}
end

