--Controller of userman
local _M = require( "controller.generic" )()
local cgilua = require "cgilua"

------------
local function ResizeIMAGE( id, w, h, a, tp_x, tp_y, tp_mode, tp_fuzz )
	if not id or id == "" then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_ARGUMENT .. [["}]]
	end

	local r,x,msg = _M.MasterCall( "BUS.ResizeIMAGE", id, w, h, a, tp_x, tp_y, tp_mode, tp_fuzz )
	if r then 
		if x then
			return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_RESIZE_DONE .. [[", "Refresh":1 }]]
		else
			return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_RESIZE_FAIL .. " - " .. (msg or "Unknown").. [[" }]]
		end
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_RESIZE_FAIL .. [["}]]
	end
end

local function SetIMAGEAlpha( id, alpha )
	if not id or id == "" then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_ARGUMENT .. [["}]]
	end

	local r,x,msg = _M.MasterCall( "BUS.SetIMAGEAlpha", id, alpha )
	if r then 
		if x then
			return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_SET_ALPHA_DONE .. [[", "Refresh":1 }]]
		else
			return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_SET_ALPHA_FAIL .. " - " .. (msg or "Unknown").. [[" }]]
		end
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_SET_ALPHA_FAIL .. [["}]]
	end
end

local function GetIMAGEDesc( id )
	if not id or id == "" then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_ARGUMENT .. [["}]]
	end

	local r,desc = _M.MasterCall( "BUS.GetIMAGEDesc", id )
	if r and desc then 
		local size_w = 0
		local size_h = 0
		local png_alpha = 0
		local tp_x = "null"
		local tp_y = "null"
		local tp_mode = "replace"
		local tp_fuzz = 10
		if type(desc.size) == "table" then
			size_w = desc.size[1] or 0
			size_h = desc.size[2] or 0
			local test_str = size_w .. "x" .. size_h
			if desc.orig_size and desc.orig_size == test_str then
				size_w = 0
				size_h = 0
			end
		end
		png_alpha = desc.png_alpha_thres or 0
		
		if type(desc.transparent) == "table" then
			if desc.transparent[1] and desc.transparent[2] then
				tp_x = desc.transparent[1]
				tp_y = desc.transparent[2]
			end
			tp_mode = desc.transparent[3] or "replace"
			tp_fuzz = desc.transparent[4] or 10
		end

		return [[ {"Result":200, "data":{
"size_w":]] .. size_w .. [[,
"size_h":]] .. size_h .. [[,
"png_alpha":]] .. png_alpha .. [[,
"tp_x":]] .. tp_x .. [[,
"tp_y":]] .. tp_y .. [[,
"tp_mode":"]] .. tp_mode .. [[",
"tp_fuzz":]] .. tp_fuzz .. [[
} }]]
	else
		return [[ {"Result":200, "data":{
"size_w":0,
"size_h":0,
"png_alpha":0,
"tp_x":null,
"tp_y":null,
"tp_mode":"replace",
"tp_fuzz":10
}
}]]
	end
end

local function checkUse(id)
	local params = {}
	for i=0, 15 do
		params["osd.stream[0].items[" .. i .. "].osd_type"] = "none"
		params["osd.stream[0].items[" .. i .. "].image_id"] = ""
		params["osd.stream[1].items[" .. i .. "].osd_type"] = "none"
		params["osd.stream[2].items[" .. i .. "].image_id"] = ""
	end

	local is_ok, skip, result = _M.MasterCall( "BUS.Get", params )
	if not is_ok or not skip then
		return true
	end

	for i=0, 15 do
		if (result["osd.stream[0].items[" .. i .. "].osd_type"] == "image" and 
		result["osd.stream[0].items[" .. i .. "].image_id"] == id )
			or
			(result["osd.stream[1].items[" .. i .. "].osd_type"] == "image" and 
		result["osd.stream[1].items[" .. i .. "].image_id"] == id ) then
			return true
		end
	end
	
end

local function RemoveIMAGE(id)
	if not id or id == "" then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_IMAGE_ID .. [["}]]
	end

	if checkUse(id) then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.USED_IMAGE_ID .. [["}]]
	end

	local r,x,msg = _M.MasterCall( "BUS.RemoveIMAGE", id )
	if r then 
		if x then
			return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_REMOVE_DONE .. [[", "Refresh":1 }]]
		else
			return [[ {"Result":400, "Msg":"]] .. (msg or "Unknown").. [[" }]]
		end
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_REMOVE_FAIL .. [["}]]
	end
end
-------------- AJAX CALL ----------------
_M.AjaxExport ("ResizeIMAGE", ResizeIMAGE, "json") 
_M.AjaxExport ("SetIMAGEAlpha", SetIMAGEAlpha, "json") 
_M.AjaxExport ("RemoveIMAGE", RemoveIMAGE, "json" )
_M.AjaxExport ("GetIMAGEDesc", GetIMAGEDesc, "json" )
---------------------------------------------

function _M.GetIMAGE()
	local r,gets = _M.MasterCall( "BUS.GetIMAGEList")
	if r and gets then
		return gets
	else
		return nil
	end
end

return _M

