--Controller of modem
local _M = require( "controller.generic" )()
local cgilua = require "cgilua"

----------------------- SAJAX functions ----------------------------
local function DeleteConnection( obj, conPath )
	if not obj then
		return [[{ "Result":400, "Msg":"]] .. _M.langs.INVALID_MODEM_OBJ .. [["}]]
	end

	if not conPath then
		return [[{ "Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_CONNECTION .. [["}]]
	end

	local r, code = _M.MasterCall( "NETWORK_MANAGER.DeleteConnection", conPath )
	if r and code then
		return [[{ "Result":200, "Msg":"]] .. _M.langs.MSG_DELCONN_DONE .. [["}]]
	else
		return [[{ "Result":400, "Msg":"]] .. _M.langs.ERRMSG_DELCONN_FAIL .. [["}]]
	end
end

----------------------------------------------------------------------------

local function ConnectConnection(obj, conPath )
	if not obj then
		return [[{ "Result":400, "Msg":"]] .. _M.langs.INVALID_MODEM_OBJ .. [["}]]
	end

	if not conPath then
		return [[{ "Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_CONNECTION .. [["}]]
	end

	local r, code = _M.MasterCall( "NETWORK_MANAGER.ConnectDevice", obj, conPath )
	if r and code then
		return [[{ "Result":200, "Msg":"]] .. _M.langs.MSG_CON_DONE .. [["}]]
	else
		return [[{ "Result":400, "Msg":"]] .. _M.langs.ERRMSG_CON_FAIL .. [["}]]
	end
end

----------------------------------------------------------------------------

local function deactiveConnection( obj, conPath )
	if not obj then
		return [[{ "Result":400, "Msg":"]] .. _M.langs.INVALID_MODEM_OBJ .. [["}]]
	end

	local r, code = _M.MasterCall( "NETWORK_MANAGER.DeactivateDevice", obj )
	if r and code then
		return [[{ "Result":200, "Msg":"]] .. _M.langs.MSG_DCON_DONE .. [["}]]
	else
		return [[{ "Result":400, "Msg":"]] .. _M.langs.ERRMSG_DCON_FAIL .. [["}]]
	end

end

----------------------------------------------------------------------------
_M.AjaxExport ("DeleteConnection", DeleteConnection, "json") 
_M.AjaxExport ("ConnectConnection", ConnectConnection, "json" )
_M.AjaxExport ("deactiveConnection", deactiveConnection, "json" )
----------------------------------------------------------------------------

---------------------------------------------------------------------
-- Check the MODEM arguments
---------------------------------------------------------------------
function _M.Check()
	local obj, objPath,devname,port
	if not cgilua.QUERY or not cgilua.QUERY.obj then
		cgilua.put( "<h2>" .. _M.langs.INVALID_MODEM_OBJ .. "</h2>" )
		return nil
	end

	obj = cgilua.QUERY.obj
	objPath = _M.utils.url_encode(obj)
	devname = cgilua.QUERY.devname
	port = cgilua.QUERY.port

	local r, devInfo = _M.MasterCall( "NETWORK_MANAGER.GetDeviceInfo", obj, {GENERIC=true} )	--No options, so only get base informations.
	if not r or type(devInfo) ~= "table" then
		cgilua.put( "<h2>" .. _M.langs.INTERNAL_ERROR .. "</h2>" )
		if devInfo and type(devInfo) == "string" then
			cgilua.put( "<p>" .. devInfo .. "</p>" )
		end
		return nil
	end

	local fname
	r, fname = _M.MasterCall( "DEVICE_NAMES.GetDeviceName", devInfo.Name, _M.pageLang )
	
	local activeUuid = ""
	if devInfo.ActiveConnection and devInfo.ActiveConnection ~= "" and devInfo.ActiveConnection ~= "/" then
		local activeConnInfo
		r, activeConnInfo = _M.MasterCall( "NETWORK_MANAGER.GetActivateConnectionDetails", devInfo.ActiveConnection )
		if r and activeConnInfo then
			activeUuid = activeConnInfo[devInfo.ActiveConnection].Uuid
		end
	end
	
	-- The results.
	local rTable = {
		devname = devname,
		port = port,
		obj = obj,
		objPath = objPath,
		fname = fname,
		activeUuid = activeUuid,
		connections = {}
	}

	if type(devInfo.AvailableConnections) == "table" then
		local connDetails
		r, connDetails = _M.MasterCall( "NETWORK_MANAGER.GetConnectionDetails", devInfo.AvailableConnections )
		--r, connDetails = _M.MasterCall( "NETWORK_MANAGER.ListConnectionAndGetDetails", { interface=devInfo.Name } )
		if r and connDetails then
			local k,v 
			for k,v in pairs(connDetails) do
				local title
				local isCurrent = false
				local nType
				local netName
				local netApn
				local withSimPin = false
				local number
				
				title = v["connection"]["id"]
				if not title or title == "" then
					title = _M.langs.UNKNOWN
				end

				if v["connection"]["uuid"] and v["connection"]["uuid"] == activeUuid then
					isCurrent = true
				end

				if v["connection"]["type"] then
					if v["connection"]["type"] == "cdma" then
						nType = "cdma"
					else
						nType = "gsm"
					end
				else
					nType = "unknown"
				end

				if nType == "gsm" then
					netName = (v["gsm"]["network-id"] and v["gsm"]["network-id"] ~= "" and v["gsm"]["network-id"] or _M.langs.AUTO) 
					netApn = ( v["gsm"]["apn"] or "" )
					number = v["gsm"]["number"] or _M.langs.NO_NUMBER
					withSimPin = v["gsm"]["pin"] and v["gsm"]["pin"] ~= ""
				elseif nType == "cdma" then
					number = v["cdma"]["number"] or _M.langs.NO_NUMBER
				end
				local kPath = _M.utils.url_encode(k)
				
				table.insert( rTable.connections, {
					title = title,
					isCurrent = isCurrent,
					nType = nType,
					netName = netName,
					netApn = netApn,
					withSimPin = withSimPin,
					number = number,
					connObj = k,
					connObjPath = kPath
				} )
			end	--For
		end --If get details
	end -- If Avaliable connections got.
	
	return rTable
end

---------------------------------

return _M

