--Controller of modem_set
local _M = require( "controller.generic" )()
local nu = require "kiloview.netUtil"
local cgilua = require "cgilua"

---------- AJAX Call -----------

local function SaveSet( args )

	if not args or type(args) ~= "table" then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_POST_ARGS .. [["}]]
	end

	if not args.nettype or ( args.nettype ~= "gsm" and args.nettype ~= "cdma" ) then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_MODEM_TYPE .. [["}]]
	end

	local devname = args.devname
	local port = args.port
	-- if true then
	-- 	return [[ {"Result":400, "Msg":"devname=]] .. args.devname .. [["}]]
	-- end

	local connSettings = {
		["connection"] = {},
		["ipv4"] = {method="auto"},
		["devname"] = devname
	}

	connSettings["connection"]["type"] = args.nettype
	connSettings["connection"]["autoconnect"] = true --(args.autoConn == "y")
	connSettings["connection"]["interface-name"] = port
	connSettings["connection"]["only4G"] = args.only_4GLTE or "no"

	if args.newConn and args.newConn == "y" then	--New a connection.
		if not args.id or args.id == "" then
			return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_NAME .. [["}]]
		end

		connSettings["connection"]["id"] = args.id
	else --Change a connection
		if args.uuid and args.uuid ~= "" then
			connSettings["connection"]["uuid"] = args.uuid
			local r, detail = _M.MasterCall( "NETWORK_MANAGER.ListConnectionAndGetDetails", {uuid=args.uuid} )
			if r and detail then
				local k,v
				local found
				for k,v in pairs(detail) do
					found = v
					break
				end
				
				if found then
					connSettings["connection"]["id"] = found["connection"]["id"]
				end
			end
		end
	end

	if args.nettype == "gsm" then
		if not args.gsm_apn or args.gsm_apn == "" then
			return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_GSM_APN .. [["}]]
		end

		connSettings["gsm"] = {}
		connSettings["gsm"]["apn"] = args.gsm_apn or ""
		connSettings["gsm"]["number"] = args.gsm_number or ""
		connSettings["gsm"]["username"] = type(args.gsm_user) and args.gsm_user ~= "" and args.gsm_user or nil
		
		local psk = args.gsm_pass
		if psk == "~dont~change" then
			psk = nil
		end

		connSettings["gsm"]["password"] = psk

		psk = args.gsm_pin
		if psk == "~dont~change" then
			psk = nil
		end

		connSettings["gsm"]["pin"] = psk
	else
		connSettings["cdma"] = {}
		connSettings["cdma"]["number"] = args.cdma_number or ""
		connSettings["cdma"]["username"] = type(args.cdma_user) and args.cdma_user ~= "" and args.cdma_user or nil
		
		local psk = args.cdma_pass
		if psk == "~dont~change" then
			psk = nil
		end

		connSettings["cdma"]["password"] = psk

	end

	if args.newConn and args.newConn == "y" then
		local r, o = _M.MasterCall( "NETWORK_MANAGER.AddConnection", connSettings )
		if r and o then
			return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_SAVE_DONE .. [["}]]
		else
			return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_SAVE_FAIL .. [["}]]
		end
	else
		local r, o = _M.MasterCall( "NETWORK_MANAGER.ChangeConnectionDetails", args.specConn, connSettings )
		if r and o then
			return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_SAVE_DONE .. [["}]]
		else
			return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_SAVE_FAIL .. [["}]]
		end
	end
end

--------------------------------
_M.AjaxExport ("SaveSet", SaveSet, "json") 
--------------------------------

-- Check the request arguments

function _M.Check()
	local obj, objPath,devname,port
	local newOne = false
	local specConn

	if not cgilua.QUERY or not cgilua.QUERY.obj then
		cgilua.put( "<h2>" .. _M.langs.INVALID_MODEMSET_PARAM .. "</h2>" )
		return nil
	else
		obj = cgilua.QUERY.obj

		if cgilua.QUERY.newcon then	--New connection
			newOne = true
		else
			if cgilua.QUERY.con then --Specified connection (Modify)
				specConn = cgilua.QUERY.con
			else
				cgilua.put( "<h2>" .. _M.langs.INVALID_MODEMCONN_PARAM .. "</h2>" )
				return nil
			end
		end
	end

	objPath = _M.utils.url_encode(obj)
	devname = cgilua.QUERY.devname
	port = cgilua.QUERY.port

	local r, devInfo = _M.MasterCall( "NETWORK_MANAGER.GetDeviceInfo", obj, {MODEM = true} )
	if not r or not devInfo then
		cgilua.put( [[<h2>]] .. _M.langs.GET_DEVINFO_ERROR .. [[<hr class="title_line" /></h2>
						<p>]] .. _M.langs.GET_DEVINFO_ERROR_DETAIL .. [[</p>
						<a href="#" onclick="ShowPage('netlist.lp'); return false;">]] .. _M.langs.RETURN_TO_NETLIST .. [[</a> ]] )
		return nil
	end

	local id = ""
	local netType = "gsm"
	local gsm_apn = ""
	local gsm_user = ""
	local gsm_pass = ""
	local gsm_number = "*#99#"
	local gsm_pin = ""
	local cdma_user = ""
	local cdma_pass = ""
	local cdma_number = "#777"
	local autoconn = true
	local uuid = ""
	local only4G = "no"

	if newOne then	--Get AP information, and put to parameters.
		--NM_DEVICE_MODEM_CAPABILITY_CDMA_EVDO: 0x2
		--remotedostring( "print(...)", devInfo.MODEM, devInfo.MODEM.CurrentCapabilities )
		if devInfo.MODEM.CurrentCapabilities:match("CDMA") then
			netType = "cdma"
		--else is default 'gsm'
		end
	elseif specConn then	--Modify the connection
		local r, connInfo , only4g = _M.MasterCall( "NETWORK_MANAGER.GetConnectionDetails", specConn,devname )
		if not r or not connInfo or not connInfo[specConn] then
			cgilua.put( [[<h2>]] .. _M.langs.INVALID_SAVED_CONN .. [[<hr class="title_line" /></h2>
							<p>]] .. _M.langs.INVALID_SAVED_CONN_DETAIL .. [[</p>
							<a href="#" onclick="ShowPage('modem.lp?obj=]] .. objPath .. [['); return false;">]] .. _M.langs.RETURN_TO_MODEM_MAN .. [[</a> ]] )
			return nil
		end

		connInfo = connInfo[specConn]

		id = connInfo["connection"]["id"]
		netType = connInfo["connection"]["type"]
		uuid = connInfo["connection"]["uuid"]
		only4G = only4g or "no"

		if netType == "cdma" and connInfo["cdma"] then
			cdma_user = connInfo["cdma"]["username"] or ""
			cdma_pass = "~dont~change"	--Don't change default. (connInfo["cdma"]["password"] or "")
			cdma_number = connInfo["cdma"]["number"] or ""
		elseif connInfo["gsm"] then
			gsm_user = connInfo["gsm"]["username"] or ""
			gsm_pass = "~dont~change"	--Don't change default. (connInfo["gsm"]["password"] or "")
			gsm_number = connInfo["gsm"]["number"] or ""
			gsm_apn = connInfo["gsm"]["apn"] or ""
			gsm_pin = connInfo["gsm"]["pin"] or ""
		end
	end
	
	return {
		obj = obj,
		objPath = objPath,
		newOne = newOne,
		specConn = specConn,
		id = id,
		netType = netType,
		gsm_apn = gsm_apn,
		gsm_user = gsm_user,
		gsm_pass = gsm_pass,
		gsm_number = gsm_number,
		gsm_pin = gsm_pin,
		cdma_user = cdma_user,
		cdma_pass = cdma_pass,
		cdma_number = cdma_number,
		autoconn = autoconn,
		uuid = uuid,
		devname = devname,
		port = port,
		only4G = only4G

	}
	
end

function _M.getRevision(devname)
	local r, modemInfo = _M.MasterCall( "NETWORK_MANAGER.GetModemInfo", devname)
	if not r or not modemInfo then
		return nil
	end
	local revision
	if modemInfo and modemInfo["ModemInfo"] and modemInfo["ModemInfo"]["Revision"] then
		revision = modemInfo["ModemInfo"]["Revision"]
	else
		revision = nil
	end
	return revision	
end
--------------------------------

return _M

