--Controller of streamlist
local _M = require( "controller.generic" )()
local cgilua = require "cgilua"

-------------- AJAX CALL ----------------

local function ConfigStreamMode(mode)
	if 	not mode or	(mode ~= "main" and mode ~= "main+sub" and mode ~= "main+sub+snap" and mode ~= "main+snap")	then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.ERROR_INVALID_STREAM_MODE .. [["} ]]
	end

	if (mode == 'main') then
		local r,done = _M.MasterCall( "BUS.Set", {["record_m.stream[1].start"] = 0} )
		if not r or not done then
			return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_FAIL .. [["} ]]
		end
	end

	local r,done = _M.MasterCall( "BUS.Set", { ["encoder.encode_mode"] = mode } )
	if r and done then
		return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_SUCCESS .. [["} ]]
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_FAIL .. [["} ]]
	end
end

-------------- AJAX CALL ----------------
local function StartRecord( streamId )
	local streamNum = tonumber(streamId)
	if not streamNum then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_INVALID_ARGS .. [["} ]]
	end

	local r, done = _M.MasterCall( "BUS.Set", {["record_m.stream[" .. streamNum .. "].start"] = 1} )
	if r and done then
		return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_SUCCESS .. [["} ]]
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_FAIL .. [["} ]]
	end
end

local function StopRecord( streamId )
	local streamNum = tonumber(streamId)
	if not streamNum then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_INVALID_ARGS .. [["} ]]
	end

	local r, done = _M.MasterCall( "BUS.Set", {["record_m.stream[" .. streamNum .. "].start"] = 0} )
	if r and done then
		return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_SUCCESS .. [["} ]]
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_FAIL .. [["} ]]
	end
end

local function EnableService(streamId,service,en)
	en = tonumber(en)
	if en ~= 0 then
		en = true
	else
		en = false
	end

	local r,done = _M.MasterCall( "BUS.ControlStreamServiceEnable", streamId, service, en )
	if r and done then
		return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_SUCCESS .. [["} ]]
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_FAIL .. [["} ]]
	end
end
------------
local function AddStream( streamId, streamType )
	local thisPrefix = "streamer.stream[" .. tonumber(streamId) .. "].Dynamic_services"
	local gets = {
		[ thisPrefix .. "[0].serviceType" ] = "",
		[ thisPrefix .. "[1].serviceType" ] = "",
		[ thisPrefix .. "[2].serviceType" ] = "",
		[ thisPrefix .. "[3].serviceType" ] = "",
		[ thisPrefix .. "[4].serviceType" ] = "",
		[ thisPrefix .. "[5].serviceType" ] = "",
		[ thisPrefix .. "[6].serviceType" ] = "",
		[ thisPrefix .. "[7].serviceType" ] = ""
	}

	local r,done
	r,done,gets = _M.MasterCall( "BUS.Get", gets )
	if not r or not done or type(gets) ~= "table" then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_GETCONFIG_ERROR .. [["} ]]
	else
		local validIndex
		local idx
		local i
		for i=0,7 do
			idx = thisPrefix .. "[" .. i .. "].serviceType"
			if gets[idx] == "" then
				validIndex = idx
				break
			end
		end
		
		if streamType == "HLS" then
			r, done = _M.MasterCall( "BUS.GetStreamCount", streamType , 2)
			if not done then
				return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_HLS_GETCONFIG_ERROR .. [["} ]]
			end
		end
		if validIndex then
			local sets = { [ validIndex ] = streamType or "" }
			r, done = _M.MasterCall( "BUS.Set", sets )
			if r and done then
				return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_ADDSTREAM_DONE .. [["} ]]
			else
				return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_FAIL .. [["} ]]
			end
		else
			return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_GETCONFIG_ERROR .. [["} ]]
		end
	end
end
-------------
local function RemoveStream( streamId, streamIndex )
	if not streamId or not streamIndex then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_INVALID_ARGS .. [["} ]]
	end
	streamId = tonumber(streamId)
	streamIndex = tonumber(streamIndex)
	if streamId < 0 or streamId > 2 or streamIndex < 0 or streamIndex > 7 then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_INVALID_ARGS .. [["} ]]
	end

	local sets = {
		[ "streamer.stream[" .. streamId .. "].Dynamic_services[" .. streamIndex .. "].serviceType" ] = ""
	}

	local r, done = _M.MasterCall( "BUS.Set", sets )
	if r and done then
		return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_REMOVESTREAM_DONE .. [["} ]]
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_FAIL .. [["} ]]
	end
end
-----------------
local function ChangeAudioChannel( streamId, channelId )
	if not streamId or not channelId then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_INVALID_ARGS .. [["} ]]
	end
	streamId = tonumber(streamId)
	channelId = tonumber(channelId)
	if streamId < 0 or streamId > 2 then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_INVALID_ARGS .. [["} ]]
	end

	local sets = {
		[ "streamer.stream[" .. streamId .. "].audioChannel" ] = channelId
	}

	local r, done = _M.MasterCall( "BUS.Set", sets )
	if r and done then
		return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_SUCCESS .. [["} ]]
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_FAIL .. [["} ]]
	end
end
local function SetNoVideo( val )                                                  
	local r, done = _M.MasterCall( "BUS.Set", { ["streamer.no_video"] = tonumber(val) or 0 } )
	if r and done then                                                                            
		return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_SUCCESS .. [["} ]]             
	else                                                                 
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_FAIL .. [["} ]]                 
	end                                                                                          
end  

-------------- AJAX CALL ----------------
_M.AjaxExport ("ConfigStreamMode", ConfigStreamMode, "json") 
_M.AjaxExport ("EnableService", EnableService, "json" )
_M.AjaxExport ("AddStream", AddStream, "json" )
_M.AjaxExport ("RemoveStream", RemoveStream, "json" )
_M.AjaxExport ("ChangeAudioChannel", ChangeAudioChannel, "json" )
_M.AjaxExport ("StartRecord", StartRecord, "json" )
_M.AjaxExport ("StopRecord", StopRecord, "json" )
_M.AjaxExport ("SetNoVideo", SetNoVideo, "json" )                                                 

---------------------------------------------

function _M.GetStreamStatus( streamIdr )
	local x, cfgs = _M.MasterCall( "BUS.GetStreamStatus", streamIdr )
end

function _M.GetNoVideo()                                                          
	local x, done, cfgs = _M.MasterCall( "BUS.Get", { ["streamer.no_video"] = 0 }  )          
	if x and done and cfgs then                                                                   
		return cfgs["streamer.no_video"] or 0                                             
	else                                                                 
		return 0                                                                           
	end                                                                                          
end   
---------------------------------------------

return _M

