--[[
    对讲模块 API 
--]]
local MSG = require "msg"
local CO = require "coservice"
local cjson = require "cjson"
local _dbg = require "coaf.debug"

local dset = {}

local function getIP()
  local headers  = ngx.req.get_headers()
  local clientIP = headers["X-Forwarded-For"]
  if clientIP == nil or string.len(clientIP) == 0 or clientIP == "unknown" then
          clientIP = headers["Proxy-Client-IP"]
  end
  if clientIP == nil or string.len(clientIP) == 0 or clientIP == "unknown" then
          clientIP = headers["WL-Proxy-Client-IP"]
  end
  if clientIP == nil or string.len(clientIP) == 0 or clientIP == "unknown" then
          clientIP = ngx.var.remote_addr    
  end
  -- 对于通过多个代理的情况，第一个IP为客户端真实IP,多个IP按照','分割
  if clientIP ~= nil and string.len(clientIP) >15  then
          local pos  = string.find(clientIP, ",", 1)
          clientIP   = string.sub(clientIP, 1, pos-1)
  end
  return clientIP
end

--[[
    添加对讲会话
--]]
local function API_talk_add(red, args)
    if args and args.ip then 
        local proxy = CO.GetProxy( "/talkback", "talkService", "local", 6 )
        if not proxy then
          MSG.ERROR( "无法连接系统服务" )
          return true
        else
          local remote = getIP()
          local r, rtc, msg = proxy.TalkBackChannelCreate( remote, args.ip )
          proxy:Destroy()
          if r then
            if rtc then 
              MSG.OK( "", { data = msg })
            else
              MSG.OK( "", { status = msg })
            end
          else
            MSG.ERROR( rtc )
          end
        end
    else
      MSG.ERROR( "参数错误" )
    end
    return true
end
--[[
    删除对讲会话
--]]
local function API_talk_del(red, args)
  if args and args.id then 
    local proxy = CO.GetProxy( "/talkback", "talkService", "local", 6 )
    if not proxy then
      MSG.ERROR( "无法连接系统服务" )
      return true
    else 
      local r = proxy.TalkBackChannelDestroy( args.id )
      proxy:Destroy()
      if r then 
        MSG.OK( "" )
      else
        MSG.ERROR( "TalkBackChannelDestroy error" )
      end
    end
else
  MSG.ERROR( "参数错误" )
end
return true
end

--[[
    设置对讲,选中某个对讲设备，
    enable = 1 开启对讲，则将把当前选择的ip赋值给selected_ip
    enable = 0 关闭对讲，则将把selected_ip 清空
--]]
local function API_talk_dev_set(red, args)
    if args and args.id and args.enable then 
        local proxy = CO.GetProxy( "/talkback", "talkService", "local", 6 )
        if not proxy then
          MSG.ERROR( "无法连接系统服务" )
          return true
        else 
          local r, msg = proxy.TalkBackDeviceSelected( args.id, args.enable )
          proxy:Destroy()
          if r then 
            MSG.OK( "", { status = msg } )
          else
            MSG.ERROR( msg )
          end
        end
    else
      MSG.ERROR( "参数错误" )
    end
    return true
end

--[[
    获取对讲设备列表
--]]
local function API_talk_dev_list(red, args)
  local proxy = CO.GetProxy( "/talkback", "talkService", "local", 6 )
  if not proxy then
      MSG.ERROR( "无法连接系统服务" )
      return true
  else 
      local r, data = proxy.GetAllInfos()
      proxy:Destroy()
      if r and data then 
          MSG.OK( "", { data = data })
      else
          MSG.ERROR( "GetAllInfos error" )
      end
  end
  return true
end

--[[
    手动添加对讲设备
--]]
local function API_talk_dev_add(red, args)
  if args and args.ip then 
    local proxy = CO.GetProxy( "/talkback", "talkService", "local", 6 )
    if not proxy then
      MSG.ERROR( "无法连接系统服务" )
      return true
    else 
      local r, ret, msg = proxy.TalkBackDeviceAdd( args )
      proxy:Destroy()
      if r then 
        if ret then
          MSG.OK( "" )
        else
          MSG.ERROR( msg )
        end
      else
        MSG.ERROR( "TalkBackDeviceAdd error" )
      end
    end
  else
    MSG.ERROR( "参数错误" )
  end
  return true
end

--[[
    删除手动添加的对讲设备
--]]
local function API_talk_dev_del(red, args)
  if args and args.id then 
    local proxy = CO.GetProxy( "/talkback", "talkService", "local", 6 )
    if not proxy then
      MSG.ERROR( "无法连接系统服务" )
      return true
    else 
      local r = proxy.TalkBackDeviceDel( args.id )
      proxy:Destroy()
      if r then 
        MSG.OK( "" )
      else
        MSG.ERROR( "TalkBackDeviceDel error" )
      end
    end
  else
    MSG.ERROR( "参数错误" )
  end
  return true
end

--[[
    网页应答的sdp
--]]
local function API_talk_answersdp(red, args)
  if args then
      local proxy = CO.GetProxy( "/talkback", "talkService", "local", 6 )
      if not proxy then
        MSG.ERROR( "无法连接系统服务" )
        return true
      else 
        local r = proxy.ParseAnswerSdp( args )
        proxy:Destroy()
        MSG.OK( "" )
      end
  else
    MSG.ERROR( "参数错误" )
  end
  return true
end

--[[
    网页应答的sdp
--]]
local function API_talk_headset(red, args)
  if args and (args.headset=="USB" or args.headset=="usb" or args.headset=="3.5mm" or args.headset=="3.5MM") then
      local proxy = CO.GetProxy( "/talkback", "talkService", "local", 6 )
      if not proxy then
        MSG.ERROR( "无法连接系统服务" )
        return true
      else 
        local r = proxy.SelectHeadsetInput( args )
        proxy:Destroy()
        if r then
          MSG.OK( "" )
        else
          MSG.ERROR( "parse sdp ERROR" )
        end
      end
  else
    MSG.ERROR( "answersdp参数错误" )
  end
  return true
end

--[[
    获取设备序列号
--]]
local function API_talk_serial(red, args)
  local proxy = CO.GetProxy( "/talkback", "talkService", "local", 6 )
  if not proxy then
      MSG.ERROR( "无法连接系统服务" )
      return true
  else 
      local r, data = proxy.GetDevSerial()
      proxy:Destroy()
      if r and data then 
          MSG.OK( "", { data = data })
      else
          MSG.ERROR( "GetAllInfos error" )
      end
  end
  return true
end

dset.APIS = dset.APIS or {}
dset.APIS["talk_add"] = API_talk_add
dset.APIS["talk_del"] = API_talk_del
dset.APIS["talk_dev_set"] = API_talk_dev_set
dset.APIS["talk_dev_list"] = API_talk_dev_list
dset.APIS["talk_dev_add"] = API_talk_dev_add
dset.APIS["talk_dev_del"] = API_talk_dev_del
dset.APIS["talk_answersdp"] = API_talk_answersdp
dset.APIS["talk_headset"] = API_talk_headset
dset.APIS["talk_serial"] = API_talk_serial
--------------------------------------
return dset