local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

local _M = {
}

local function CheckMount( point )
	local point_len = #point
	local f = io.open( "/proc/mounts", "r" )
	if f then
		local already_mount = nil
		local ln = f:read("*l")
		while ln do
			if ln:sub(1,point_len) == point then
				already_mount = ln:match(".-%s+(.-)%s+.*")
				break
			end
			ln = f:read("*l")
		end
		f:close()
		return already_mount
	else
		return nil
	end
end

function _M.Mount( event, id, args )
	if id and type(args) == "table" then
		if not args.host or args.host == "" then
			return nil, "Invalid CIFS host"
		end
		local host = args.host
		if not args.volumn or args.volumn == "" then
			return nil, "Invalid CIFS volumn"
		end
		local volumn = args.volumn
		local name
		if not args.name or args.name == "" then
			name = id
		else
			name = args.name
		end

		local point = "//" .. host
		if volumn:match("^/") then
			point = point .. volumn
		else
			point = point .. "/" .. volumn
		end

		local mnt = CheckMount( point )
		if mnt then
			return mnt, name
		end

		local options = "rw"
		if args.options and args.options ~= "" then
			if args.options:match("rw") then
				options = args.options
			else
				options = options .. "," .. args.options
			end
		end

		local pid = coaf.Fork( function()
			os.execute( "/bin/mkdir -p /media/nas/" .. id )
			os.execute( "/bin/mount -t cifs -o " .. options .. " " .. point .. " /media/nas/" .. id )
			os.exit(0)
		end )

		if pid and pid > 0 then
			local r,err = coaf.WaitPID(pid)
			if r then
				mnt = CheckMount(point)
				if mnt then
					return mnt, name
				else
					return nil, "Fail mount CIFS volumn"
				end
			else
				os.execute( "/bin/kill -1 " .. pid )
				return nil, "Fail to mount CIFS volumn"
			end
		else
			return nil, "Fail to start CIFS mounting"
		end
	else
		return nil, "Invalid mount arguments"
	end
end

function _M.UMount( event, id )
	local f = io.open( "/proc/mounts", "r" )
	if f then
		local found = false
		local ln = f:read("*l")
		while ln do
			local mnt = ln:match(".-%s+(.-)%s+.*")
			if mnt == ("/media/nas/" .. id) then
				found = true
				break
			end
			ln = f:read("*l")
		end
		f:close()

		if found then
			local r,status,code = os.execute( "/bin/umount -l /media/nas/" .. id )
			if r and code == 0 then
				return true
			else
				return false, status, code
			end
		else
			return true
		end
	else
		return true
	end
end

return _M

