local coaf = require "coaf.core"
local thread = require "coaf.thread"
local _dbg = require "coaf.debug"
local run_daemon = require "coaf.run_daemon"
local client = require "coaf.client"
local client_adp = require "coaf.cli_adapter.simple"
local dyn_service = require "coserver.dynamic_service"
local rpc_daemon = require "coaf.rpc.daemon"
local avahi = require "coserver.discovery_tools.avahi"

require "cocrypto"

----------------
local _M

local _Cast = {
}

function _Cast.CanYouTouchMe( unique_id )
	local connObj = rpc_daemon:GetInvokingConnection()
	local addr = connObj and connObj.connected_addr

	local check_seq = { "refuse", "out-only", "in-only" }
	local step_returns = { "unacceptable", "unacceptable", "-" }
	local step
	local is_inonly
	for step=1, #check_seq do
		local filter = _M.filters[check_seq[step]]
		if filter and type(filter) == "table" then
			local i, v
			for i, v in pairs(filter) do
				if i == "*" or unique_id == i or unique_id:match(i) then
					if addr and (v == "*" or addr == v or addr:match(v)) then
						if step_returns[step] == '-' then
							is_inonly = true
							break
						else
							return step_returns[step]
						end
					end
				end
			end
		end
	end

	local conn_daemon = require "coserver.conn_daemon"
	conn_daemon.AllowAcceptProxy( unique_id, "ANY" )

	local status = _M.GetTargetStatus(unique_id)
	if status then
	--NOTE: To make things be easy, the policy is changed to: Who first who win!
		return "yes"
	end

	local cocore = require "coserver.core"
	local r = cocore.GetLocations(unique_id)

	--NOTE: To make things be easy, the policy is changed to: Who first who win!
	if r[unique_id] then
		return "exist"
	elseif is_inonly then --MUST connection me.
		return "no"
	else --TODO:
		return "no"
	end
end

function _Cast.TellMeMyAddress( unique_id )
	local connObj = rpc_daemon:GetInvokingConnection()
	return connObj and connObj.connected_addr, connObj and connObj.connected_port
end

function _Cast.TellMeMyAlias( unique_id )
	local alias_list = _M.filters["alias"]
	if type(alias_list) == "table" and unique_id then
		local fnd = alias_list[unique_id]
		if fnd then
			return fnd
		end
		local k,v
		for k,v in pairs(alias_list) do
			if k == '' or k == '*' then
				return v
			elseif unique_id:match(k) then
				return v
			end
		end
		return unique_id
	else
		return unique_id
	end
end

function _Cast.RequestDynamicAuthorKey( unique_id, alias_id )
	local connObj = rpc_daemon:GetInvokingConnection()
	local addr = connObj and connObj.connected_addr or "unknown"

	local item = _M.dynamic_keys[unique_id]
	local key
	if not item then
		key = CoAF_CRYPTO.get_challenge()
		item = {
			["unknown"] = key
		}
		item[addr] = key
		_M.dynamic_keys[ unique_id ] = item
	else
		if item[addr] then
			key = item[addr]
		else
			key = CoAF_CRYPTO.get_challenge()
			item[addr] = key
		end
	end

	if alias_id and alias_id ~= unique_id then
		local alias_item = _M.dynamic_keys[alias_id]
		if not alias_item then
			alias_item = {
				["unknown"] = key
			}
			alias_item[addr] = key
			_M.dynamic_keys[alias_id] = alias_item
		else
			alias_item[addr] = key
		end
	end

	return key
end


function _Cast.GetPublishOptions()
	if _M.publish_options then
		return _M.publish_options
	else
		return {
			group = "manual"
			--link_token = _M.default_link_token
		}
	end
end

----------------

_M = {
	ip_families = { "IPV4" },
	tool = nil,

	CASTER = _Cast,

	UNIQUE_ID = nil,
	global_UNIQUE_ID = nil,
	global_AUTHOR_KEY = nil,

	publish_options = nil,
	default_link_token = nil,
	discovery_key = nil,

	discovery_started = false,
	discovery_group_match = nil,

	filters = {
	},

	all_discovered = {
	},

	all_manual = {
	},

	dynamic_keys = {
	},

	__init = false,

	order_count = 10000,
	manual_order_count = 0,
}

----------------

local function Discover_UNIQUE_ID()
	if _M.UNIQUE_ID then
		return _M.UNIQUE_ID
	elseif _M.global_UNIQUE_ID then
		if type(_M.global_UNIQUE_ID) == "function" then
			return _M.global_UNIQUE_ID() or coaf.GetHostname()
		else
			return _M.global_UNIQUE_ID
		end
	else
		return coaf.GetHostname()
	end
end

local function Discover_AUTHOR_KEY(request, addr)
	if request and request.headers then
		local ret
		if type(_M.global_AUTHOR_KEY) == "function" then
			ret =  _M.global_AUTHOR_KEY(request, addr)
		else
			ret = _M.global_AUTHOR_KEY
		end

		local loc = request.headers.LOC
		local uniq = request.headers.UNIQ

		if loc == "local" and request.headers.SVC == "_CO_DAEMON.discovery" then
			if _M.discovery_key then
				if type(ret) ~= "table" then
					ret = { ret }
				end
				table.insert( ret, _M.discovery_key )
			end

			if _M.default_link_token then
				if type(_M.default_link_token) == "string" and _M.default_link_token ~= _M.discovery_key then
					if type(ret) ~= "table" then
						ret = { ret }
					end
					table.insert( ret, _M.default_link_token )
				elseif uniq and type(_M.default_link_token) == "table" then
					local token = _M.default_link_token[ uniq ]
					if not token then
						local k, v
						for k,v in pairs(_M.default_link_token) do
							if uniq:match(k) then
								token = v
								break
							end
						end

						if not token then
							token = _M.default_link_token["*"]
						end
					end

					--With options.
					if type(token) == "table" then
						if addr and token.allow_addresses then
							local matchs = token.allow_addresses
							if type(matchs) == "string" then
								matchs = { matchs }
							end
							local i,v,fnd
							for i,v in ipairs(matchs) do
								if v == addr or addr:match(v) or v == '*' then
									fnd = true
									break
								end
							end
							if fnd then
								token = token.token
							else
								token = nil
							end
						else
							token = token.token
						end
					end

					if token then
						if type(ret) ~= "table" then
							ret = { ret }
						end
						table.insert( ret, token )
					end
				end
			end
		end

		--Pick the unique ID passed from request, if exist.
		if uniq and uniq ~= "local" and _M.dynamic_keys[uniq] then
			addr = addr or "unknown"
			if _M.dynamic_keys[uniq][addr] then
				if type(ret) ~= "table" then
					ret = { ret }
				end
				table.insert( ret, _M.dynamic_keys[uniq][addr] )
			end
		end
		return ret
	end

	if type(_M.global_AUTHOR_KEY) == "function" then
		return _M.global_AUTHOR_KEY(request, addr)
	else
		return _M.global_AUTHOR_KEY
	end
end

function _M.Init()
	if not _M.__init then
		--TODO: this is a hack action
		if not client.socket then
			client.InitSocket(nil, client_adp)
		end

		--Don't log (print) client trying messages (too many)
		_dbg.GlobalDisableTags("CoAF.Client")

		avahi.Init() 
		local cocore = require "coserver.core"
		--Handle special configures.
		_M.global_UNIQUE_ID = cocore.configs.UNIQUE_ID
		_M.global_AUTHOR_KEY = cocore.configs.AUTHOR_KEY

		cocore.configs.UNIQUE_ID = Discover_UNIQUE_ID
		cocore.configs.AUTHOR_KEY = Discover_AUTHOR_KEY
		--_M.default_link_token = CoAF_CRYPTO.get_challenge():sub(1,16)
		--avahi.SetHostname( Discover_UNIQUE_ID() )
	end

	return true
end

function _M.Cleanup()
	if _M.discovery_started then
		_M.StopDiscovery()
		_M.RemoveAllDiscovered(true)
	end
	avahi.StopAllPublishServices()
	_M.publish_options = nil
end

function _M.SetUniqueID(uid)
	--TODO: Update all dynamic services and proxy links.
	_M.UNIQUE_ID = uid
end

function _M.SetLinkToken(token)
	if token and _M.default_link_token ~= token then
		_M.default_link_token = token
	end
end

function _M.PublishGlobal(options)
	local cocore = require "coserver.core"
	local unique_id = Discover_UNIQUE_ID()

	local port = 14000
	if cocore.configs.BIND then
		local bind = cocore.configs.BIND
		if type(bind) ~= "table" then
			bind = { bind }
		end
		local fetched_port
		local i, item, addr, p
		for i, item in ipairs(bind) do
			if not item:match("^unix") then
				addr, p = item:match("(.+):([0-9]*)")
				if addr and addr ~= "localhost" and not addr:match("^127%.") then
					if addr == "" or addr == "*" then
						fetched_port = nil
						port = tonumber(p) or port
						break
					else
						fetched_port = p
					end
				end
			end
		end
		if fetched_port then
			port = tonumber(fetched_port) or port
		end
	end

	local txts = {}
	local fnd_link_token
	options = options or {}

	local k,v
	local fnd_group
	for k,v in pairs(options) do
		if k == "group" then
			fnd_group = true
			table.insert( txts, k .. "=" .. tostring(v) )
		elseif k == "link_token" then
			fnd_link_token = v
			--_dbg.Warn( "CoServer.Discovery", "Used user passed link_token '", v, "' instead of random." )
			_dbg.Warn( "CoServer.Discovery", "Strong advice that don't passed link_token to options!" )
		else
			table.insert( txts, k .. "=" .. tostring(v) )
		end
	end
	if not fnd_group then
		table.insert( txts, "group=public" )
		options["group"] = "public"
	end

	--if fnd_link_token then
	--	_M.discovery_key = fnd_link_token
	--elseif not _M.discovery_key then
	--
	if not _M.discovery_key then
		--_M.discovery_key = _M.default_link_token
		_M.discovery_key = CoAF_CRYPTO.get_challenge():sub(1,16)
	end

	options["link_token"] = _M.discovery_key

	--if not fnd_link_token then
	table.insert( txts, "link_token=" .. _M.discovery_key )
	--end

	--'k' as successful flag, and 'v' as error message.
	k,v = avahi.PublishService( "global-coaf-discovery", unique_id, "_coaf._tcp", port, txts )
	if k then
		_M.publish_options = options
	end
	return k,v
end

function _M.StopPublishGlobal()
	--_M.discovery_key = nil
	_M.publish_options = nil
	return avahi.StopPublishService( "global-coaf-discovery" )
end

function _M.PublishService(id, name, service_type, port, txts)
	if not avahi then
		return nil, "Not initialized"
	elseif id == "global-coaf-discovery" then
		return nil, "Not allowed"
	else
		return avahi.PublishService(id, name, service_type, port, txts)
	end
end

function _M.StopPublishService(id)
	if not avahi then
		return nil, "Not initialized"
	else
		--TODO: More verification.
		return avahi.StopPublishService(id)
	end
end

-----------------------------------------------------------------------------
local function disc_callback( service_type, action, service_item )
	if action == "cleanup" then
		--TODO: Should me clear all services?
		return
	elseif not service_item or not service_item.name or not service_item.protocol or not service_item.interface then
		_dbg.Error( "CoServer.Discovery", "Invalid service content for event:", action, "[", service_type, "]" )
		return
	--Ignore local service.
	elseif service_item.is_local then
		return
	else
		local p = service_item.protocol:upper()
		local i, fnd
		for i=1, #_M.ip_families do
			if p == _M.ip_families[i] then
				fnd = true
				break
			end
		end
		if not fnd then
			return
		end
	end

	--NOTE here:
	local location = _Cast.TellMeMyAlias(service_item.name)

	if action == "new" or action == "update" or action == "resolve" then
		if not service_item.resolve_info then -- or service_item.resolve_info.untrust then
			return
		end

		local item = _M.all_discovered[ location ]
		local txts = service_item.resolve_info.txt
		local fnd_group, fnd_link_token
		local options = {}
		if txts then
			local i, v
			local key,val
			for i,v in ipairs(txts) do
				key,val = v:match("(.-)=(.*)$")
				if key then
					options[key] = val
				else
					options[v] = ''
				end
				if key == "group" then
					fnd_group = val
				elseif key == "link_token" then
					fnd_link_token = val
				end
			end
		end

		if not fnd_group then
			return
		end

		if _M.discovery_group_match then
			local i, matcher, matched
			for i,matcher in ipairs(_M.discovery_group_match) do
				if matcher == "" or matcher == "*" or fnd_group:match(matcher) then
					matched = true
					break
				end
			end
			if not matched then
				return
			end
		end

		local check_seq = { "refuse", "in-only" }
		local step
		local addr = service_item.resolve_info.address

		for step=1, #check_seq do
			local filter = _M.filters[check_seq[step]]
			if filter and type(filter) == "table" then
				local i, v
				for i, v in pairs(filter) do
					if i == "*" or location == i or location:match(i) then
						if v == "*" or addr == v or addr:match(v) then
							return
						end
					end
				end
			end
		end

		local createNew
		local addr_item = {
			address = service_item.resolve_info.address,
			port = service_item.resolve_info.port,
			--txts = txts,
			authorKey = fnd_link_token
		}

		if not item then --New a discovered item
			item = {
				order = _M.order_count,
				location = location,
				options = options, --TODO: It may not be correct, but now can use.
				addresses = {
					[1] = addr_item,
					[service_item.interface .. ":" .. service_item.protocol] = addr_item,
				}
			}
			createNew = true
		else
			local old_addr = item.addresses[ service_item.interface .. ":" .. service_item.protocol ]
			if not old_addr then
				item.addresses[ service_item.interface .. ":" .. service_item.protocol ] = addr_item
				table.insert( item.addresses, addr_item )
			elseif old_addr.address == addr_item.address and old_addr.port == addr_item.port and old_addr.authorKey == addr_item.authorKey then
				--No changed.
				return
			else
				old_addr.address = addr_item.address
				old_addr.port = addr_item.port
				old_addr.authorKey = addr_item.authorKey
			end
		end

		if createNew then
			local r = dyn_service.AddService( location, "ANY", nil, nil, nil, true) --No bidi link.
			if not r then
				return
			end
			dyn_service.SetAddressList( location, "ANY", item.addresses )
			_M.order_count = _M.order_count + 1
			_M.all_discovered[ location ] = item
		else --Wakeup if it slept.
			dyn_service.AddressListRetry( location, "ANY" )
		end

	elseif action == "remove" then --or action == "resolve-failed" then
		local item = _M.all_discovered[ location ]
		if item then
			local addr_item = item.addresses[ service_item.interface .. ":" .. service_item.protocol ]
			if addr_item then
				item.addresses[ service_item.interface .. ":" .. service_item.protocol ] = nil
				local i,v
				for i,v in ipairs(item.addresses) do
					if v == addr_item then
						table.remove( item.addresses, i )
						break
					end
				end
				if #item.addresses == 0 then
					dyn_service.RemoveService( location, "ANY", 5 ) --5 seconds delayed remove service
					_M.all_discovered[ location ] = nil
				else
					--Retry again.
					dyn_service.AddressListRetry( location, "ANY" )
				end
			end
		end
	end
end

-----------------------------------------------------------------------------
function _M.StartDiscovery(match_groups)
	if _M.discovery_started then
		return false, "Already started"
	end
	if type(match_groups) == "string" then
		match_groups = { match_groups }
	elseif type(match_groups) ~= "table" then
		match_groups = nil
	end

	_M.discovery_group_match = match_groups

	avahi.SetDiscoveryCallback( disc_callback )
	avahi.StartDiscovery("_coaf._tcp")

	_M.discovery_started = true
	return true
end

function _M.StopDiscovery()
	if _M.discovery_started then
		avahi.StopDiscovery()
		_M.discovery_started = false
		_M.RemoveAllDiscovered(false)
		--_M.dynamic_keys = {}
	end
end

-----------------------------------------------------------------------------

function _M.AddManualTarget( id, addr, port, authKey )
	if not id or not addr then
		return false, "Invalid arguments"
	end

	local item = _M.all_manual[id]
	if item then
		return false, "exist"
	end

	port = port or 14000
	local r = dyn_service.AddService( id, "ANY", addr, port, authKey, false ) --Need bidi check.
	if not r then
		return false, "Create service failed"
	end

	_M.all_manual[id] = {
		order = _M.manual_order_count,
		location = id
	}

	_M.manual_order_count = _M.manual_order_count + 1
	return true
end

function _M.RemoveManualTarget( id )
	if not id then
		return false, "Invalid argument"
	end
	local item = _M.all_manual[id]
	if item then
		dyn_service.RemoveService( id, "ANY" )
		_M.all_manual[id] = nil
	end
	return true
end

function _M.RemoveAllManualTargets()
	local k,v
	for k,v in pairs(_M.all_manual) do
		dyn_service.RemoveService( v.location, "ANY" )
	end
	_M.all_manual = {}
	_M.manual_order_count = 0
end

function _M.RemoveAllDiscovered(includeManual)
	local k,v
	for k,v in pairs(_M.all_discovered) do
		dyn_service.RemoveService( v.location, "ANY" )
	end
	_M.all_discovered = {}
	_M.order_count = 10000

	if includeManual then
		_M.RemoveAllManualTargets()
	end
end

function _M.GetDiscoveryList()
	local ret = {}
	local k,v,o
	for k,v in pairs(_M.all_manual) do
		o = dyn_service.GetServiceDetail(v.location, "ANY")
		local addr, port
		if o then
			addr = o.target_addr or o.last_try_addr or ""
			port = tonumber(o.target_port or o.last_try_port or 0)
		else
			addr = ""
			port = 0
		end

		table.insert(ret, {
			order = v.order or 999999,
			id = v.location,
			manual = true,
			address = addr,
			port = port,
			options = o and o.publish_options,
			status = o and o.status or "unknown"
		})
	end
	for k,v in pairs(_M.all_discovered) do
		o = dyn_service.GetServiceDetail(v.location, "ANY")
		if o and (o.status == "ready" or o.status == "online") then
			table.insert(ret, {
				order = v.order or 999999,
				id = v.location,
				address = o.target_addr or o.last_try_addr or "",
				port = tonumber(o.target_port or o.last_try_port or 0),
				options = o.publish_options or v.options,
				status = o.status
			})
		end
	end

	table.sort( ret, function(a,b)
		if a == b then
			return false
		elseif not a.order or not b.order then
			return false
		else
			return a.order <= b.order
		end
	end)

	return ret
end

function _M.GetTargetStatus(id)
	if id then
		local obj = dyn_service.GetServiceDetail(id, "ANY")
		if not obj then
			return nil
		end
		return obj.status, obj.target_addr or obj.last_try_addr, obj.target_port or obj.last_try_port
	else
		return nil
	end
end

--Filters:
--   "refuse": Refuse to connect
--   "in-only": Only allow income connection (Not auto connect to target)
--   "out-only": Only self connect to target (Not allow income connection)
--   "alias": Set target aliases
--
--for refuse, in-only and out-only:
--   id = "special-id", or
--   id = "match-pattern",
--
--   option = "special-address" or
--   option = "address-match-pattern"
--
--for alias:
--   id = "special-id",
--   option = "alias"

function _M.AddFilter( filter_name, id, option )
	if filter_name and id then
		local list = _M.filters[filter_name]
		if not list then
			list = {
				[id] = option
			}
			_M.filters[filter_name] = list
		else
			list[id] = option
		end
		return true
	else
		return false, "invalid"
	end
end

function _M.RemoveFilter( filter_name, id )
	if not filter_name then
		return false, "invalid"
	end

	local list = _M.filters[filter_name]
	if not list then
		return true
	end

	if id then
		if id == "*" then
			_M.filters[filter_name] = {}
		else
			list[id] = nil
		end
		return true
	else
		return false, "bad id"
	end
end

function _M.RemoveAllFilters()
	_M.filters = {}
	return true
end

return _M

