require "features"
local periphery = require "periphery"
local I2C = periphery.I2C

local _M = {
	validLangs = { "cn" },
	defaultLangs = "cn",
	hardwareVersion = "1.4.00",
	hardwareVersionCode = 0x14,
	hardwareVersionSwCode = 0x00,
	init = false
}

local CMD_ADDR_LIST = {0xf1, 0xf5}
local DEV_ADDR, I2C_DEVICE = 0xf4, "/dev/i2c-3"

function _M.Check()
	if _M.init then
		return
	end
	
	os.execute("/bin/sleep 5")

	local fpga_i2c = nil
	local r = pcall( function() fpga_i2c = I2C( I2C_DEVICE ) end )
	if not r or not fpga_i2c then
		print( "***ERROR: Could not open language checker because hardware reason!" )
	else
		_M.init = true
		local msg = { {CMD_ADDR_LIST[1]}, {0x00, flags = I2C.I2C_M_RD} }
		local is_ok = pcall(function() fpga_i2c:transfer( DEV_ADDR >> 1, msg ) end)	--0xf4 is the FPGA device address.
		if not is_ok then
			DEV_ADDR = 0xE4
			I2C_DEVICE = "/dev/i2c-0"
			CMD_ADDR_LIST = {0xa0}
			pcall( function() fpga_i2c = I2C( I2C_DEVICE ) end )
			_G.ENABLE_CONVERTER = false
			msg = { {CMD_ADDR_LIST[1]}, {0x00, flags = I2C.I2C_M_RD} }
			fpga_i2c:transfer( DEV_ADDR >> 1, msg )
		end
		if msg[2][1] >= 0x10 and msg[2][1] <= 0x99 then
			_M.hardwareVersionCode = msg[2][1]
			_M.hardwareVersion = "" .. (msg[2][1]>>4)
			_M.hardwareVersion = _M.hardwareVersion .. "."
			_M.hardwareVersion = _M.hardwareVersion .. (msg[2][1] & 0x0f)
		elseif msg[2][1] >= 0xc1 then
			_M.hardwareVersionCode = msg[2][1]
			_M.hardwareVersionSwCode = 0x20
			_M.hardwareVersion = "4.1." .. (msg[2][1]>>4)
			_G.AUDSEL_BY_FPGA = true
		end

		if CMD_ADDR_LIST[2] then
			msg = { {CMD_ADDR_LIST[2]}, {0x00, flags = I2C.I2C_M_RD} }
			fpga_i2c:transfer( DEV_ADDR >> 1, msg )	--0xf4 is the FPGA device address.
			if msg[2][1] >= 0x10 and msg[2][1] <= 0x99 then
				_M.hardwareVersionSwCode = msg[2][1]
				_M.hardwareVersion = _M.hardwareVersion .. "." .. (msg[2][1]>>4) .. (msg[2][1] & 0x0f)
			else
				_M.hardwareVersion = _M.hardwareVersion .. ".00"
			end
		end
		fpga_i2c:close()
		fpga_i2c = nil
		--[[
		local defaultLangs = 0x01
		local msg = { {0xf2}, {0x00, 0x00, 0x00, 0x00, flags = I2C.I2C_M_RD} }
		
		r = pcall( function() fpga_i2c:transfer( DEV_ADDR >> 1, msg ) end )
		if not r then
			print( "***ERROR: Check hardware language configuration" )
			return
		end
		
		_M.init = true
		
		local rslt = msg[2]
		print( "Language support:", string.format( "0x%02x", rslt[1] ) )
		if rslt[1] == 0x01 then
			_M.validLangs = { "cn" }
			_M.defaultLangs = "cn"
			fpga_i2c = nil
			return
		elseif rslt[1] == 0x02 then
			_M.validLangs = { "tw" }
			_M.defaultLangs = "tw"
			fpga_i2c = nil
			return
		elseif rslt[1] == 0x04 then
			_M.validLangs = { "en" }
			_M.defaultLangs = "en"
			fpga_i2c = nil
			return
		else
			_M.validLangs = { "cn", "en", "tw" }
			--To check default language
		end
		
		print( "Default language:", string.format( "0x%02x", rslt[3] ) )
		
		if rslt[3] == 0x01 then
			_M.defaultLangs = "cn"
		elseif rslt[3] == 0x02 then
			_M.defaultLangs = "tw"
		else
			_M.defaultLangs = "en"
		end
		fpga_i2c = nil
		]]
	end
end

return _M
