local _M = {}

function _M.GetTimeDateInfo()
	local file = io.popen( "/usr/bin/timedatectl status", "r" )
	if not file then
		return nil
	else
		local result = {}
		local line = file:read("l")
		while line do
			if line:match("Local time:") then
				result["local"] = line:match("Local time:%s*%w+%s*([0-9%-]+%s*[0-9%:]+)")
			elseif line:match("Universal time:") then
				result["utc"] = line:match("Universal time:%s*%w+%s*([0-9%-]+%s*[0-9%:]+)")
			elseif line:match("Time zone:") then
				result["zone"] = line:match("Time zone:%s*([^%s]+)")
				local zonename,offset = line:match( "%((%w+)[,%s]+([%+%-0-9]+)%)" )
				result["zone_name"] = zonename
				result["zone_offset"] = offset
			elseif line:match("NTP enabled:") then
				result["ntp_enabled"] = line:match("NTP enabled:%s*(%w+)")
			elseif line:match("NTP synchronized:") then
				result["ntp_sync"] = line:match("NTP synchronized:%s*(%w+)")
			end
			
			line = file:read("l")
		end
		file:close()
		return result
	end
end


function _M.SetNTP(en, addresses, noSave)
	if en and ( en == 1 or en == "yes" or en == "true" ) then
		if addresses then
			_M.SetNTPAddresses( addresses, noSave )
			os.execute( "/usr/bin/timedatectl set-ntp FALSE" )	--First, try disable it, then update addresses.
		end
		os.execute( "/usr/bin/timedatectl set-ntp TRUE" )
		if not noSave then
			if _G.cfg_timer then
				_G.cfg_timer.useNtp = true
				_G.cfg_timer:__TRIGGER(true)
			end
		end
	else
		os.execute( "/usr/bin/timedatectl set-ntp FALSE" )
		if not noSave then
			if _G.cfg_timer then
				_G.cfg_timer.useNtp = false
				_G.cfg_timer:__TRIGGER(true)
			end
		end
	end
end

--Only set addresses.
local lfs = require "lfs"
function _M.SetNTPAddresses( addresses, noSave )
	if noSave and not lfs.attributes("/data/configs/timectl.conf") then
		os.execute("/bin/cp /etc/systemd/timesyncd.conf /data/configs/timectl.conf")
	end
	if addresses then
		local file = io.open( "/etc/systemd/timesyncd.conf", "w" )
		if file then
			file:write( [[
[Time]
NTP=]] .. addresses .. [[

FallbackNTP=time1.google.com time2.google.com time3.google.com
RootDistanceMaxSec=5
PollIntervalMinSec=32
PollIntervalMaxSec=2048

]] )
			file:close()
		end
	end
	if not noSave then
		os.execute("/bin/cp /etc/systemd/timesyncd.conf /data/configs/timectl.conf")
	end
end

function _M.GetNTPAddress()
	local file = io.open( "/etc/systemd/timesyncd.conf", "r" )
	if file then
		local rslt = nil
		local line = file:read( "l" )
		while line do
			if not line:match( "^%s*#.*" ) then	--Comment
				rslt = line:match( "^%s*NTP%s*%=%s*(.+)" )
				if rslt then
					break
				end
			end
			line = file:read( "l" )
		end
		file:close()
		return rslt or ""
	else
		return nil
	end
end


function _M.SetTimezone( tz )
	if tz then
		local done,err = os.execute( "/usr/bin/timedatectl set-timezone \"" .. tz .. "\"" )
		return done,err
	end
	return false
end

local coaf = require "coaf.core"
function _M.SetTime( time )
	if time then
		coaf.ScheduleDelayedTask(function()
			os.execute( "/usr/bin/timedatectl set-time \"" .. time .. "\"" )
		end)
		if _G.cfg_timer then
			_G.cfg_timer.useNtp = false
			_G.cfg_timer:__TRIGGER(true)
		end
	end
end

return _M


