--The pelco-D PTZ protocol.

local function CheckSum(buf)
	local i
	local csum = 0
	for i=2,#buf-1 do
		csum = csum + buf[i]
	end
	return csum % 256
end

local function CalcSpeed(spd)
	if not spd then
		return 0x1f
	elseif spd < 0 or spd > 1 then
		return 0xff
	else
		return math.floor( spd*0x3f )
	end
end

local function PelcoD_stop(addr)
	local buf = { 0xff, (addr or 0), 0x00, 0x00, 0x00, 0x00, 0x00 }
	buf[7] = CheckSum(buf)
	return {buf}
end


local _M = {

name = "Pelco-D",

AckFilter = nil,
maxAddresses = 32,

supports = {
	["up"] = function(addr,spd)
		local buf = { 0xff, (addr or 0), 0x00, 0x08, 0x00, CalcSpeed(spd), 0x00 } 
		buf[7] = CheckSum(buf)
		return {buf}
	end,

	["down"] = function(addr,spd)
		local buf = { 0xff, (addr or 0), 0x00, 0x10, 0x00, CalcSpeed(spd), 0x00 } 
		buf[7] = CheckSum(buf)
		return {buf}
	end,
	
	["left"] = function(addr,spd)
		local buf = { 0xff, (addr or 0), 0x00, 0x04, CalcSpeed(spd), 0x00, 0x00 } 
		buf[7] = CheckSum(buf)
		return {buf}
	end,

	["right"] = function(addr,spd)
		local buf = { 0xff, (addr or 0), 0x00, 0x02, CalcSpeed(spd), 0x00, 0x00 } 
		buf[7] = CheckSum(buf)
		return {buf}
	end,

	["left-up"] = function(addr,pan_spd,tilt_spd)
		local buf = { 0xff, (addr or 0), 0x00, 0x04 + 0x08, CalcSpeed(pan_spd), CalcSpeed(tilt_spd), 0x00 } 
		buf[7] = CheckSum(buf)
		return {buf}
	end,

	["right-up"] = function(addr,pan_spd,tilt_spd)
		local buf = { 0xff, (addr or 0), 0x00, 0x02 + 0x08, CalcSpeed(pan_spd), CalcSpeed(tilt_spd), 0x00 } 
		buf[7] = CheckSum(buf)
		return {buf}
	end,

	["left-down"] = function(addr,pan_spd,tilt_spd)
		local buf = { 0xff, (addr or 0), 0x00, 0x04 + 0x10, CalcSpeed(pan_spd), CalcSpeed(tilt_spd), 0x00 } 
		buf[7] = CheckSum(buf)
		return {buf}
	end,

	["right-down"] = function(addr,pan_spd,tilt_spd)
		local buf = { 0xff, (addr or 0), 0x00, 0x02 + 0x10, CalcSpeed(pan_spd), CalcSpeed(tilt_spd), 0x00 } 
		buf[7] = CheckSum(buf)
		return {buf}
	end,

	["stop-move"] = PelcoD_stop,

	["zoom-in"] = function(addr,spd)
		local spd_buf = nil
		if spd then
			if spd >= 0 and spd < 1 then
				spd_buf = { 0xff, (addr or 0), 0x00, 0x25, 0x00, math.floor( spd*4 ), 0x00 }
			else
				spd_buf = { 0xff, (addr or 0), 0x00, 0x25, 0x00, 0x00, 0x00 }
			end
			spd_buf[7] = CheckSum(spd_buf)
		end

		local buf = { 0xff, (addr or 0), 0x00, 0x20, 0x00, 0x00, 0x00 } 
		buf[7] = CheckSum(buf)

		if spd_buf then
			return {spd_buf, buf}
		else
			return {buf}
		end
	end,

	["zoom-out"] = function(addr,spd)
		local spd_buf = nil
		if spd then
			if spd >= 0 and spd < 1 then
				spd_buf = { 0xff, (addr or 0), 0x00, 0x25, 0x00, math.floor( spd*4 ), 0x00 }
			else
				spd_buf = { 0xff, (addr or 0), 0x00, 0x25, 0x00, 0x00, 0x00 }
			end
			spd_buf[7] = CheckSum(spd_buf)
		end

		local buf = { 0xff, (addr or 0), 0x00, 0x40, 0x00, 0x00, 0x00 } 
		buf[7] = CheckSum(buf)

		if spd_buf then
			return {spd_buf, buf}
		else
			return {buf}
		end
	end,

	["stop-zoom"] = PelcoD_stop,

	["focus-near"] = function(addr,spd)
		local spd_buf = nil
		if spd then
			if spd >= 0 and spd < 1 then
				spd_buf = { 0xff, (addr or 0), 0x00, 0x27, 0x00, math.floor( spd*4 ), 0x00 }
			else
				spd_buf = { 0xff, (addr or 0), 0x00, 0x27, 0x00, 0x00, 0x00 }
			end
			spd_buf[7] = CheckSum(spd_buf)
		end

		local buf = { 0xff, (addr or 0), 0x00, 0x80, 0x00, 0x00, 0x00 } 
		buf[7] = CheckSum(buf)

		if spd_buf then
			return {spd_buf, buf}
		else
			return {buf}
		end
	end,

	["focus-far"] = function(addr,spd)
		local spd_buf = nil
		if spd then
			if spd >= 0 and spd < 1 then
				spd_buf = { 0xff, (addr or 0), 0x00, 0x27, 0x00, math.floor( spd*4 ), 0x00 }
			else
				spd_buf = { 0xff, (addr or 0), 0x00, 0x27, 0x00, 0x00, 0x00 }
			end
			spd_buf[7] = CheckSum(spd_buf)
		end

		local buf = { 0xff, (addr or 0), 0x01, 0x00, 0x00, 0x00, 0x00 } 
		buf[7] = CheckSum(buf)

		if spd_buf then
			return {spd_buf, buf}
		else
			return {buf}
		end
	end,

	["stop-focus"] = PelcoD_stop,

	["iris-up"] = function(addr,spd)
		local buf = { 0xff, (addr or 0), 0x04, 0x00, 0x00, 0x00, 0x00 } 
		buf[7] = CheckSum(buf)
		return {buf}
	end,

	["iris-down"] = function(addr,spd)
		local buf = { 0xff, (addr or 0), 0x02, 0x00, 0x00, 0x00, 0x00 } 
		buf[7] = CheckSum(buf)
		return {buf}
	end,

	["stop-iris"] = PelcoD_stop,
	["stop-all"] = PelcoD_stop,

	["save-preset"] = function(addr,id)
		id = id or 1
		if id <= 0 or id > 0x7f then
			id = 0x7f
		end

		local buf = { 0xff, (addr or 0), 0x00, 0x03, 0x00, id, 0x00 } 
		buf[7] = CheckSum(buf)
		return {buf}
	end,

	["remove-preset"] = function(addr,id)
		id = id or 1
		if id <= 0 or id > 0x7f then
			id = 0x7f
		end

		local buf = { 0xff, (addr or 0), 0x00, 0x05, 0x00, id, 0x00 } 
		buf[7] = CheckSum(buf)
		return {buf}
	end,

	["load-preset"] = function(addr,id)
		id = id or 1
		if id <= 0 or id > 0x7f then
			id = 0x7f
		end

		local buf = { 0xff, (addr or 0), 0x00, 0x07, 0x00, id, 0x00 } 
		buf[7] = CheckSum(buf)
		return {buf}
	end
}

}



return _M

